#ifndef _MMB_H_
#define _MMB_H_
//
//	13may97/hson	
//	- added asyncMsg flag
//
//	16may97/hson	
//	- changes made so that one copy of this file is usable for both mnti and
//   the drivers.
//
//    11jul97/bhsi
//     - added MSB (Mercury Stream Block) for mntRegisterAsyncStreams() function

// Mercury Message Block Control Header
//
// note: the world's a better place if we keep the bit fields
//       aligned to 8-bit bytes.
//
typedef struct {
  USHORT	cmdSize;					// command message size in bytes
  struct {
	 USHORT   matchSrcAddr:1;	// must be set always
    USHORT   matchDstAddr:1;
	 USHORT   matchXID:1;
	 USHORT   matchMsgType:1;

	 USHORT   msgSent:1;
	 USHORT   partialReplies:1;
	 USHORT   emptyMsg:1;		// no command msg to send
	 USHORT   streamOut:1;

	 USHORT   asyncMsg:1;		// async msg
	 USHORT   Reserved:7;
  } flags;
  
  USHORT	    replySize;
  UCHAR	    expectedReplyCount;
  UCHAR	    actualReplyCount;
  USHORT	    timeout;
  USHORT	    replyOffset;
} MMB_CONTROL, *PMMB_CONTROL;


// Mercury Message header
//
typedef  struct  {
  ULONG	flags:8;			// bit 0-2 : version id
								//       3 : normal/urgent (0/1)
								//     4-6 : msg null/parameter/extended (00/10/20)
								//       7 : always set for little-endian
								//     
  ULONG	transaction:24;
  ULONG	type;				// bit 17-22 : msg type
								//     23-24 : msg class
  ULONG	srcNode:16;		// source addr
  ULONG	srcBoard:8;
  ULONG	srcProcessor:8;
  ULONG	destNode:16;	// destination addr
  ULONG	destBoard:8;
  ULONG	destProcessor:8;
  ULONG	srcInstance:8;
  ULONG	srcComponent:8;
  ULONG	destInstance:8;
  ULONG	destComponent:8;
  ULONG	msgsize;	      // msg payload size *or* sram data block offset
} MERC_MSG, *PMERC_MSG, QMsg, *QMsgRef;		


// Mercury Message Block (MMB)
//
typedef struct {
  MMB_CONTROL mmbCtrl;
  MERC_MSG	  mercMsg;
  union {
    ULONG payload;
    ULONG replies;
  } mm;
} MMB, *PMMB, *LPMMB;

typedef struct {
  ULONG sequence;
  UCHAR bufFlags;
  UCHAR encoding;
  UCHAR pad1;
  UCHAR sysFlags;       // read-only		
  ULONG canTakeLimit;   // read-only
  ULONG initialCanTake; // read-only
  ULONG currentCanTake; // read-only
  ULONG requestedSize;  // read-only
  ULONG actualSize;     // read-only
} STRM_HDR, *PSTRM_HDR;

typedef struct {
   STRM_HDR    strmHdr;
   ULONG       readCompletionMask;
   USHORT      timeout;
   ULONG       xferLen;
   ULONG       xferDone;
} MSB, *PMSB, *LPMSB;

#define COMPLETE_ON_EOD   0x01   // used in readCompletionMask
#define COMPLETE_ON_EOT   0x02
#define COMPLETE_ON_EOF   0x04
#define COMPLETE_ON_USR1  0x08
#define COMPLETE_ON_USR2  0x10
#define COMPLETE_ON_USR3  0x20
#define COMPLETE_ON_USR4  0x40
#define COMPLETE_ON_USR5  0x80
#define STREAM_CLOSED     0x01	// read-only system flag
#define STREAM_BROKEN     0x02	// read-only system flag
#define STREAM_OVERRUN    0x04	// read-only system flag

#endif
