/***********************************************************************
 *        FILE: msilib.cpp
 * DESCRIPTION: Cross compatibility Library file for DIALOGIC Windows NT
 *
 *   Copyright (c) 1991-1996 Dialogic Corp. All Rights Reserved
 *
 *   THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Dialogic Corp.
 *   The copyright notice above does not evidence any actual or
 *   intended publication of such source code.
 **********************************************************************/

#ifndef __CROSS_COMPAT_LIB__
#define __CROSS_COMPAT_LIB__
#endif

#ifndef __CROSS_COMPAT_LIB__
#define __CROSS_COMPAT_LIB__
#endif

#include <windows.h>
#include <errno.h>
#include "srllib.h"
#include "dtilib.h"
#include "msilib.h"

//
// function prototypes
//
static int  ms_int_notimp(int ddd);
static char *ms_null_notimp(int ddd);

//
// typedefs and defines
//
typedef void (*func_ptr)(void);
struct func_table {
	char *fname;
	func_ptr fptr;
};

#define ms_setbrdparm_ID           0
#define ms_getbrdparm_ID           1
#define ATMS_STATINFO_ID           2
#define ATMS_FEMINFO_ID            3
#define ms_genziptone_ID           4
#define ms_estconf_ID              5
#define ms_delconf_ID              6
#define ms_addtoconf_ID            7
#define ms_remfromconf_ID          8
#define ms_setcb_ID                9
#define ms_getcb_ID                10
#define ms_getcde_ID               11
#define ms_setcde_ID               12
#define ms_monconf_ID              13
#define ms_unmonconf_ID            14
#define ms_genring_ID              15 
#define ms_setvol_ID               16
#define ms_getevtmsk_ID            17
#define ms_stopfn_ID               18
#define ATMS_NUMCHAN_ID            19
#define ms_open_ID                 20
#define ms_close_ID                21     
#define ms_setevtmsk_ID            22
#define ms_listen_ID               23
#define ms_unlisten_ID             24
#define ms_getxmitslot_ID          25
#define ms_tstcom_ID               26
#define ms_tstdat_ID               27
#define ms_getctinfo_ID            28
#define ATMS_DNLDVER_ID            29
#define ATMS_TSSGBIT_ID            30
#define ms_chgxtder_ID             31
#define ms_delxtdcon_ID            32
#define ms_estxtdcon_ID            33
#define ms_dsprescount_ID          34
#define ms_getcnflist_ID           35
#define ms_getevt_ID               36
#define ms_setstparm_ID            37
#define ms_genringex_ID            38
#define ms_genringCallerID_ID      39
#define ms_SetMsgWaitInd_ID        40


typedef int    (*ptr_ms_setbrdparm)(int, unsigned char, void *);/* Set MSI Board parameters */
typedef int    (*ptr_ms_getbrdparm)(int, unsigned char, void *);/* Get MSI Board parameters */
typedef int    (*ptr_ATMS_STATINFO)(int, char *);               /* Get station info */
typedef char * (*ptr_ATMS_FEMINFO)(int);                      /* Get channel info */
typedef int    (*ptr_ms_genziptone)( int);                      /* Generate ziptone */
typedef int    (*ptr_ms_estconf)(int, MS_CDT *,int, int, int *);/* Estab a conference */
typedef int    (*ptr_ms_delconf)(int, int);                     /* Delete conference */
typedef int    (*ptr_ms_addtoconf)(int,int, MS_CDT *);          /* Add a party to conf*/
typedef int    (*ptr_ms_remfromconf)(int,int, MS_CDT *);        /* Remove a party from conf*/
typedef int    (*ptr_ms_setcb)(int,int, MS_CDT *, void *);      /* Set a control block */
typedef int    (*ptr_ms_getcb)(int,int, MS_CDT *, void *);      /* Get a control block */
typedef int    (*ptr_ms_getcde)(int, int, MS_CDT *);            /* Get conf. descr. elem */
typedef int    (*ptr_ms_setcde)(int,int,MS_CDT *);              /* Set conf. descr. elem */
typedef int    (*ptr_ms_monconf)(int,int, long *);              /* Monitor conference */
typedef int    (*ptr_ms_unmonconf)(int, int);                   /* Unmonitor conference */
typedef int    (*ptr_ms_genring)(int,unsigned short, unsigned short); /* Generate ringing */
typedef int    (*ptr_ms_setvol)(int,int,int);                   /* Set volume */
typedef int    (*ptr_ms_getevtmsk)(int, int , unsigned short *);/* Get event mask */
typedef int    (*ptr_ms_stopfn)(int, int);                      /* Stop multitasking fn */
typedef int    (*ptr_ATMS_NUMCHAN)(int);                        /* Get number of channels */
typedef int    (*ptr_ms_open)(char *, int);                     /* Open MSI/SC device */
typedef int    (*ptr_ms_close)(int);                            /* Close MSI/SC device */      
typedef int    (*ptr_ms_setevtmsk)(int, int, unsigned short, int); /* Set event mask */
typedef int    (*ptr_ms_listen)(int, SC_TSINFO *);              /* Listen to station */
typedef int    (*ptr_ms_unlisten)(int);                         /* Unlisten to station */
typedef int    (*ptr_ms_getxmitslot)(int, SC_TSINFO *);         /* Get xmit ts */
typedef int    (*ptr_ms_tstcom)(int, unsigned int);             /* Test communication */
typedef int    (*ptr_ms_tstdat)(int, unsigned int);             /* Test data */
typedef int    (*ptr_ms_getctinfo)(int, CT_DEVINFO *);          /* Get device info */
typedef long   (*ptr_ATMS_DNLDVER)(int);                        /* Get download version */
typedef long   (*ptr_ATMS_TSSGBIT)(int);                        /* Get hook state */
typedef int    (*ptr_ms_chgxtder)(int,int,MS_CDT *);            /* Change connection extender */
typedef int    (*ptr_ms_delxtdcon)(int,int);                    /* Delete Extended connection */
typedef int    (*ptr_ms_estxtdcon)(int,MS_CDT *,int *);         /* Establish extended connection*/
typedef int    (*ptr_ms_dsprescount)(int, int *);               /* Get DSP resource count */
typedef int    (*ptr_ms_getcnflist)(int, int, int *, MS_CDT *); /* Get config list */
typedef int    (*ptr_ms_getevt)(int, EV_EBLK *, int);           /* Get MSI event */
typedef int    (*ptr_ms_setstparm)(int, unsigned char, void *); /* Set MSI Station parameters */
typedef int    (*ptr_ms_genringex)(int,unsigned short, unsigned short, unsigned short); 
								/* Generate Distinctive ringing */
typedef int    (*ptr_ms_genringCallerID)(int,unsigned short, unsigned short, unsigned short,char*,void*); 
								/* Generate distinctive ringing with CallerID*/                                                                                /* Generate ringing */
typedef int    (*ptr_ms_SetMsgWaitInd)(int,unsigned short,void*,void*); 
								/* Set Message Waiting Indicator*/                                                                                /* Generate ringing */


static struct func_table MsiFunc[] = {
{ "ms_setbrdparm",              (func_ptr)ms_int_notimp },
{ "ms_getbrdparm",              (func_ptr)ms_int_notimp }, 
{ "ATMS_STATINFO",              (func_ptr)ms_int_notimp },
{ "ATMS_FEMINFO",               (func_ptr)ms_null_notimp },  
{ "ms_genziptone",              (func_ptr)ms_int_notimp }, 
{ "ms_estconf",                 (func_ptr)ms_int_notimp },    
{ "ms_delconf",                 (func_ptr)ms_int_notimp },    
{ "ms_addtoconf",               (func_ptr)ms_int_notimp },  
{ "ms_remfromconf",             (func_ptr)ms_int_notimp }, 
{ "ms_setcb",                   (func_ptr)ms_int_notimp },       
{ "ms_getcb",                   (func_ptr)ms_int_notimp },      
{ "ms_getcde",                  (func_ptr)ms_int_notimp },     
{ "ms_setcde",                  (func_ptr)ms_int_notimp },     
{ "ms_monconf",                 (func_ptr)ms_int_notimp },    
{ "ms_unmonconf",               (func_ptr)ms_int_notimp }, 
{ "ms_genring",                 (func_ptr)ms_int_notimp },    
{ "ms_setvol",                  (func_ptr)ms_int_notimp },    
{ "ms_getevtmsk",               (func_ptr)ms_int_notimp }, 
{ "ms_stopfn",                  (func_ptr)ms_int_notimp },    
{ "ATMS_NUMCHAN",               (func_ptr)ms_int_notimp }, 
{ "ms_open",                    (func_ptr)ms_int_notimp },      
{ "ms_close",                   (func_ptr)ms_int_notimp },          
{ "ms_setevtmsk",               (func_ptr)ms_int_notimp }, 
{ "ms_listen",                  (func_ptr)ms_int_notimp },    
{ "ms_unlisten",                (func_ptr)ms_int_notimp },  
{ "ms_getxmitslot",             (func_ptr)ms_int_notimp }, 
{ "ms_tstcom",                  (func_ptr)ms_int_notimp },      
{ "ms_tstdat",                  (func_ptr)ms_int_notimp },      
{ "ms_getctinfo",               (func_ptr)ms_int_notimp },   
{ "ATMS_DNLDVER",               (func_ptr)ms_int_notimp },   
{ "ATMS_TSSGBIT",               (func_ptr)ms_int_notimp },   
{ "ms_chgxtder",                (func_ptr)ms_int_notimp },    
{ "ms_delxtdcon",               (func_ptr)ms_int_notimp },   
{ "ms_estxtdcon",               (func_ptr)ms_int_notimp },   
{ "ms_dsprescount",             (func_ptr)ms_int_notimp }, 
{ "ms_getcnflist",              (func_ptr)ms_int_notimp },  
{ "ms_getevt",                  (func_ptr)ms_int_notimp },
{ "ms_setstparm",               (func_ptr)ms_int_notimp },
{ "ms_genringex",               (func_ptr)ms_int_notimp },    
{ "ms_genringCallerID",         (func_ptr)ms_int_notimp },    
{ "ms_SetMsgWaitInd",           (func_ptr)ms_int_notimp }    
};

int ms_libinit(HINSTANCE LibHandle)
{
	struct func_table *msip = MsiFunc;
	func_ptr ProcAddress;
	DWORD NumberofFunctions;

	NumberofFunctions = sizeof(MsiFunc)/sizeof(MsiFunc[0]);

	//
	// The library has been loaded previously. Set up each entry to a default
	// not implemented function and then overwrite the true address if
	// the function is present in the DLL.
	//
	while (NumberofFunctions --) {
		ProcAddress = (func_ptr)GetProcAddress(LibHandle, msip->fname);
		if (ProcAddress) {
			msip->fptr = ProcAddress;
		}
				msip++;
	}

	return 0;
}

char *ATMS_FEMINFO(int devh)                 
{
	ptr_ATMS_FEMINFO p;

	p = (ptr_ATMS_FEMINFO)*MsiFunc[ATMS_FEMINFO_ID].fptr;
	return (*p)(devh);
}

int ms_setbrdparm(int devh, unsigned char param, void *valuep)
{
	ptr_ms_setbrdparm p;

	p = (ptr_ms_setbrdparm)*MsiFunc[ms_setbrdparm_ID].fptr;
	return (*p)(devh,param,valuep);
}

long ATMS_DNLDVER(int devh)
{
	ptr_ATMS_DNLDVER p;

	p = (ptr_ATMS_DNLDVER)*MsiFunc[ATMS_DNLDVER_ID].fptr;
	return (*p)(devh);
}

long ATMS_TSSGBIT(int devh)
{
	ptr_ATMS_TSSGBIT p;

	p = (ptr_ATMS_TSSGBIT)*MsiFunc[ATMS_TSSGBIT_ID].fptr;
	return (*p)(devh);
}

int ms_getbrdparm(int devh, unsigned char param, void *valuep)
{
	ptr_ms_getbrdparm p;

	p = (ptr_ms_getbrdparm)*MsiFunc[ms_getbrdparm_ID].fptr;
	return (*p)(devh,param,valuep);
}

int ATMS_STATINFO(int devh, char *statinfop)
{
	ptr_ATMS_STATINFO p;

	p = (ptr_ATMS_STATINFO)*MsiFunc[ATMS_STATINFO_ID].fptr;
	return (*p)(devh,statinfop);
}

int ms_genziptone(int devh)
{
	ptr_ms_genziptone p;

	p = (ptr_ms_genziptone)*MsiFunc[ms_genziptone_ID].fptr;
	return (*p)(devh);
}

int ms_estconf(int devh, MS_CDT *cdtp,int numpty, int confattr, int *confID)
{
	ptr_ms_estconf p;

	p = (ptr_ms_estconf)*MsiFunc[ms_estconf_ID].fptr;
	return (*p)(devh,cdtp, numpty, confattr, confID);
}

int ms_delconf(int devh, int confID)
{
	ptr_ms_delconf p;

	p = (ptr_ms_delconf)*MsiFunc[ms_delconf_ID].fptr;
	return (*p)(devh,confID);
}

int ms_addtoconf(int devh,int confID, MS_CDT *cdtp)
{
	ptr_ms_addtoconf p;

	p = (ptr_ms_addtoconf)*MsiFunc[ms_addtoconf_ID].fptr;
	return (*p)(devh, confID, cdtp);
}

int ms_remfromconf(int devh,int confID, MS_CDT *cdtp)
{
	ptr_ms_remfromconf p;

	p = (ptr_ms_remfromconf)*MsiFunc[ms_remfromconf_ID].fptr;
	return (*p)(devh, confID, cdtp);
}

int ms_setcb(int devh,int param, MS_CDT *cdtp, void * valuep)
{
	ptr_ms_setcb p;

	p = (ptr_ms_setcb)*MsiFunc[ms_setcb_ID].fptr;
	return (*p)(devh,param,cdtp, valuep);
}

int ms_getcb(int devh,int param, MS_CDT *cdtp, void * valuep)
{
	ptr_ms_getcb p;

	p = (ptr_ms_getcb)*MsiFunc[ms_getcb_ID].fptr;
	return (*p)(devh,param,cdtp, valuep);
}

int ms_getcde(int devh, int confID, MS_CDT *cdtp)
{
	ptr_ms_getcde p;

	p = (ptr_ms_getcde)*MsiFunc[ms_getcde_ID].fptr;
	return (*p)(devh,confID, cdtp);
}

int ms_setcde(int devh,int confID,MS_CDT *cdtp )
{
	ptr_ms_setcde p;

	p = (ptr_ms_setcde)*MsiFunc[ms_setcde_ID].fptr;
	return (*p)(devh, confID, cdtp);
}

int ms_monconf(int devh,int confID, long *lts)
{
	ptr_ms_monconf p;

	p = (ptr_ms_monconf)*MsiFunc[ms_monconf_ID].fptr;
	return (*p)(devh, confID, lts);
}

int ms_unmonconf(int devh, int confID)
{
	ptr_ms_unmonconf p;

	p = (ptr_ms_unmonconf)*MsiFunc[ms_unmonconf_ID].fptr;
	return (*p)(devh, confID);
}

int ms_genring(int devh,unsigned short len, unsigned short mode) 
{
	ptr_ms_genring p;

	p = (ptr_ms_genring)*MsiFunc[ms_genring_ID].fptr;
	return (*p)(devh, len, mode);
}

int ms_setvol(int devh,int type,int steps)
{
	ptr_ms_setvol p;

	p = (ptr_ms_setvol)*MsiFunc[ms_setvol_ID].fptr;
	return (*p)(devh, type, steps);
}

int ms_getevtmsk(int devh, int event, unsigned short *bitmaskp)
{
	ptr_ms_getevtmsk p;

	p = (ptr_ms_getevtmsk)*MsiFunc[ms_getevtmsk_ID].fptr;
	return (*p)(devh, event, bitmaskp);
}

int ms_stopfn(int devh, int fctid)
{
	ptr_ms_stopfn p;

	p = (ptr_ms_stopfn)*MsiFunc[ms_stopfn_ID].fptr;
	return (*p)(devh, fctid);
}

int ATMS_NUMCHAN(int devh)
{
	ptr_ATMS_NUMCHAN p;

	p = (ptr_ATMS_NUMCHAN)*MsiFunc[ATMS_NUMCHAN_ID].fptr;
	return (*p)(devh);
}

int ms_open(char *name, int flag)
{
	ptr_ms_open p;

	p = (ptr_ms_open)*MsiFunc[ms_open_ID].fptr;
	if (p == (ptr_ms_open)ms_int_notimp) {
		errno = EINVAL;
		return -1;
	}
		
	return (*p)(name, flag);
}

int ms_close(int devh)      
{
	ptr_ms_close p;

	p = (ptr_ms_close)*MsiFunc[ms_close_ID].fptr;
	return (*p)(devh);
}

int ms_setevtmsk(int devh, int event, unsigned short bitmask, int action)
{
	ptr_ms_setevtmsk p;

	p = (ptr_ms_setevtmsk)*MsiFunc[ms_setevtmsk_ID].fptr;
	return (*p)(devh, event, bitmask, action);
}

int ms_listen(int devh, SC_TSINFO *tsinfop)
{
	ptr_ms_listen p;

	p = (ptr_ms_listen)*MsiFunc[ms_listen_ID].fptr;
	return (*p)(devh,tsinfop);
}

int ms_unlisten(int devh)
{
	ptr_ms_unlisten p;

	p = (ptr_ms_unlisten)*MsiFunc[ms_unlisten_ID].fptr;
	return (*p)(devh);
}

int ms_getxmitslot(int devh, SC_TSINFO *tsinfop)
{
	ptr_ms_getxmitslot p;

	p = (ptr_ms_getxmitslot)*MsiFunc[ms_getxmitslot_ID].fptr;
	return (*p)(devh, tsinfop);
}

int ms_tstcom(int devh, unsigned int tmo)
{
	ptr_ms_tstcom p;

	p = (ptr_ms_tstcom)*MsiFunc[ms_tstcom_ID].fptr;
	return (*p)(devh, tmo);
}

int ms_tstdat(int devh, unsigned int tmo)
{
	ptr_ms_tstdat p;

	p = (ptr_ms_tstdat)*MsiFunc[ms_tstdat_ID].fptr;
	return (*p)(devh, tmo);
}

int ms_getctinfo(int devh, CT_DEVINFO *devinfop)
{
	ptr_ms_getctinfo p;

	p = (ptr_ms_getctinfo)*MsiFunc[ms_getctinfo_ID].fptr;
	return (*p)(devh, devinfop);
}

int ms_chgxtder(int devh,int xid,MS_CDT * cdtp) 
{
	ptr_ms_chgxtder p;

	p = (ptr_ms_chgxtder)*MsiFunc[ms_chgxtder_ID].fptr;
	return (*p)(devh, xid, cdtp);
}

int ms_delxtdcon(int devh,int xid)         
{
	ptr_ms_delxtdcon p;

	p = (ptr_ms_delxtdcon)*MsiFunc[ms_delxtdcon_ID].fptr;
	return (*p)(devh, xid);
}

int ms_estxtdcon(int devh,MS_CDT *cdtp,int *xid)
{
	ptr_ms_estxtdcon p;

	p = (ptr_ms_estxtdcon)*MsiFunc[ms_estxtdcon_ID].fptr;
	return (*p)(devh, cdtp, xid);
}

int ms_dsprescount(int devh, int *cntp)
{
	ptr_ms_dsprescount p;

	p = (ptr_ms_dsprescount)*MsiFunc[ms_dsprescount_ID].fptr;
	return (*p)(devh, cntp);
}

int ms_getcnflist(int devh, int confID, int * numpty, MS_CDT * cdtp)
{
	ptr_ms_getcnflist p;

	p = (ptr_ms_getcnflist)*MsiFunc[ms_getcnflist_ID].fptr;
	return (*p)(devh, confID, numpty, cdtp);
}

int ms_getevt(int devh, EV_EBLK *eblkp, int tmo)
{
	ptr_ms_getevt p;

	p = (ptr_ms_getevt)*MsiFunc[ms_getevt_ID].fptr;
	return (*p)(devh, eblkp, tmo);
}

int ms_setstparm(int devh, unsigned char param, void *valuep)
{
	ptr_ms_setstparm p;

	p = (ptr_ms_setstparm)*MsiFunc[ms_setstparm_ID].fptr;
	return (*p)(devh,param,valuep);
}

int ms_genringex(int devh,unsigned short len, unsigned short mode, unsigned short cadid) 
{
	ptr_ms_genringex p;

	p = (ptr_ms_genringex)*MsiFunc[ms_genringex_ID].fptr;
	return (*p)(devh, len, mode, cadid);
}

int ms_genringCallerID(int devh,unsigned short len, unsigned short mode, unsigned short cadid, char* OriginationAddress,void* RFU) 
{
	ptr_ms_genringCallerID p;

	p = (ptr_ms_genringCallerID)*MsiFunc[ms_genringCallerID_ID].fptr;
	return (*p)(devh, len, mode, cadid,OriginationAddress,RFU);
}

int ms_SetMsgWaitInd(int devh,unsigned short IndicatorState,void* RFU1, void* RFU2) 
{
	ptr_ms_SetMsgWaitInd p;

	p = (ptr_ms_SetMsgWaitInd)*MsiFunc[ms_SetMsgWaitInd_ID].fptr;
	return (*p)(devh, IndicatorState, RFU1, RFU2);
}

int ms_int_notimp(int devh)
{
	dt_set_lasterr(devh, EDT_INVMSG);
	return -1;
}

char *ms_null_notimp(int devh)
{
	dt_set_lasterr(devh,EDT_INVMSG);
	return (char *)NULL;
}
