/***************************************************************************
 *		Copyright (c) 1996 Dialogic Corp.
 *		All Rights Reserved.
 *
 *		THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Dialogic Corp.
 *		The copyright notice above does not evidence any actual or
 *		intended publication of such source code.
 ***************************************************************************/

/****************************************************************************
 * FILE:         qstream.h
 * ORG:          Dialogic Corporation
 * ORIG-DATE:    28-Aug-96
 * LAST-MOD:     03-Feb-97
 * PURPOSE:
 *    This file contains the defines and structure for stream operation.
 *
 ***************************************************************************/

#ifndef __QSTREAM_H__
#define __QSTREAM_H__ 

// Define flag if compiling MNTI host library.
// Undefine flag if compiling applications.
#ifndef __MNTILIB_
#define MNTAPI __declspec(dllimport)
#else
#define MNTAPI __declspec(dllexport)
#endif

// Streams specific typedefs 
typedef void	*QBufRef;

// Structure for function mntSetStreamHeader
typedef struct {
   STRM_HDR StrmHdr;
   ULONG ReadCompletion;
} STRM_IOCBLK, *PSTRM_IOCBLK;


// Define the maximum number of stream groups for a board
#define  MNT_STREAM_MAX_NUM_GROUPS	    32
#define  MNT_DEFAULT_DATA_BLOCK_SIZE  4032
#define  QAGENT_COMPONENT                0
#define  QAGENT_PROCESSOR                1
#define  QAGENT_INSTANCE                12


// Structures to support mntGetStreamInfo
typedef struct {
	UInt32	GroupID;
	UInt32	NumStreams;
	UInt32	StreamSize;
} STRM_GROUP_CFG;

typedef struct {
	int				 NumStrmGroups;
	int				 DataBlockSize;
	STRM_GROUP_CFG  StrmGroups[MNT_STREAM_MAX_NUM_GROUPS];
}STRM_INFO, *PSTRM_INFO;


// Define ModeFlags for function mntAttachMercStream
#define MNT_STREAM_FLAG_READ			0x0001
#define MNT_STREAM_FLAG_WRITE			0x0002
#define MNT_STREAM_FLAG_NO_FLUSH		0x0004
#define MNT_STREAM_FLAG_FLUSH			0x0008
#define MNT_STREAM_FLAG_IGNORE_HEADER	0x0010
#define MNT_STREAM_FLAG_PERSISTENT	0x0020

#define MNT_STREAMSIZE_NORMAL       16224    //4056 * 4

// Define MCD flags for function mntAttachMercStream
//define MCD_STREAM_FLAG_READ			   0x0000  (bit 0)
#define MCD_STREAM_FLAG_WRITE			   0x0001
//define MCD_STREAM_FLAG_NO_FLUSH		0x0000  (bit 2)
#define MCD_STREAM_FLAG_FLUSH			   0x0004
#define MCD_STREAM_FLAG_PERSISTENT	   0x0008

#define MCD_STREAM_FLAG_IGNORE_HEADER  0x0010

// Flags used in bufFlags of STRM_HDR for function mntSetStreamHeader
#define MNT_EOD   0x01   
#define MNT_EOT   0x02
#define MNT_EOF   0x04
#define MNT_USR1  0x08
#define MNT_USR2  0x10
#define MNT_USR3  0x20
#define MNT_USR4  0x40
#define MNT_USR5  0x80

#define MNT_EOS   0x80

#ifdef __cplusplus
extern "C" {
#endif

// Function prototypes for stream I/O
MNTAPI BOOL WINAPI mntAttachMercStream(HANDLE	hDevice,
                                ULONG nBoardNumber,
                                USHORT nModeFlags,
                                PULONG	lpMercStreamID,
                                PULONG lpStreamSize,
                                USHORT nTimeout, 
                                LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntDetachMercStream(HANDLE	hDevice, 
                                USHORT nTimeout,
                                LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntTerminateStream(HANDLE	hDevice,
                                ULONG nBoardNumber,
                                USHORT nModeFlags,
                                ULONG	nMercStreamID,
                                USHORT nTimeout, 
                                LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntGetMercStreamID(HANDLE hDevice, 
                               PULONG lpMercStreamID,
                               PULONG lpBoardNumber);
MNTAPI BOOL WINAPI mntSetStreamHeader(HANDLE hDevice,
                               PSTRM_HDR lpHeader,
                               ULONG ReadCompletionMask);
MNTAPI BOOL WINAPI mntGetStreamHeader(HANDLE hDevice, PSTRM_HDR lpHeader);
MNTAPI BOOL WINAPI mntSetStreamIOTimeout(HANDLE hDevice, USHORT nTimeout);
MNTAPI BOOL WINAPI mntCompleteStreamIo(HANDLE hDevice);
MNTAPI BOOL WINAPI mntGetStreamInfo(ULONG nBoardNumber, PSTRM_INFO lpStrmInfos);
MNTAPI BOOL WINAPI mntCheckStreamOrphans(HANDLE hDevice, PULONG	lpOrphanBytes);
MNTAPI BOOL WINAPI mntRegisterAsyncStreams(HANDLE hDevice, 
		                              ULONG nCount,
                                    HANDLE *lpEvents,
                                    PVOID *lpBuffers,
                                    LPMSB *lpMSBs);

#ifdef __cplusplus
}
#endif

#endif