/***********************************************************************
 *        FILE: srllib.h
 * DESCRIPTION: Header File for DIALOGIC Standard Runtime Windows
 *              NT/95 library
 *
 *   Copyright (c) Dialogic Corp. All Rights Reserved
 *
 *   THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Dialogic Corp.
 *   The copyright notice above does not evidence any actual or
 *   intended publication of such source code.
 ********************************************************************/


#ifndef __SRLLIB_H__
#define __SRLLIB_H__


#include <windows.h>


/*
 * Included header files
 */
#include "srl_eblk.h"
#include "srltpt.h"


/* Define NULL if it hasn't been defined already */
#ifndef NULL
#define NULL        0L
#endif

/**
 ** Defines used for standard attributes 
 **/
/*
 * Undefined attribute or Failed to get attribute
 */
#define AT_FAILURE   -1
#define AT_FAILUREP  NULL

#define EDV_BADDESC  -1
#define EDV_NOERROR  0

/* Open Flags for devices */
#define OF_DEFAULT   0x00  /* Open device without automatic notification */
#define O_NOTIFY     0x01  /* Open device with automatic notification */


/*
 * General Defines
 */
#define EV_ANYEVT          0xFFFFFFFF  /* Matches Any Event Type */
#define EV_ANYDEV          -1          /* Matches Any Device */
#define SR_DFLT_DATASIZE   48          /* Max size for SRl's statically allocated
                                          event data area */

/* 
 * EVENT MANAGEMENT
 */
#define SR_INTERPOLLID  0x01  /* Parameter id for inter-poll delay */
#define SR_MODEID       0x02  /* Set SRL to run in SIGNAL/POLL mode */
#define SR_DATASZID     0x03  /* Parameter id for getting/setting SRL's
                                 preallocated event data memory size */
#define SR_QSIZEID      0X04  /* Maximum size of SRL's internal event queue */
#define SR_MODELTYPE    0x05  /* Set SRL model type (for NT only) */
#define SR_USERCONTEXT  0x06  /* Allows user to set per device context */
#define SR_WIN32INFO	   0x07  /* Set Win32 Notification mechanism */     
#define SR_STATISTICS   0x08  /* Set statistics monitoring */
#define SR_PARMHIPRIMODE 0x09 /* High Priority handler mode */

#define SR_POLLMODE     0     /* Run SRL in polling mode */
#define SR_SIGMODE      1     /* Run SRL in signalling/interrupt mode */
#define SRL_DEVICE      0     /* The SRL device */
#define SR_TMOUTEVT     0     /* Timeout event - occurs on the SRL DEVICE */

#define SR_STASYNC      0     /* Single threaded async model */
#define SR_MTASYNC      1     /* Multithreaded asynchronous model */
#define SR_MTSYNC       2     /* Multithreaded synchronous model */

#define SR_NOTIFY_ON    0     /* Turn on message notification */
#define SR_NOTIFY_OFF   1     /* Turn off message notification */

#define SR_HIPRIDEFAULT 0        
#define SR_HIPRISYNC    1


/* SRL errors */
#define ESR_NOERR       0     /* No SRL errors */
#define ESR_SCAN        1     /* SRL scanning function returned an error */
#define ESR_PARMID      2     /* Unknown parameter id */
#define ESR_TMOUT       3     /* Returned by ATDV_LASTERR(SRL_DEVICE) when an
                                 SRL function times out */
#define ESR_SYS         4     /* System error - consult errno */
#define ESR_DATASZ      5     /* Invalid size for default event data memory */
#define ESR_QSIZE       6     /* Illegal event queue size */
#define ESR_NOHDLR      7     /* No such handler */
#define ESR_MODE        8     /* Illegal mode for this operation */
#define ESR_NOTIMP      9     /* Function not implemented */

#define SR_TMOUT        -1    /* Returned by event scanning functions
                                 (e.g. sr_waitevt()) when they time out */

/*
 * Defines for DLL modes (to be passed to the xx_libinit functions)
 */
#define DLGC_ST         0     /* Single threaded operation */
#define DLGC_MT         1     /* Multithreaded operation */

/*
 * SRL Win/32 Synchronization Structure (use sr_setparm to set)
 */
typedef struct srlwin32info_tag {
   DWORD		dwTotalSize;	/* Structure size */
   HANDLE		ObjectHandle;	/* Object Handle */
   DWORD		dwHandleType;   /* Handle type */
   DWORD		UserKey;        /* User supplied key */
   LPOVERLAPPED	lpOverlapped;   /* pointer to an overlapped */
} SRLWIN32INFO, *LPSRLWIN32INFO;

/*
 * Flags for dwHandleType
 */
#define  SR_IOCOMPLETIONPORT	1	/* I/O Completion Port */
#define  SR_RESETEVENT			2	/* Reset Event */
#define  SR_WINDOWHANDLE        3   /* Window Handle */

#if defined(__cplusplus)
extern "C" {
#endif

/* SRL standard attribute functions */
extern char * __cdecl ATDV_ERRMSGP(long ddd);
extern long __cdecl   ATDV_IOPORT(long ddd);
extern long __cdecl   ATDV_IRQNUM(long ddd);
extern long __cdecl   ATDV_LASTERR(long ddd);
extern char * __cdecl ATDV_NAMEP(long ddd);
extern long __cdecl   ATDV_SUBDEVS(long ddd);

/* Wait for next event */
extern long __cdecl sr_waitevt(long tmout);
extern long __cdecl sr_waitevtEx(long *handlep, int count, long tmout, long *evthp);

/* Enable an event handler */
extern long __cdecl sr_enbhdlr(long ddd, unsigned long event_type,
                       long (*hdlr)(unsigned long parm));

/* Disable an event handler */
extern long __cdecl sr_dishdlr(long ddd, unsigned long event_type,
                       long (*hdlr)(unsigned long parm));

extern long __cdecl sr_getboardcnt(char *brdname, int *brdcnt);

#if (defined(__BORLANDC__) || defined(DLGC_CLIB))
extern long __cdecl sr_getevtdev(unsigned long evt_handle);
extern long __cdecl sr_getevttype(unsigned long evt_handle);
extern long __cdecl sr_getevtlen(unsigned long evt_handle);
extern void * __cdecl sr_getevtdatap(unsigned long evt_handle);
#elif defined(__cplusplus)
extern long __cdecl sr_getevtdev(...);         /* Get the ddd of the current event */
extern long __cdecl sr_getevttype(...);        /* Get the type of the current event */
extern long __cdecl sr_getevtlen(...);         /* Get the data length for crnt evt */
extern void * __cdecl sr_getevtdatap(...);      /* Get the data pointer for crnt evt */
#else
extern long __cdecl sr_getevtdev();
extern long __cdecl sr_getevttype();
extern long __cdecl sr_getevtlen();
extern void * __cdecl sr_getevtdatap();
#endif

extern long __cdecl sr_hold(void);             /* Hold off events in SIGNAL mode */
extern long __cdecl sr_release(void);          /* Release events in SIGNAL mode */

/* Get the value of an SRL paramater */
extern long __cdecl sr_getparm(long ddd, long parmid, void* valuep);

/* Set the value of an SRL paramater */
extern long __cdecl sr_setparm(long ddd, long parmid, void* valuep);

extern void __cdecl sr_NotifyEvent(HWND handle, unsigned int message, unsigned int flag);

extern long __cdecl sr_libinit(unsigned short flag);
extern long __cdecl sr_GetVersion(char *libnamep, LPDWORD fileverp, LPDWORD prodverp);
extern long __cdecl sr_GetDllVersion(LPDWORD fileverp, LPDWORD prodverp);
extern void __cdecl sr_cleanup(void);
extern long __cdecl sr_putevt(long dev, unsigned long evt, long len, void  *datap, long err);

/* terminate the extern "C" for c plus plus */
#if defined(__cplusplus)
}
#endif


#endif

