//
//  Copyright  1999-2003 by Ai-Logix, Inc.
//  All rights reserved.
//
//  NTiAPI.h
//      SmartTAP API declaration file
//
//  AI-LOGIX, INC. PROPRIETARY.
//

#ifndef _INC_NTIAPI_H
#define _INC_NTIAPI_H

#include "SysLimits.h"

#if _WIN32
#include <pshpack1.h>
#endif

// NOTE: User application should include either afxmt.h or windows.h before including
//          this header file.
//#include <afxmt.h>
//#include <windows.h>

#include "legends.h"


#include "NTiEvent.h"   // definitions of NTI events
#include "NTiData.h"    // declarations of interface structure
#include "NTiErr.h"     // result and error messages


#ifndef NV_CO_RESIDE
#include "NTI_NV.h"
#endif


//
// Result (return) codes
//

typedef DWORD               MT_RESULT;              // result value
typedef MT_RESULT *         PMT_RESULT;             // pointer to result value


#ifdef UNICODE

typedef MT_FINDEX_TABLEW   MT_FINDEX_TABLE;
typedef PMT_FINDEX_TABLEW  PMT_FINDEX_TABLE;
typedef PCMT_FINDEX_TABLEW PCMT_FINDEX_TABLE;

#else

typedef MT_FINDEX_TABLEA   MT_FINDEX_TABLE;
typedef PMT_FINDEX_TABLEA  PMT_FINDEX_TABLE;
typedef PCMT_FINDEX_TABLEA PCMT_FINDEX_TABLE;

#endif // UNICODE


// Unicode Function Definitions
#ifdef UNICODE

#define MTCallString        MTCallStringW
#define MTCallStringAsync   MTCallStringAsyncW
#define MTCallStringEx      MTCallStringExW
#define MTDialString        MTDialStringW
#define MTDialStringAsync   MTDialStringAsyncW
#define MTDialStringEx      MTDialStringExW
#define MTGetLastError      MTGetLastErrorW
#define MTGetLastErrorVal   MTGetLastErrorW( NULL, NULL, NULL )
#define MTRecFile           MTRecFileW
#define MTRecFileEx         MTRecFileExW
#define MTRecFileAsync      MTRecFileAsyncW
#define MTPlayFile          MTPlayFileW
#define MTPlayFileEx        MTPlayFileExW
#define MTPlayFileAsync     MTPlayFileAsyncW
#define MTPlayIndex         MTPlayIndexW
#define MTPlayIndexEx       MTPlayIndexExW
#define MTPlayIndexAsync    MTPlayIndexAsyncW

#define MTSendFax           MTSendFaxW
#define MTRecvFax           MTRecvFaxW
#define MTSendFaxEx         MTSendFaxExW
#define MTRecvFaxEx         MTRecvFaxExW

#else

#define MTCallString        MTCallStringA
#define MTCallStringAsync   MTCallStringAsyncA
#define MTCallStringEx      MTCallStringExA
#define MTDialString        MTDialStringA
#define MTDialStringAsync   MTDialStringAsyncA
#define MTDialStringEx      MTDialStringExA
#define MTGetLastError      MTGetLastErrorA
#define MTGetLastErrorVal   MTGetLastErrorA( NULL, NULL, NULL )
#define MTRecFile           MTRecFileA
#define MTRecFileEx         MTRecFileExA
#define MTRecFileAsync      MTRecFileAsyncA
#define MTPlayFile          MTPlayFileA
#define MTPlayFileEx        MTPlayFileExA
#define MTPlayFileAsync     MTPlayFileAsyncA
#define MTPlayIndex         MTPlayIndexA
#define MTPlayIndexEx       MTPlayIndexExA
#define MTPlayIndexAsync    MTPlayIndexAsyncA

#define MTSendFax           MTSendFaxA
#define MTRecvFax           MTRecvFaxA
#define MTSendFaxEx         MTSendFaxExA
#define MTRecvFaxEx         MTRecvFaxExA

#endif /* !UNICODE */


//
// Callback Functions
//

typedef void (CPP_CALLING_CONVENTION * MTCALLBACK)( CHANNEL nChannel, int EventCount, PMT_EVENT pEvent, LPARAM lParam );
typedef void (CPP_CALLING_CONVENTION * MTCOMPLETE)( CHANNEL nChannel, MT_EVENT TermEvent, LPARAM lParam );


//
// Device IO Functions
//

typedef BOOL  (CPP_CALLING_CONVENTION * MTDEVICEIO)( PVOID const fHandler, PVOID const pBuf, ULONG const nByteCount, PULONG const pByteAccessed );
typedef DWORD (CPP_CALLING_CONVENTION * MTDEVICESEEK)( PVOID const fHandler, ULONG const nByteCount, int const fDirection );


//
// Configuration Functions
//

_DLLIMPEXPORT int CALLING_CONVENTION
MTBoardGetConfig(
    PMTADAPTER_CONFIGURATION const     pConfig,
    PULONG const              pLength
    );

_DLLIMPEXPORT int CALLING_CONVENTION
MTBoardSetConfig(
    const PMTADAPTER_CONFIGURATION       pConfig,
    PULONG const              pLength
    );

_DLLIMPEXPORT int CALLING_CONVENTION
MTGetAdapterEEPROMConfig(
    const BOARD               nBoard,
    PCHAR const               pEEPROMConfig,
    PULONG const              pLength
    );

_DLLIMPEXPORT int CALLING_CONVENTION
MTSetAdapterEEPROMConfig(
    const BOARD               nBoard,
    PCHAR const               pEEPROMConfig,
    PULONG const              pLength
    );

//
// System Functions
//

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSysPutEvent(
    PMT_EVENT const     pMTEvent,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT     CALLING_CONVENTION
MTSysSetEventCallback(
    MTCALLBACK  const   Callback,
    const LPARAM        lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSysClearEventCallback(
    VOID
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSysWaitForEvent(
    const DWORD         dwTimeoutMs,
    PMT_EVENT const     pMTEvent,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSysStartup(
    VOID
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetSystemInfo(
    PMTSYS_INFO const   pInfo,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSysShutdown(
    VOID
    );

_DLLIMPEXPORT int CALLING_CONVENTION
MTSysGetConfig(
    PMTSYS_CONFIGURATION const       pConfig,
    PULONG const                     pLength
    );

_DLLIMPEXPORT int CALLING_CONVENTION
MTSysSetConfig(
    const PMTSYS_CONFIGURATION        pConfig,
    PULONG const                           pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetVersion(
    PMT_VERSION const pDriverVersion,           // Driver version
    PMT_VERSION const pFWVersion                // Firmware version on board 0
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetDLLVersion(
    PMT_VERSION const pDLLVersion
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTResyncTime(
    VOID
    );


//
// Board Information Functions
//

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetAdapterInfo(
    const BOARD             nBoard,
    PMTADAPTER_INFO const   pBoardInfo,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetAdapterXInfo(
    const BOARD             nBoard,
    PMTBOARD_XINFO const    pBoardXInfo,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetBoardOEMInfo(
    const BOARD         nBoard,
    PMTOEM_INFO const   pOem_info,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetBoardAssemblyInfo(
    const BOARD         nBoard,
    PSERIAL_INFO const  pBoard_SerialInfo,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTBlinkBoard(
    const BOARD         nBoard
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStopBlinkBoard(
    const BOARD         nBoard
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTOpenBoard(
    const BOARD         nBoard
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCloseBoard(
    const BOARD         nBoard
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetBoardOpenStatus(
    const BOARD         nBoard,
    BOOLEAN * const     pOpen
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetAdapterToDefault(
    const BOARD             nBoard
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetBoardStatistics(
    const BOARD             nBoard,
    PMTRUNTIME_STATISTICS const pRunTimeStat,
    PULONG const            pLength
    );

// Clear statistics works only when there is no PanicCode recorded
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTResetBoardStatistics(
    const BOARD             nBoard
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTBoardSetIdleCoding(
    const BOARD             nBoard,
    const int               nTrunk,         // 0 for the 1st trunk, and etc.
    const int               CodingFormat    // ULAWPXD or ALAWPXD
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTBoardGetIdleCoding(
    const BOARD             nBoard,
    const int               nTrunk,         // 0 for the 1st trunk, and etc.
    int * const             pCodingFormat   // Default ULAWPXD
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTBoardSetTerminationImpedance(
    const BOARD             nBoard,
    const int               nTrunk,         // 0 for the 1st trunk, and etc.
    const int               Impedance       // 0 for Hi-Impedance, 1 for Low-Impedance
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTBoardGetTerminationImpedance(
    const BOARD             nBoard,
    const int               nTrunk,         // 0 for the 1st trunk, and etc.
    int * const             pImpedance      // Default Hi-Impedance
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTBoardPCMSetSignalPolarity(
    const BOARD             iBrd,
    const int               iTrunk,         // 0 for the 1st trunk, and etc.
    const PMTBOARD_PCM_SIGNAL_POLARITY pSignal,
    PULONG const            pLength         // Number of bytes for structure Signal
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTBoardPCMGetSignalPolarity(
    const BOARD             iBrd,
    const int               iTrunk,         // 0 for the 1st trunk, and etc.
    PMTBOARD_PCM_SIGNAL_POLARITY const pSignal,      // Default to Normal
    PULONG const            pLength         // Number of bytes for structure Signal
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTBoardPCMTrunkFrameSyncControl(
    const BOARD             nBoard,
    const int               nTrunk,         // 0 for the 1st trunk, and etc.
    const BOOL              fTxFrameSyncEnable   // false for disable, true for enable
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTBoardPCMTrunkFrameSyncStatus(
    const BOARD             nBoard,
    const int               nTrunk,         // 0 for the 1st trunk, and etc.
    BOOL * const            pTxFrameSyncEnable    // Default to disable
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTBoardPCMTrunkSlotTxControl(
    const BOARD             nBoard,
    const int               iTrunk,         // 0 for the 1st trunk, and etc.
    const DWORD32           fEnableBitMap   // bit value 0 for disable, 1 for enable
                                            // bit 0 for first slot on the specified trunk and etc.
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTBoardPCMTrunkSlotTxStatus(
    const BOARD             nBoard,
    const int               iTrunk,         // 0 for the 1st trunk, and etc.
    PDWORD32 const          pEnableBitMap   // pointer to a 4-byte (32 bits) slot table
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTBoardGetCustomSwitchSetting (
    const BOARD             nBoard,
    int * const             pCurrentSwitchSetting
    );

//
// Channel Functions
//      Global Channel Index (GCI) Functions
//

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetGCI(
    const BOARD         nBoard,
    const CHANNEL       nChannelOnBoard,
    PCHANNEL const      pGCI
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetGCIMap(
    const CHANNEL       nGCI,
    PBOARD const        pBoard,
    PUSHORT const       pTDMBoardType,
    PCHANNEL const      pChannelOnBoard
    );


//
// Channel Functions
//      Event Functions

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTOpenChannel(
    const CHANNEL       nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCloseChannel(
    const CHANNEL       nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetChannelOpenStatus(
    const CHANNEL       nChannel,
    BOOLEAN * const     pOpen
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetChannelToDefault(
    const CHANNEL       nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetChannelInfo(
    const CHANNEL       nChannel,
    PMTCHAN_INFO const  pChannel_info,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetChannelStatus(
    const CHANNEL           nChannel,
    PMTCHAN_STATUS const    pChan_status,
    PULONG const            pLength
   );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetStatistics(
    const CHANNEL           nChannel,
    PMTRUNTIME_STATISTICS const pRunTimeStat,
    PULONG const            pLength
    );

// Clear statistics works only when there is no PanicCode recorded
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTResetStatistics(
    const CHANNEL           nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTFlushEvents(
    const CHANNEL       nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetEventFilters(
    const CHANNEL       nChannel,
    const USHORT        EventEnable,
    const USHORT        nRings          // Updates MT_PSTN.ring_count field
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetEventFilters(
    const CHANNEL       nChannel,
    const PUSHORT       pEventEnable,
    const PUSHORT       pRings
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTWaitForChannelEvent(
    const CHANNEL       nChannel,
    const DWORD         dwTimeoutMs,
    PMT_EVENT const     pMTEvent,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTWaitForAdapterEvent(
    const BOARD         nBoard,
    const DWORD         dwTimeoutMs,
    PMT_EVENT const     pMTEvent,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetChannelEvent(
    const DWORD         dwTimeoutMs,
    PMT_EVENT const     pMTEvent,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetChannelOwner(
    const CHANNEL   nChannel,
    int * const     pPid
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTEventControl(
    const USHORT        Action
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPutChannelEvent(
    const CHANNEL       nChannel,
    PMT_EVENT const     pMTEvent,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPutBoardEvent(
    const BOARD         nBoard,
    PMT_EVENT const     pMTEvent,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT     CALLING_CONVENTION
MTSetBoardEventCallback(
    const BOARD         nBoard,         // 0 for the 1st board, -1 for all boards
    MTCALLBACK  const   Callback,
    const LPARAM        lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTClearBoardEventCallback(
    const BOARD         nBoard          // 0 for the 1st board, -1 for all boards
    );

_DLLIMPEXPORT MT_RESULT     CALLING_CONVENTION
MTSetEventCallback(
    const CHANNEL       nChannel,
    MTCALLBACK  const   Callback,
    const LPARAM        lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTClearEventCallback(
    const CHANNEL       nChannel
    );

_DLLIMPEXPORT MT_RESULT     CALLING_CONVENTION
MTSetPriorityEventCallback(
    const CHANNEL       nChannel,
    const LONG          EventCode,
    MTCALLBACK const    Callback,
    const LPARAM        lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTClearPriorityEventCallback(
    const CHANNEL       nChannel,
    const LONG          EventCode
    );

//
// Channnel Function
//      Control Functions

// Abort and flush all functions on the specified channel
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStopChannel(
    const CHANNEL       nChannel
    );

// Stop the current active function on the specified channel
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStopCurrentFunction(
    const CHANNEL       nChannel,
    const ULONG         fOperation      // MT_ENCODE, MT_DECODE, or MT_COLLECT_DTMF
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStartCurrentFunction(
    const CHANNEL       nChannel,
    const ULONG         fOperation
    );


//
// Channnel Function
//      Parameter Functions

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetPSTNParams(
    const CHANNEL       nChannel,
    PMT_PSTN const      pRegs,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetPSTNParams(
    const CHANNEL       nChannel,
    PMT_PSTN const      pRegs,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetToneGenParams(
    const CHANNEL       nChannel,
    const UCHAR         Type,
    PMT_GENREGS const   pRegs,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetToneGenParams(
    const CHANNEL       nChannel,
    const UCHAR         Type,
    PMT_GENREGS const   pRegs,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetToneDetectParams(
    const CHANNEL           nChannel,
    const UCHAR             Type,
    PMT_DETECTREGS const    pRegs,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetDTMFDetectParams(
    const CHANNEL           nChannel,
    PMT_DTMF_DETECTPARAMS const    pParams,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetACTDParams(
    const CHANNEL           nChannel,
    PMT_ACTPARAMS const     pActParams,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTEnableMixing(
    const CHANNEL           nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTDisableMixing(
    const CHANNEL           nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetMixingStatus(
    const CHANNEL           nChannel,
    BOOLEAN * const         pOn
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetLimiterControl(
    const CHANNEL       nChannel,
    const USHORT        Value               // 0 for off, 1 for on (default)
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetLimiterControl(
    const CHANNEL       nChannel,
    PUSHORT const       pValue
    );

#if 0   // Moved to ToBeDeletedInJun04.h
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetPxdParams(
    const CHANNEL       nChannel,
    PMT_PSTNPXD const   pRegs,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetPxdParams(
    const CHANNEL       nChannel,
    PMT_PSTNPXD const   pRegs,
    PULONG const        pLength
    );
#endif

// Subscriber Line Interface Circuit Control
// SLIC control is only available on station channel
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetSLIC(
    const CHANNEL       nChannel,
    const USHORT        Value          // 0 for Power Down, 1 for Active Normal
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetSLIC(
    const CHANNEL       nChannel,
    PUSHORT const       pValue
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetECControl(
    const CHANNEL       nChannel,
    const USHORT        Value
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetECControl(
    const CHANNEL       nChannel,
    PUSHORT const       pValue
    );

// The following APIs is available only when EC is on
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetECFreezeAdaptation(
    const CHANNEL       nChannel,
    const USHORT        Value          // 0 for unfreeze, 1 for freeze adaptation
    );

// The following APIs is available only when EC is on
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetECFreezeAdaptation(
    const CHANNEL       nChannel,
    PUSHORT const       pValue
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetECParams(
    const CHANNEL       nChannel,
    PMT_ECPARAMS const  pParams,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetECParams(
    const CHANNEL       nChannel,
    PMT_ECPARAMS const  pParams,
    PULONG const        pLength
    );


//
// Channel Functions
//      Channel Input Parameter Functions

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputSetToneDetectParams(
    const CHANNEL           nChannel,
    const SHORT             nInput,
    const UCHAR             Type,
    PMT_DETECTREGS const    pRegs,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputGetToneDetectParams(
    const CHANNEL           nChannel,
    const SHORT             nInput,
    UCHAR const             Type,
    PMT_DETECTREGS const    pRegs,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputSetDTMFDetectParams(
    const CHANNEL           nChannel,
    const SHORT             nInput,
    PMT_DTMF_DETECTPARAMS const    pParams,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputGetDTMFDetectParams(
    const CHANNEL           nChannel,
    const SHORT             nInput,
    PMT_DTMF_DETECTPARAMS const    pParams,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputSetACTDParams(
    const CHANNEL           nChannel,
    const SHORT             nInput,
    PMT_ACTPARAMS const     pActParams,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputGetACTDParams(
    const CHANNEL           nChannel,
    const SHORT             nInput,
    PMT_ACTPARAMS const     pActParams,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputGetACTDDetectPower(
    const CHANNEL           nChannel,
    const SHORT             nInput,
    float * const           pPower          // Activity in dBm
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputACTDControl(
    const CHANNEL           nChannel,
    const SHORT             nInput,
    const BOOLEAN           fOn             // TRUE for ACTD on, FALSE for ACTD off
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputACTDStatus(
    const CHANNEL           nChannel,
    const SHORT             nInput,
    BOOLEAN * const         pOn
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputToneDetectControl(
    const CHANNEL           nChannel,
    const SHORT             nInput,
    const UCHAR             toneType,
    const ULONG             fQueueControl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputToneDetectStatus(
    const CHANNEL           nChannel,
    const SHORT             nInput,
    const UCHAR             toneType,
    PULONG const            pQueueStatus
    );


//
// Channel Functions
//      Stream Functions

// WAV Streaming functions with termination of time and buffer size

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetStreamingConfig(
    const CHANNEL       nChannel,
    const ULONG         BufSize     // API internal buffer size, default 64K bytes
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetStreamingConfig(
    const CHANNEL       nChannel,
    const PULONG        pBufSize    // API internal buffer size, default 64K bytes
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStartStreaming(
    const CHANNEL           nChannel,
    const USHORT            Action,
    const PMT_IO_CONTROL    pIoCtl
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStreamBufOut(
    const CHANNEL       nChannel,
    const DWORD         dwTimeoutMs,
    PUCHAR const        pBuf,
    PULONG const        pBufSize
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStreamBufIn(
    const CHANNEL       nChannel,
    const DWORD         dwTimeoutMs,
    PUCHAR const        pBuf,
    PULONG const        pBufSize
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStreamBufPause(       // Controls StreamOut only
    const CHANNEL   nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStreamBufResume(      // Controls StreamOut only
    const CHANNEL   nChannel
    );


// Stream control function

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStopStreaming(        // Stop the specified streaming
    const CHANNEL   nChannel,
    const USHORT    Action,
    const USHORT    StopMode
    );

//
// Channel Functions
//      Call Setup Functions

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCallStringA(
    const CHANNEL           nChannel,
    PCSTR                   pCall_string,
    const PMT_IO_CONTROL    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCallStringW(
    const CHANNEL           nChannel,
    PCWSTR                  pCall_string,
    const PMT_IO_CONTROL    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCallStringExA(
    const CHANNEL           nChannel,
    PCSTR                   pCall_string,
    const PMT_IO_CONTROL    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCallStringAsyncA(
    const CHANNEL           nChannel,
    PCSTR                   pCall_string,
    const PMT_IO_CONTROL    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCallStringExW(
    const CHANNEL           nChannel,
    PCWSTR                  pCall_string,
    const PMT_IO_CONTROL    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCallStringAsyncW(
    const CHANNEL           nChannel,
    PCWSTR                  pCall_string,
    const PMT_IO_CONTROL    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTDialStringA(
    const CHANNEL           nChannel,
    const PCSTR             pDial_string,
    const PMT_IO_CONTROL    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTDialStringW(
    const CHANNEL           nChannel,
    const PCWSTR            pDial_string,
    const PMT_IO_CONTROL    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTDialStringExA(
    const CHANNEL           nChannel,
    const PCSTR             pDial_string,
    const PMT_IO_CONTROL    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTDialStringAsyncA(
    const CHANNEL           nChannel,
    const PCSTR             pDial_string,
    const PMT_IO_CONTROL    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTDialStringExW(
    const CHANNEL           nChannel,
    const PCWSTR            pDial_string,
    const PMT_IO_CONTROL    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTDialStringAsyncW(
    const CHANNEL           nChannel,
    const PCWSTR            pDial_string,
    const PMT_IO_CONTROL    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTEnableCallerID(
    const CHANNEL           nChannel,
    const USHORT            nLevel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetCallerIDStatus(
    const CHANNEL           nChannel,
    PUSHORT const           pState,
    PUSHORT const           pLevel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTDisableCallerID(
    const CHANNEL           nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetCallerID(
    const CHANNEL           nChannel,
    PUSHORT const           pStringLen,
    PUCHAR const            pIDString
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTFlushCallerID(
    const CHANNEL       nChannel
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetCallStatus(
    const CHANNEL           nChannel,
    const PCALL_RESULT      pCall_result,
    PULONG const            pLength
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetDigitsEx(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetDigitsAsync(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetDigits(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTReadDigit(
    const CHANNEL       nChannel,
    const PUCHAR        pDigit
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTReadDTMFTone(
    const CHANNEL       nChannel,
    const PUCHAR        pTone,
    const PUCHAR        pDirection
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTReadMFTone(
    const CHANNEL       nChannel,
    const PUCHAR        pTone,
    const PUCHAR        pDirection
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTClearDTMFDigits(
    const CHANNEL nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTClearMFTones(
    const CHANNEL nChannel
    );


//
// Channel Functions
//      Media access with application provided Device IO Functions

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetDeviceIO(
    const CHANNEL           nChannel,
    const MTDEVICEIO        DeviceRead,
    const MTDEVICEIO        DeviceWrite,
    const MTDEVICESEEK      DeviceSeek
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayDevice(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayDeviceEx(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayDeviceAsync(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTRecDevice(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTRecDeviceEx(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTRecDeviceAsync(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );


//
// Channel Functions
//      Play Functions

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayBuffer(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayBufferEx(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayBufferAsync(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayFileA(
    const CHANNEL           nChannel,
    PCSTR                   pFileName,
    const PMT_IO_CONTROL    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayFileW(
    const CHANNEL           nChannel,
    PCWSTR                  pFileName,
    const PMT_IO_CONTROL    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayFileExA(
    const CHANNEL           nChannel,
    PCSTR                   pFileName,
    const PMT_IO_CONTROL    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayFileAsyncA(
    const CHANNEL           nChannel,
    PCSTR                   pFileName,
    const PMT_IO_CONTROL    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayFileExW(
    const CHANNEL           nChannel,
    PCWSTR                  pFileName,
    const PMT_IO_CONTROL    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayFileAsyncW(
    const CHANNEL           nChannel,
    PCWSTR                  pFileName,
    const PMT_IO_CONTROL    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayIndexA(
    const CHANNEL           nChannel,
    PCSTR                   pFileName,
    PCMT_INDEX_TABLE        pIxTable,
    const PMT_IO_CONTROL    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayIndexExA(
    const CHANNEL           nChannel,
    PCSTR                   pFileName,
    PCMT_INDEX_TABLE        pIxTable,
    const PMT_IO_CONTROL    pIoCtl ,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayIndexAsyncA(
    const CHANNEL           nChannel,
    PCSTR                   pFileName,
    PCMT_INDEX_TABLE        pIxTable,
    const PMT_IO_CONTROL    pIoCtl ,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayIndexW(
    const CHANNEL           nChannel,
    PCWSTR                  pFileName,
    PCMT_INDEX_TABLE        pIxTable,
    const PMT_IO_CONTROL    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayIndexExW(
    const CHANNEL           nChannel,
    PCWSTR                  pFileName,
    PCMT_INDEX_TABLE        pIxTable,
    const PMT_IO_CONTROL    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayIndexAsyncW(
    const CHANNEL           nChannel,
    PCWSTR                  pFileName,
    PCMT_INDEX_TABLE        pIxTable,
    const PMT_IO_CONTROL    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );


//
// Channel Functions
//      Record Functions

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTRecBuffer(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTRecBufferEx(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTRecBufferAsync(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTRecFileA(
    const CHANNEL           nChannel,
    PCSTR                   pFileName,
    PMT_IO_CONTROL const    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTRecFileW(
    const CHANNEL           nChannel,
    PCWSTR                  pFileName,
    PMT_IO_CONTROL const    pIoCtl
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTRecFileExA(
    const CHANNEL           nChannel,
    PCSTR                   pFileName,
    PMT_IO_CONTROL const    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTRecFileAsyncA(
    const CHANNEL           nChannel,
    PCSTR                   pFileName,
    PMT_IO_CONTROL const    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTRecFileExW(
    const CHANNEL           nChannel,
    PCWSTR                  pFileName,
    PMT_IO_CONTROL const    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTRecFileAsyncW(
    const CHANNEL           nChannel,
    PCWSTR                  pFileName,
    PMT_IO_CONTROL const    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );


//
// Channel Functions
//      Gain/Volume Functions
//

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetAGC(
    const CHANNEL           nChannel,
    const SHORT             TMA,        // Target Max. Amplitude, 1 to -45 dBm, def=-6 dBm
    const SHORT             AGCR,       // Attack Gain Change Rate, in unit of -0.00212 dB/ms, def=400
    const SHORT             DGCR,       // Decay Gain Change Rate, in unit of 0.00212 dB/ms, def=4
    const SHORT             MA          // Max. Amplification, 6 dB to 48 dB in 6dB steps, def=30 dB
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetAGC(
    const CHANNEL           nChannel,
    PSHORT const            pTMA,
    PSHORT const            pAGCR,
    PSHORT const            pDGCR,
    PSHORT const            pMA
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetGain(
    const CHANNEL       nChannel,
    const SHORT         Gain            // 24 dB to -50 (equals to muting)
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetGain(
    const CHANNEL       nChannel,
    const PSHORT        pGain           // 24 dB to -50 (equals to muting)
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStepGain(
    const CHANNEL       nChannel,
    const SHORT         step            // STEP_DOWN, STEP_RESET, or STEP_UP
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTAGCControl(
    const CHANNEL       nChannel,
    const BOOLEAN       fOn             // TRUE for AGC on, FALSE for AGC off
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTAGCStatus(
    const CHANNEL       nChannel,
    BOOLEAN * const     pOn             // TRUE for AGC on, FALSE for AGC off
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetAVC(
    const CHANNEL           nChannel,
    const SHORT             TMA,        // Target Max. Amplitude, 1 to -45 dBm, def=0 dBm
    const SHORT             AVCR,       // Attack Volume Change Rate, in unit of -0.00212 dB/ms, def=400
    const SHORT             DVCR,       // Decay Volume Change Rate, in unit of 0.00212 dB/ms, def=8
    const SHORT             MA          // Max. Amplification, 6 dB to 48 dB in 6dB steps, def=42 dB
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetAVC(
    const CHANNEL           nChannel,
    PSHORT const            pTMA,
    PSHORT const            pAVCR,
    PSHORT const            pDVCR,
    PSHORT const            pMA
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetVolume(
    const CHANNEL       nChannel,
    const SHORT         Volume          // 24 dB to -50 (equals to muting)
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetVolume(
    const CHANNEL       nChannel,
    const PSHORT        pVolume         // 24 dB to -50 (equals to muting)
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStepVolume(
    const CHANNEL       nChannel,
    const SHORT         step            // STEP_DOWN, STEP_RESET, or STEP_UP
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTAVCControl(
    const CHANNEL       nChannel,
    const BOOLEAN       fOn             // TRUE for AVC on, FALSE for AVC off
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTAVCStatus(
    const CHANNEL       nChannel,
    BOOLEAN * const     pOn             // TRUE for AGC on, FALSE for AGC off
    );


//
// Channel Functions
//      Channel Input AGC/Gain Functions

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputSetAGC(
    const CHANNEL           nChannel,
    const SHORT             nInput,     // Input index: 0 for primary, 1 for secondary input
    const SHORT             TMA,        // Target Max. Amplitude, 1 to -45 dBm, def=-6 dBm
    const SHORT             AGCR,       // Attack Gain Change Rate, in unit of -0.00212 dB/ms, def=400
    const SHORT             DGCR,       // Decay Gain Change Rate, in unit of 0.00212 dB/ms, def=4
    const SHORT             MA          // Max. Amplification, 6 dB to 48 dB in 6dB steps, def=30 dB
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputGetAGC(
    const CHANNEL           nChannel,
    const SHORT             nInput,     // Input index: 0 for primary, 1 for secondary input
    PSHORT const            pTMA,
    PSHORT const            pAGCR,
    PSHORT const            pDGCR,
    PSHORT const            pMA
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputSetGain(
    const CHANNEL       nChannel,
    const SHORT         nInput,         // Input index: 0 for primary, 1 for secondary input
    const SHORT         Gain            // 24 dB to -50 (equals to muting)
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputGetGain(
    const CHANNEL       nChannel,
    const SHORT         nInput,         // Input index: 0 for primary, 1 for secondary input
    const PSHORT        pGain           // 24 dB to -50 (equals to muting)
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputStepGain(
    const CHANNEL       nChannel,
    const SHORT         nInput,         // Input index: 0 for primary, 1 for secondary input
    const SHORT         step            // STEP_DOWN, STEP_RESET, or STEP_UP
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputAGCControl(
    const CHANNEL       nChannel,
    const SHORT         nInput,         // Input index: 0 for primary, 1 for secondary input
    const BOOLEAN       fOn             // TRUE for AGC on, FALSE for AGC off
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChInputAGCStatus(
    const CHANNEL       nChannel,
    const SHORT         nInput,         // Input index: 0 for primary, 1 for secondary input
    BOOLEAN * const     pOn             // TRUE for AGC on, FALSE for AGC off
    );


//
// Channel Functions
//      HIGHPASS Filter Functions
//

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTHighpassFilterControl(
    const CHANNEL       nChannel,
    const BOOLEAN       fOn             // TRUE for filter on, FALSE for filter off
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTHighpassFilterStatus(
    const CHANNEL       nChannel,
    BOOLEAN * const     pOn             // TRUE for filter on, FALSE for filter off
    );

//
// Channel Functions
//      Voice/Answering Machine Detection Functions
//

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChSetVoiceDetectParams(
    const CHANNEL               nChannel,
    PMT_VOICEDETECTPARAMS const pVoiceDetectParams,
    PULONG const                pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChGetVoiceDetectParams(
    const CHANNEL               nChannel,
    PMT_VOICEDETECTPARAMS const pVoiceDetectParams,
    PULONG const                pLength
    );

//
// Channel Functions
//      TONE Functions
//

// CPM Tones

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetCpmToneParams(
    const CHANNEL       nChannel,
    const USHORT        Index,
    const PMT_CPMTONE   pCpm,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetCpmToneParams(
    const CHANNEL       nChannel,
    const PMT_CPMTONE   pCpm,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTResetCpmToneParams(
    const CHANNEL       nChannel,
    const UCHAR         Mode            // 0 for disable all CPM tones, others
                                        // for reset CPM profile to registered default
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChGetCPMSignalParams(
    const CHANNEL       nChannel,
    const USHORT        Index,
    const PMT_CPT_TONE  pCpm,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChSetCPMSignalParams(
    const CHANNEL       nChannel,
    const PMT_CPT_TONE   pCpm,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChResetCPMSignalParams(
    const CHANNEL       nChannel,
    const UCHAR         Mode            // 0 for disable all CPM tones, others
                                        // for reset CPM profile to registered default
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChVoiceDetectControl(
    const CHANNEL       nChannel,
    const BOOLEAN       fOn             // TRUE for AMD on, FALSE for AMD off
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChVoiceDetectStatus(
    const CHANNEL       nChannel,
    BOOLEAN * const     pOn             // TRUE for AMD on, FALSE for AMD off
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChCPMControl(
    const CHANNEL       nChannel,
    const BOOLEAN       fOn             // FALSE to disable CPM, others to enable CPM
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChCPMStatus(
    const CHANNEL       nChannel,
    BOOLEAN * const     pOn             // FALSE for CPM disabled, others for CPM enabled
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCalculateFilterRange(
    double          dFreq1,
    double          dFreq2,
    double          dFreq3,
    PCPT_SIGNAL     pCPTSignal
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTLoadSignalProfile(
	PCHAR           pProfileName, 
	PCHAR           pSignalName, 
	PMT_CPT_TONE    pCPTSignal
	);


// User Tones

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetUsrToneParams(
    const CHANNEL           nChannel,
    const USHORT            Index,
    const PMT_USRTONE       pUsrTone,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTClearUsrToneParams(
    const CHANNEL           nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetUsrToneParams(
    const CHANNEL           nChannel,
    const PMT_USRTONE       pUsrTone,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChGetUsrSignalParams(
    const CHANNEL           nChannel,
    const USHORT            Index,
    const PMT_USR_TONE      pUsrTone,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChClearUsrSignalParams(
    const CHANNEL           nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChSetUsrSignalParams(
    const CHANNEL           nChannel,
    const PMT_USR_TONE      pUsrTone,
    PULONG const            pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChCPMSignalControl(
    const CHANNEL           nChannel,
    const USHORT            Index,          // 0-14 for CPM signals
    const BOOLEAN           fOn             // 0 to disable, others to enable
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChCPMSignalStatus(
    const CHANNEL           nChannel,
    const USHORT            Index,          // 0-14 for CPM signals
    BOOLEAN * const         pOn             // 0 for disabled, others for enabled
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChUsrSignalControl(
    const CHANNEL           nChannel,
    const USHORT            Index,          // 15-19 for user defined signals
    const BOOLEAN           fOn             // 0 to disable, others to enable
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTChUsrSignalStatus(
    const CHANNEL           nChannel,
    const USHORT            Index,          // 15-19 for user defined signals
    BOOLEAN * const         pOn             // 0 for disabled, others for enabled
    );


// ALERT Tones

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetAlertToneParams(
    const CHANNEL       nChannel,
    const PMT_ALERTTONE pAlert,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetAlertToneParams(
    const CHANNEL       nChannel,
    const PMT_ALERTTONE pAlert,
    PULONG const        pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStartAlertTone(
    const CHANNEL       nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStopAlertTone(
    const CHANNEL       nChannel
    );


// Play Tones

#define MAX_REPEAT_TONE_COUNT       10
// The maximum number of tones to paly in MTPlayTone() repeat mode set
// through MT_IO_CONTROL.StartControl field with bit REPEAT_TONE set

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayToneEx(
    const CHANNEL           nChannel,
    PMT_IO_CONTROL const    pIoCtl,
    const LPOVERLAPPED      pOverlapped
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayToneAsync(
    const CHANNEL           nChannel,
    PMT_IO_CONTROL const    pIoCtl,
    const MTCOMPLETE        pCompletionRoutine,
    const LPARAM            lParam
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTPlayTone(
    const CHANNEL           nChannel,
    const PMT_IO_CONTROL    pIoCtl
    );


//
// Channel Functions
//      Termination/Monitor Functions
//

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetLastTerm(
    const CHANNEL       nChannel,
    PULONG const        pEventCode
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetTerm(
    const CHANNEL       nChannel,
    const ULONG         TermEnable
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetTerm(
    const CHANNEL       nChannel,
    const PULONG        pTermEnable
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetMoni(
    const CHANNEL       nChannel,
    const ULONG         MoniEnable
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetMoni(
    const CHANNEL       nChannel,
    const PULONG        pMoniEnable
    );


//
// Channel Functions
//      MUX/MVIP bus Routines
//

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStartMUX(
    USHORT * const  pTotalChans
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStopMUX(
    VOID
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStartMUXOnChannel(
    const CHANNEL       nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStopMUXOnChannel(
    const CHANNEL       nChannel
    );


#define MAX_MVIP_SLOT               512     // 16 streams with 32 slots per stream
#define MAX_H100_SLOT               4096    // 32 streams with 128 slots per stream

typedef struct {
    ULONG   iTransmitter:   16;             // Index of transmitter
    ULONG   fTxType:        4;              // Flag of the transmitter:
                                            // 0x00 - none
                                            // 0x01 - channel
                                            // 0x02 - audio jack
                                            // 0x03 - summation
                                            // 0x04 - framer
    ULONG   NumReceiver:    12;             // Max of 4095 receiver connected to this slot
} MT_CT_SLOT, *PMT_CT_SLOT;

// Definition for fTxType for MT_CT_SLOT
#define CT_NONE             0x00
#define CT_DR               0x01
#define CT_NI               0x02
#define CT_CH               (CT_DR|CT_NI)
#define CT_AJ               0x04
#define CT_SUMM             0x05
#define CT_FRAMER           0x06



_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetCTSlotConnection(
    const int           BusType,            // MUX_MVIP or MUX_H100
    const int           StartStream,        // Indexed 0 - 15 for MVIP; 0 - 31 for H100
    const int           StartTimeSlot,      // Indexed 0 - 31 for MVIP; 0 - 31,63,127 for H100
    int * const         pSlotCount,         // Pointer to count of the MT_CT_SLOT buffer
    PMT_CT_SLOT const   pSlotTable          // Pointer to MT_CT_SLOT buffer
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTResetCTSlotConnection( void );


//
// Audio Jack Functions
//      MUX/MVIP bus Routines
//

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetAJMap(
    const int           iAudioJack,     // Index of audio jack, 0 to 15
    PBOARD const        pBoard          // Address to take the returned board index
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTAJTalk(
    const int           iAudioJack,     // Index of audio jack, 0 to 15
    const int           iStream,        // Index of TDM stream: 0 - 15 for MVIP
    const int           iSlot           // Index of TDM slot on a stream: 0 - 31 for MVIP
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTAJListen(
    const int           iAudioJack,     // Index of audio jack, 0 to 15
    const int           iStream,        // Index of TDM stream: 0 - 15 for MVIP
    const int           iSlot           // Index of TDM slot on a stream: 0 - 31 for MVIP
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetAJTalk(
    const int           iAudioJack,     // Index of audio jack, 0 to 15
    int * const         pStream,        // Address for TDM stream index: 0 - 15 for MVIP,
                                        // -1 for not connected
    int * const         pSlot           // Address for TDM slot index: 0 - 31 for MVIP,
                                        // -1 for not connected
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetAJListen(
    const int           iAudioJack,     // Index of audio jack, 0 to 15
    int * const         pStream,        // Address for TDM stream index: 0 - 15 for MVIP,
                                        // -1 for not connected
    int * const         pSlot           // Address for TDM slot index: 0 - 31 for MVIP,
                                        // -1 for not connected
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTResetAJTalk(
    const int           iAudioJack      // Index of audio jack, 0 to 15
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTResetAJListen(
    const int           iAudioJack      // Index of audio jack, 0 to 15
    );


//
// LoopVoltage/LoopCurrent/Ring detection status Functions
//

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetLVDetectState(
    const CHANNEL       nChannel,
    const PUCHAR        pState          // 0 for not detecting none
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetLCDetectState(
    const CHANNEL       nChannel,
    const PUCHAR        pState          // 0 for not detecting none
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetRingDetectState(
    const CHANNEL       nChannel,
    const PUCHAR        pState          // 0 for not detecting none
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetLVParams(
    const CHANNEL       nChannel,
    const SHORT         ThresholdLow,   // Loop voltage threshold low: ranged
                                        //  -60V to 60V, default -16V
    const SHORT         ThresholdHigh,  // Loop voltage threshold high: ranged
                                        //  -60V to 60V, default 16V
    const USHORT        msDeglitchTime  // De-bouncing time in unit of 10ms, ranged
                                        //  10 ms to 2550 ms, default 50ms
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetLVParams(
    const CHANNEL       nChannel,
    const PSHORT        pThresholdLow,  // Pointer for loop voltage threshold low
    const PSHORT        pThresholdHigh, // Pointer for loop voltage threshold high
    const PUSHORT       pmsDeglitchTime // Pointer for de-bouncing time in unit of 10ms
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetLVDetectValue(
    const CHANNEL       nChannel,
    const PSHORT        pValue          // Pointer for current loop voltage
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetReverseLoopPolarity(
    const CHANNEL       nChannel,
    const BOOLEAN       fReversed       // FALSE for normal, TRUE for reversed
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetReverseLoopPolarity(
    const CHANNEL       nChannel,
    BOOLEAN * const     pReversed       // Pointer for reversed polarity setting
    );

//
// Summation Functions
//

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetSummationInfo(
    const int           iSummation,     // Index of summation function, 0 for the first
    PMTSUMMATION_INFO const pInfo,      // Pointer to the resource information structure
    PULONG const        pLength         // Pointer to the size of the data structure
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTResetSummation(
    const int           iSummation      // Index of summation function, 0 for the first
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStartSummation(
    const int           iSummation,     // Index of summation function, 0 for the first
    const int           iStream,        // Index of TDM output stream: 0 - 15 for MVIP
    const int           iSlot           // Index of TDM output slot on a stream: 0 - 31 for MVIP
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTStopSummation(
    const int           iSummation      // Index of summation function, 0 for the first
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTAddSummationInput(
    const int           iSummation,     // Index of summation function, 0 for the first
    const int           iInput,         // Index of summation input: 0 - 15/31
    const int           iStream,        // Index of TDM output stream: 0 - 15 for MVIP
    const int           iSlot,          // Index of TDM output slot on a stream: 0 - 31 for MVIP
    const SHORT         Gain            // Initial gain for the specified input: max 24 dB min -50 dB
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTRemoveSummationInput(
    const int           iSummation,     // Index of summation function, 0 for the first
    const int           iInput          // Index of summation input: 0 - 15/31
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetSummationInputs(
    const int           iSummation,     // Index of summation function, 0 for the first
    int * const         pInputs,        // Pointer of number of input settings to get
                                        // from the 1st input
    int * const         pStream,        // Pointer for TDM streams of the max. 16/32 inputs
    int * const         pSlot,          // Pointer for TDM slots of the max. 16/32 inputs
    SHORT * const       pGain           // Pointer for gains of the max. 16/32 inputs
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetSummationOutput(
    const int           iSummation,     // Index of summation function, 0 for the first
    int * const         pStream,        // Pointer for TDM stream of the output
    int * const         pSlot,          // Pointer for TDM slot of the output
    PSHORT const        pVolume         // Pointer for current volume of the output
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetSummationGain(
    const int           iSummation,     // Index of summation function, 0 for the first
    const int           iInput,         // Index of summation input: 0 - 15/31
    const SHORT         Gain            // Gain for the specified input
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetSummationGain(
    const int           iSummation,     // Index of summation function, 0 for the first
    const int           iInput,         // Index of summation input: 0 - 15/31
    PSHORT const        pGain           // Pointer for current gain for the specified input
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetSummationVolume(
    const int           iSummation,     // Index of summation function, 0 for the first
    const SHORT         Volume          // Output volume for the specified summation output
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetSummationVolume(
    const int           iSummation,     // Index of summation function, 0 for the first
    PSHORT  const       pVolume         // Pointer for current volume for the specified summation output
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetSummationAVC(
    const int           iSummation,     // Index of summation function, 0 for the first
    const SHORT         TMA,            // Target Max. Amplitude, 1 to -45 dBm, def=0 dBm
    const SHORT         AVCR,           // Attack Volume Change Rate, in unit of -0.00212 dB/ms, def=400
    const SHORT         DVCR,           // Decay Volume Change Rate, in unit of 0.00212 dB/ms, def=8
    const SHORT         MA              // Max. Amplification, 6 dB to 48 dB in 6dB steps, def=42 dB
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetSummationAVC(
    const int           iSummation,     // Index of summation function, 0 for the first
    PSHORT const        pTMA,           // Pointer for Target Max. Amplitude
    PSHORT const        pAVCR,          // Pointer for Attack Volume Change Rate
    PSHORT const        pDVCR,          // Pointer for Decay Volume Change Rate
    PSHORT const        pMA             // Pointer for Max. Amplification
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSummationAVCControl(
    const int           iSummation,     // Index of summation function, 0 for the first
    const BOOLEAN       fOn             // TRUE for AVC on, FALSE for AVC off
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSummationAVCStatus(
    const int           iSummation,     // Index of summation function, 0 for the first
    BOOLEAN * const     pOn             // TRUE for AVC on, FALSE for AVC off
    );


//
// CT Bus functions
//

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTArmMasterAlarm(
    const BOARD         nBoard
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTDisarmMasterAlarm(
    const BOARD         nBoard
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetMasterAlarmStatus(
    const BOARD         nBoard,
    BOOLEAN * const     pEnabled
    );



_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTArmCTAlarm(
    const BOARD         nBoard,
    const int           BusType,        // MUX_MVIP or MUX_H100
    const int           nMaster         // MODE_MASTER, MODE_MASTER_A or MODE_MASTER_B
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTDisarmCTAlarm(
    const BOARD         nBoard,
    const int           BusType,        // MUX_MVIP or MUX_H100
    const int           nMaster         // MODE_MASTER, MODE_MASTER_A or MODE_MASTER_B
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetCTAlarmStatus(
    const BOARD         nBoard,
    const int           BusType,        // MUX_MVIP or MUX_H100
    const int           nMaster,        // MODE_MASTER, MODE_MASTER_A or MODE_MASTER_B
    BOOLEAN * const     pArmed
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetCTRoute(
    const CHANNEL       nChannel,
    const UCHAR         Direction,
    const int           Stream,
    const int           Slot
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetCTRoute(
    const CHANNEL       nChannel,
    const UCHAR         Direction,
    int * const         pStream,
    int * const         pSlot
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTResetCTRoute(
    const CHANNEL       nChannel,
    const UCHAR         Direction,
    const int           Stream,
    const int           Slot
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetInputs(
    const CHANNEL   nChannel,       // GCI index for VR64/32 channel
                                    // The following is for channel primary input connection
    const int       iStream1,       // Index of TDM stream: 0 - 15 for MVIP
    const int       iSlot1,         // Index of TDM slot on a stream: 0 to 31 for MVIP
                                    // The following is for channel secondary input connection
    const int       iStream2,       // Index of TDM stream: 0 - 15 for MVIP
    const int       iSlot2          // Index of TDM slot on a stream: 0 to 31 for MVIP
    );

#define MTChInputSetCTConnection    MTSetInputs  // Alias for easier searching


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetInputs(
    const CHANNEL   nChannel,       // GCI index for VR64/32 channel
                                    // The following is for channel primary input connection
    int * const     pStream1,       // Pointr for TDM stream index:
                                    //      0 - 15 for MVIP, -1 for not connected
    int * const     pSlot1,         // Pointer for TDM slot index on a stream:
                                    //      0 to 31 for MVIP, -1 for not connected
                                    // The following is for channel secondary input connection
    int * const     pStream2,       // Pointr for TDM stream index:
                                    //      0 - 15 for MVIP, -1 for not connected
    int * const     pSlot2          // Pointer for TDM slot index on a stream:
                                    //      0 to 31 for MVIP, -1 for not connected
    );

#define MTChInputGetCTConnection    MTGetInputs  // Alias for easier searching


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTResetInputs(
    const CHANNEL   nChannel        // GCI index for VR64/32 channel
    );

#define MTChInputResetCTConnection  MTResetInputs // Alias for easier searching


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetOutput(
    const CHANNEL       nChannel,
    const int           iStream,    // Index of TDM stream: 0-15 for MVIP, 0-31 for H100
    const int           iSlot       // Index of TDM slot on a stream: 0-31 for MVIP
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetOutput(
    const CHANNEL       nChannel,
    int * const         pStream,
    int * const         pSlot
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTResetOutput(
    const CHANNEL       nChannel
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetFramerOutput(
    const BOARD         nBoard,     // Board index
    const USHORT        nFramer,    // Index of framer on the specified board
    const USHORT        nSlot,      // Index of framer slot: 1-24 for T1, 0-31 for E1
    const int           iStream,    // Index of TDM stream: 0-15 for MVIP, 0-31 for H100
    const int           iSlot       // Index of TDM slot on a stream: 0-31 for MVIP
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetFramerOutput(
    const BOARD         nBoard,     // Board index
    const USHORT        nFramer,    // Index of framer on the specified board
    const USHORT        nSlot,      // Index of framer slot: 1-24 for T1, 0-31 for E1
    int * const         pStream,
    int * const         pSlot
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTResetFramerOutput(
    const BOARD         nBoard,     // Board index
    const USHORT        nFramer,    // Index of framer on the specified board
    const USHORT        nSlot       // Index of framer slot: 1-24 for T1, 0-31 for E1
    );

//
// Misc. Functions
//

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetLastErrorA(
    const PMT_RESULT    pExtError,
    const PUSHORT       nErrBufSize,
    const PSTR          pErrBuf
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetLastErrorW(
    const PMT_RESULT    pExtError,
    const PUSHORT       nErrBufSize,
    const PWSTR         pErrBuf
    );


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetCTMasterClock(
    const BOARD         nBoard,
    const int           BusType,    // MUX_MVIP or MUX_H100
    const int           mMaster,    // MODE_SLAVE, MODE_MASTER_A, or MODE_MASTER_B
    const USHORT        ClockSource
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetCTMasterClock(
    const BOARD         nBoard,
    const int           BusType,    // MUX_MVIP or MUX_H100
    int * const         pMaster,    // MODE_SLAVE, MODE_MASTER_A, or MODE_MASTER_B
    USHORT * const      pClockSource
    );


#define MTSetSlaveSEC8KClockSource(b,c)     MTSetCTNetrefSource( b, MUX_MVIP, c )
#define MTGetSlaveSEC8KClockSource(b,p)     MTGetCTNetrefSource( b, MUX_MVIP, p )


_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetCTNetrefSource(
    const BOARD         nBoard,
    const int           BusType,
    const USHORT        ClockSource
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetCTNetrefSource(
    const BOARD         nBoard,
    const int           BusType,
    USHORT * const      pClockSource
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetCTSlaveClockMode(
    const BOARD         nBoard,
    const int           BusType,            // MUX_MVIP or MUX_H100
    const USHORT        Clock,              // MODE_CLOCK_A or MODE_CLOCK_B
    const BOOLEAN       fAutoSwitch         // True for auto-switching select
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetCTSlaveClockMode(
    const BOARD         nBoard,
    const int           BusType,            // MUX_MVIP or MUX_H100
    PUSHORT const       pClock,
    BOOLEAN * const     pAutoSwitch
    );

#define MDSetNET1   MTSetNET1
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetNET1(
    const BOARD         nBoard,
    const USHORT        nFramer
    );

#define MDGetNET1   MTGetNET1
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetNET1(
    const BOARD         nBoard,
    USHORT * const      pFramer
    );

#define MDSetNET2   MTSetNET2
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetNET2(
    const BOARD         nBoard,
    const USHORT        nFramer
    );

#define MDGetNET2   MTGetNET2
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetNET2(
    const BOARD         nBoard,
    USHORT * const      pFramer
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetNET(
    const BOARD         nBoard,
    const USHORT        nNET,
    const USHORT        nFramer
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetNET(
    const BOARD         nBoard,
    const USHORT        nNET,
    USHORT * const      pFramer
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetFraming(
    PMT_FRAMER_STATE const  pFraming
);

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTArmFramerAlarm(
    const BOARD         nBoard,
    const USHORT        nFramer,
    const ULONG         AlarmEnable
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTDisarmFramerAlarm(
    const BOARD         nBoard,
    const USHORT        nFramer,
    const ULONG         AlarmEnable
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetFramerAlarmArmStatus(
    const BOARD         nBoard,
    const USHORT        nFramer,
    const ULONG         AlarmEnable,
    ULONG * const       pAlarmArmed
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetFramerAlarmStatus(
    const BOARD         nBoard,
    const USHORT        nFramer,
    PMT_FRAMER_STATUS const pFramerAlarmStatus
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetT1FramerStatistic(
    const BOARD         nBoard,
    const USHORT        nFramer,
    PMT_T1_FRAMER_STAT const    pFramerStat,
    PULONG const        pLength
   );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetE1FramerStatistic(
    const BOARD         nBoard,
    const USHORT        nFramer,
    PMT_E1_FRAMER_STAT const    pFramerStat,
    PULONG const        pLength
   );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetNGXFramerStatistic(
    const BOARD                 nBoard,
    const USHORT                nFramer,
    PMT_NTTE_FRAMER_STAT const  pFramerStat,
    PULONG                      pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTClearFramerStatistic(
    const BOARD         nBoard,
    const USHORT        nFramer
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetFramerLoopbackMode(
    const BOARD         nBoard,
    const USHORT        nFramer,
    const USHORT        LoopbackMode        // FRAMER_NETWORK_PAYLOAD_LOOPBACK, and etc.
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetFramerLoopbackMode(
    const BOARD         nBoard,
    const USHORT        nFramer,
    const PUSHORT       pLoopbackMode       // Default FRAMER_LOOPBACK_NONE
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTResetFramerLoopbackMode(
    const BOARD         nBoard,
    const USHORT        nFramer
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTEnableFramerABCDSignalingEvent(
    const BOARD         nBoard,
    const USHORT        nFramer
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTDisableFramerABCDSignalingEvent(
    const BOARD         nBoard,
    const USHORT        nFramer
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetFramerABCDSignalingEventStatus(
    const BOARD         nBoard,
    const USHORT        nFramer,
    PUSHORT const       fEnabled            // 1 for enabled, 0 for idsabled
   );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetTimeSlotRxSignaling(
    const BOARD         nBoard,
    const USHORT        nFramer,
    const ULONG         nTimeSlot,          // 1 to 24 for T1, 0 to 31 for E1
    PMT_TS_SIGNALING const pSignaling
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetTimeSlotTxSignaling(
    const BOARD         nBoard,
    const USHORT        nFramer,
    const ULONG         nTimeSlot,          // 1 to 24 for T1, 0 to 31 for E1
    const USHORT        Signaling
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetTimeSlotTxSignaling(
    const BOARD         nBoard,
    const USHORT        nFramer,
    const ULONG         nTimeSlot,          // 1 to 24 for T1, 0 to 31 for E1
    PMT_TS_SIGNALING const pSignaling
    );

// Call Connection API on Terminate cards (T1 RBS, ISDN)

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCC_ConnectRequest(
    const CHANNEL               nChannel,
    PMT_CC_CONNECT_REQ const    pCallInfo
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCC_ConnectResp(
    const CHANNEL               nChannel,
    PMT_CC_CONNECT_RESP const   pCallInfo
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCC_DiscRequest(
    const CHANNEL               nChannel,
    PMT_CC_DISC_REQ const       pCallInfo
    );

// Call Connection API on Terminate cards (ISDN)

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCC_FacilityRequest(
    const CHANNEL               nChannel,
    PMT_CC_FACILITY_REQ const   pFacilityInfo
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCC_UserInfoRequest(
    const CHANNEL               nChannel,
    PMT_CC_USERINFO_REQ const   pUserInfo
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCC_SuspendRequest(
    const CHANNEL               nChannel,
    PMT_CC_SUSPEND_REQ const    pSuspendInfo
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCC_ResumeRequest(
    const CHANNEL               nChannel,
    PMT_CC_RESUME_REQ const     pResumeInfo
    );


// Call Connection API on DPT ISDN

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCC_GetStatusByCh(
    const CHANNEL               nChannel,
    PMT_CC_CALL_INFO const      pCallInfo,
    PULONG const                pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTCC_GetStatusByRef(
    const ULONG                 nCallRef,
    PMT_CC_CALL_INFO const      pCallInfo,
    PULONG const                pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetDataLinkStatus(
    const BOARD                 nBoard,
    const USHORT                nTrunk,
    PULONG const                pDLStatus
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetDataLinkStatistics(
    const BOARD                 nBoard,
    const USHORT                nTrunk,
    PMT_DL_STATS const          pDLStats,
    PULONG const                pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetCCStatistics(
    const BOARD                 nBoard,
    const USHORT                nTrunk,
    PMT_CC_STATS const          pCCStats,
    PULONG const                pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetPHYStatistics(
    const BOARD                 nBoard,
    const USHORT                nTrunk,
    PMT_PHY_STATS const         pPHYStats,
    PULONG const                pLength
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTSetOutputSource(
    const CHANNEL       nChannel,
    const USHORT        Source
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetOutputSource(
    const CHANNEL       nChannel,
    PUSHORT const       pSource
    );


//
// Trunk specific APIs
//
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTOffHook(
    CHANNEL nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTOnHook(
    CHANNEL nChannel
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTGetHookState(
    CHANNEL nChannel,
    const PUCHAR        pState          // 0 for onhook
    );

_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTWink(
    CHANNEL nChannel
    );


// Wave header utility

_DLLIMPEXPORT void CALLING_CONVENTION
MakeWaveGSMHeader(PGSM_WAVE_HEADER pWaveHeader, int BytesWritten);

_DLLIMPEXPORT void CALLING_CONVENTION
MakeWavePCMHeader(PPCM_WAVE_HEADER pWaveHeader, int BytesWritten, USHORT Format);


_DLLIMPEXPORT void CALLING_CONVENTION
MTReturnEventBuffer(
    PVOID               ptrBuffer
    );

_DLLIMPEXPORT void CALLING_CONVENTION
MTGetAdapterDescription(
    const ULONG         BoardType,      // VR6409, VR3200, etc
    const PCHAR         pDescription,   // Pointer for the NULL terminated description
    const int           nLength         // Length of the description buffer
    );

_DLLIMPEXPORT void CALLING_CONVENTION
MTGetAdapterStatusDescription(
    const ULONG         StatusCode,     // BOARD_OK, etc
    const PCHAR         pDescription,   // Pointer for the NULL terminated description
    const int           nLength         // Length of the description buffer
    );

_DLLIMPEXPORT void CALLING_CONVENTION
MTGetReturnCodeDescription(
    const MT_RESULT     ReturnCode,     // MT_RET_OK, etc
    const PCHAR         pDescription,   // Pointer for the NULL terminated description
    const int           nLength         // Length of the description buffer
    );

_DLLIMPEXPORT void CALLING_CONVENTION
MTGetEventCodeDescription(
    const ULONG         EventCode,      // EVT_MAXDIGITS, etc
    const PCHAR         pDescription,   // Pointer for the NULL terminated description
    const int           nLength         // Length of the description buffer
    );

///////////////////////////////////////////////////////////////////////////////
// IP Signaling Protocol Control and Configuration Functions
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Enable/configure a Signaling Protocol
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpEnableSignalingProtocol( 	
	const BOARD                  nBoard,      // Board index (starting at 0)
	const USHORT                 nProtocolID, // Protocol ID
   	PMT_IP_PROTOCOL_PARAMS const pParams,     // Protocol dependent parameter structure
	PULONG const                 pLength      // Pointer to size (in bytes) of a
                                              //  MT_IP_PROTOCOL_PARAMS structure
	);

//-----------------------------------------------------------------------------
// Disable a Signaling Protocol
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpDisableSignalingProtocol( 	
	const BOARD                  nBoard,      // Board index (starting at 0)
	const USHORT                 nProtocolID  // Protocol ID
	);

//-----------------------------------------------------------------------------
// Retrieve enable/disable status and parameters of a Signaling Protocol
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpSignalingProtocolStatus( 	
	const BOARD                  nBoard,      // Board index (starting at 0)
	const USHORT                 nProtocolID, // Protocol ID
  	BOOLEAN * const              pOn,         // TRUE = enabled, FALSE = disabled 
   	PMT_IP_PROTOCOL_PARAMS const pParams,     // Protocol dependent parameter structure
	PULONG const                 pLength      // Pointer to size (in bytes) of a
                                              //  MT_IP_PROTOCOL_PARAMS structure
	);

//-----------------------------------------------------------------------------
// Enable/Disable D-Channel event reporting
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpDChannelEventControl( 	
    const BOARD         nBoard,         // Board index (starting at 0)
    const USHORT        nProtocolID,    // Protocol ID
    const BOOLEAN       fOn             // TRUE = enabled, FALSE = disabled 
    );

//-----------------------------------------------------------------------------
// Retrieve enable/disable status of D-Channel event reporting
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpDChannelEventStatus( 	
    const BOARD        nBoard,         // Board index (starting at 0)
    const USHORT       nProtocolID,    // Protocol ID
    BOOLEAN * const    pOn             // TRUE = enabled, FALSE = disabled 
    );

//-----------------------------------------------------------------------------
// Enable/Disable D-Channel event filtering
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpDChannelEventFilteringControl( 	
    const BOARD         nBoard,         // Board index (starting at 0)
    const USHORT        nProtocolID,    // Protocol ID
    const BOOLEAN       fOn             // TRUE = enabled, FALSE = disabled 
    );

//-----------------------------------------------------------------------------
// Retrieve enable/disable status of D-Channel event filtering
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpDChannelEventFilteringStatus( 	
    const BOARD        nBoard,         // Board index (starting at 0)
    const USHORT       nProtocolID,    // Protocol ID
    BOOLEAN * const    pOn             // TRUE = enabled, FALSE = disabled 
    );

//-----------------------------------------------------------------------------
// Enable/Disable call control event reporting
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpCCEventControl( 	
    const BOARD        nBoard,         // Board index (starting at 0)
    const USHORT       nProtocolID,    // Protocol ID
    const BOOLEAN      fOn             // TRUE = enabled, FALSE = disabled 
    );

//-----------------------------------------------------------------------------
// Retrieve enable/disable status of call control event reporting
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpCCEventStatus( 	
    const BOARD        nBoard,         // Board index (starting at 0)
    const USHORT       nProtocolID,    // Protocol ID
    BOOLEAN * const    pOn             // TRUE = enabled, FALSE = disabled 
    );

///////////////////////////////////////////////////////////////////////////////
// IP Signaling Protocol Statistics Functions
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Gets performance counters for a signaling protocol
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpGetSignalingStatistics(
    const BOARD                  nBoard,      // Board Number
    const USHORT                 nProtocolID, // Protocol ID
    PMT_IP_SIGNALING_STATS const pStats,      // Pointer to a statistics struct
    PULONG const                 pLength      // Pointer to size (in bytes) of 
                                              //  statistics structure
    );

//-----------------------------------------------------------------------------
// Resets (clears) the performance counters of a signaling protocol
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpClearSignalingStatistics( 
    const BOARD        nBoard,         // Board index (starting at 0)
    const USHORT       nProtocolID     // Protocol ID
    );

///////////////////////////////////////////////////////////////////////////////
// Media Session Control Functions
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Establish media routing rules for new media sessions
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpSetMediaRoutingParams( 	
    const BOARD                 nBoard,  // Board index (starting at 0)
    const IP_FILTER_OPTS        fFilter, // Desired filter type
    const ULONG                 nCount,  // Number of elements contained in pParams
    PMT_IP_ROUTING_PARAMS const pParams, // Pointer to a MT_IP_ROUTING_PARAMS structure 
    PULONG const                pLength	 // Pointer to size (in bytes) of a
                                         //  MT_IP_ROUTING_PARAMS structure
    );

//-----------------------------------------------------------------------------
// Retrieve current media routing rules
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpGetMediaRoutingParams( 	
    const BOARD                 nBoard,  // Board index (starting at 0)
    const IP_FILTER_OPTS        fFilter, // Pointer to desired filter type
    const ULONG                 nCount,	 // Number of elements contained in pParams
    PMT_IP_ROUTING_PARAMS const pParams, // Pointer to a MT_IP_ROUTING_PARAMS structure
    PULONG const                pLength	 // Pointer to size (in bytes) of a
                                         //  MT_IP_ROUTING_PARAMS structure
    );

//-----------------------------------------------------------------------------
// Connect a media stream associated with a session to a processing resource
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpSetSessionMediaDest( 	
    const BOARD               nBoard,     // Board index (starting at 0)
    const ULONG               nSessionID, // Session ID
    PMT_IP_SESSION_DEST const pParams,    // Pointer to a MT_IP_SESSION_DEST structure
    PULONG const              pLength     // Pointer to size (in bytes) of a
	                                      //  MT_IP_SESSION_DEST structure
    );

//-----------------------------------------------------------------------------
// Retrieve the processing resource associated with a media session
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpGetSessionMediaDest( 	
    const BOARD               nBoard,     // Board index (starting at 0)
    const ULONG               nSessionID, // Session ID
    PMT_IP_SESSION_DEST const pParams,    // Pointer to a MT_IP_SESSION_DEST structure
    PULONG const              pLength     // Pointer to size (in bytes) of a
                                          //  MT_IP_SESSION_DEST structure
    );

//-----------------------------------------------------------------------------
// Create a media session with application supplied parameters
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpCreateMediaSession( 	
    const BOARD                 nBoard,    // Board index (starting at 0)
    const PMT_IP_SESSION_PARAMS pParams,   // Pointer to a session parameters structure
    PULONG const                pLength,   // Pointer to size (in bytes) of a
                                           //  MT_IP_SESSION_PARAMS structure
    PULONG const                pSessionID // A buffer for the session number
    );

//-----------------------------------------------------------------------------
// Remove a previously created media session
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpDeleteMediaSession( 
    const BOARD         nBoard,            // Board index (starting at 0)
    const ULONG         nSessionID         // Session ID
    );

//-----------------------------------------------------------------------------
// Retrieve all parameters associated with a media session
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpGetMediaSessionInfo( 	
    const BOARD                 nBoard,     // Board index (starting at 0)
    const ULONG                 nSessionID, // Session ID
    const PMT_IP_SESSION_PARAMS pParams,    // Pointer to a session parameters structure
    PULONG const                pLength     // Pointer to size (in bytes) of a
                                            //  MT_IP_SESSION_PARAMS structure
    );

///////////////////////////////////////////////////////////////////////////////
// Station Device Status and PHY Statistics
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Retrieves the current number (count) of known stations
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpGetStationCount( 	
    const BOARD                 nBoard,      // Board index (starting at 0)
    PULONG const                pCount       // Pointer to a ULONG to receive count
    );

//-----------------------------------------------------------------------------
// Retrieves a list of current known station devices and associated parameters
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpGetStationList( 	
    const BOARD                 nBoard,      // Board index (starting at 0)
    const PULONG                pCount,      // Number of array elements contain in pParams
    const PMT_IP_STATION        pParams,     // Pointer to an array of station params structures
    PULONG const                pLength      // Pointer to size (in bytes) of a
                                             //  MT_IP_STATION structure
    );

//-----------------------------------------------------------------------------
// Retrieves parameters associated with the specified station
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpGetStationParams( 	
	const BOARD					nBoard,		// Board index (starting at 0)
	const USHORT 				nStation,	// Station ID
    const PMT_IP_STATION_PARAMS pParams,    // Pointer to a station info structure
	PULONG const				pLength		// Pointer to size (in bytes) of a
											//  MT_IP_STATION_PARAMS structure
	);

//-----------------------------------------------------------------------------
// Retrieves statistics for the specified station
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpGetStationStatistics( 	
    const BOARD                 nBoard,      // Board index (starting at 0)
    const ULONG                 nStation,    // Station ID
    const PMT_IP_STATION_STATS  pStats,      // Pointer to a station status structure
    PULONG const                pLength      // Pointer to size (in bytes) of a
                                             //  MT_IP_STATION_STATS structure
    );

//-----------------------------------------------------------------------------
// Reset (or clear) the statistics for the specified station
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpClearStationStatistics( 
    const BOARD                 nBoard,      // Board index (starting at 0)
    const ULONG                 nStation     // Station ID
    );

//-----------------------------------------------------------------------------
// Retrieves statistics for the specified network interface
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpGetNetworkInterfaceStatistics( 	
const BOARD                     nBoard,     // Board index (starting at 0)
const USHORT                    nNetworkInterface, // On-board Network Interface
const PMT_IP_NET_STATS          pStats,     // Pointer to a station status structure
PULONG const                    pLength     // Pointer to size (in bytes) of a
                                            //  MT_IP_NET_STATS structure
    );

//-----------------------------------------------------------------------------
// Reset (or clear) the statistics for the specified network interface
//-----------------------------------------------------------------------------
_DLLIMPEXPORT MT_RESULT CALLING_CONVENTION
MTIpClearNetworkInterfaceStatistics( 
    const BOARD                nBoard,            // Board index (starting at 0)
    const USHORT               nNetworkInterface  // On-board Network Interface
    );		


#if _WIN32
#include <poppack.h>
#endif

#endif

// End of NTiAPI.h
