/*--

Copyright (c) 1997  NewVoice, Inc.

Module Name:

    nvmsg.mc

Abstract:

    This portion of the file contains error code definitions specific
    to nvdspdll.dll and the NVDSP driver.

--*/

//
//  Values are 32 bit values layed out as follows:
//
//   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
//  +---+-+-+-----------------------+-------------------------------+
//  |Sev|C|R|     Facility          |               Code            |
//  +---+-+-+-----------------------+-------------------------------+
//
//  where
//
//      Sev - is the severity code
//
//          00 - Success
//          01 - Informational
//          10 - Warning
//          11 - Error
//
//      C - is the Customer code flag
//
//      R - is a reserved bit
//
//      Facility - is the facility code
//
//      Code - is the facility's status code
//
//
//
// Define the severity codes
//
#define STATUS_SEVERITY_WARNING          0x2
#define STATUS_SEVERITY_SUCCESS          0x0
#define STATUS_SEVERITY_INFORMATIONAL    0x1
#define STATUS_SEVERITY_ERROR            0x3

//
// MessageId: NV_RET_NO_DIGIT
//
// MessageText:
//
//  No digit available%0
//
#define NV_RET_NO_DIGIT                  0xE0002002L

//
// MessageId: NV_RET_STARTED
//
// MessageText:
//
//  Voice system already started%0
//
#define NV_RET_STARTED                   0xE0002003L

//
// MessageId: NV_RET_NOT_STARTED
//
// MessageText:
//
//  Voice system not started%0
//
#define NV_RET_NOT_STARTED               0xE0002004L

//
// MessageId: NV_RET_MALFUNCTION
//
// MessageText:
//
//  Hardware failed during initialization%0
//
#define NV_RET_MALFUNCTION               0xE0002005L

//
// MessageId: NV_RET_INVALID_FUNC
//
// MessageText:
//
//  Function code is not valid%0
//
#define NV_RET_INVALID_FUNC              0xE0002006L

//
// MessageId: NV_RET_INVALID_HW_INT
//
// MessageText:
//
//  Bad hardware IRQ at initialization%0
//
#define NV_RET_INVALID_HW_INT            0xE0002007L

//
// MessageId: NV_RET_INVALID_CHANNEL
//
// MessageText:
//
//  Channel number is out of range%0
//
#define NV_RET_INVALID_CHANNEL           0xE0002009L

//
// MessageId: NV_RET_BUSY
//
// MessageText:
//
//  Channel is busy%0
//
#define NV_RET_BUSY                      0xE0002010L

//
// MessageId: NV_RET_IDLE
//
// MessageText:
//
//  Channel is idle%0
//
#define NV_RET_IDLE                      0xE0002011L

//
// MessageId: NV_RET_INVALID_PARAM
//
// MessageText:
//
//  Invalid argument passed to function%0
//
#define NV_RET_INVALID_PARAM             0xE0002012L

//
// MessageId: NV_RET_NOT_AVAILABLE
//
// MessageText:
//
//  Feature is not available%0
//
#define NV_RET_NOT_AVAILABLE             0xE0002014L

//
// MessageId: NV_RET_EXTENDED_ERROR
//
// MessageText:
//
//  Extended error%0
//
#define NV_RET_EXTENDED_ERROR            0xE0002024L

//
// MessageId: NV_RET_NO_MUX
//
// MessageText:
//
//  Multiplexer is not available%0
//
#define NV_RET_NO_MUX                    0xE0002026L

//
// MessageId: NV_RET_MUX_ON
//
// MessageText:
//
//  Multiplexer system was already started%0
//
#define NV_RET_MUX_ON                    0xE0002027L

//
// MessageId: NV_RET_MUX_OFF
//
// MessageText:
//
//  Multiplexer system was not started%0
//
#define NV_RET_MUX_OFF                   0xE0002028L

//
// MessageId: NV_RET_BAD_JOIN
//
// MessageText:
//
//  Invalid crosspoint coordinates%0
//
#define NV_RET_BAD_JOIN                  0xE0002029L

//
// MessageId: NV_RET_BAD_CURSOR
//
// MessageText:
//
//  Invalid index offset%0
//
#define NV_RET_BAD_CURSOR                0xE0002031L

//
// MessageId: NV_XRET_BADHANDLE
//
// MessageText:
//
//  Bad handle%0
//
#define NV_XRET_BADHANDLE                0xE0002114L

//
// MessageId: NV_XRET_QFULL
//
// MessageText:
//
//  Event queue is full (has reached maximum limit)%0
//
#define NV_XRET_QFULL                    0xE0002118L

//
// MessageId: NV_XRET_QEMPTY
//
// MessageText:
//
//  Event queue is empty%0
//
#define NV_XRET_QEMPTY                   0xE0002119L

//
// MessageId: NV_XRET_BADFUNCTION
//
// MessageText:
//
//  Bad function%0
//
#define NV_XRET_BADFUNCTION              0xE0002120L

//
// MessageId: NV_XRET_BADPARAM
//
// MessageText:
//
//  Invalid argument passed to function%0
//
#define NV_XRET_BADPARAM                 0xE0002121L

//
// MessageId: NV_RET_DEVICE_NOTFOUND
//
// MessageText:
//
//  Voice interface device was not found
//
#define NV_RET_DEVICE_NOTFOUND           0xE0002200L

//
// MessageId: NV_RET_INTERNAL_ERROR
//
// MessageText:
//
//  Internal error in nv.dll (%1!s!@%2!s!)%0
//
#define NV_RET_INTERNAL_ERROR            0xE0002201L

//
// MessageId: NV_RET_UNKNOWN_DEVICE_ERROR
//
// MessageText:
//
//  Unknown device error (%1!s!@%2!s!)%0
//
#define NV_RET_UNKNOWN_DEVICE_ERROR      0xE0002202L

//
// MessageId: NV_RET_SERVICE_NOT_STARTED
//
// MessageText:
//
//  Voice service could not be started%0
//
#define NV_RET_SERVICE_NOT_STARTED       0xE0002203L

//
// MessageId: NV_RET_CANNOT_CREATE_EVTQMUTEX
//
// MessageText:
//
//  Unable to create mutex for the event queue%0
//
#define NV_RET_CANNOT_CREATE_EVTQMUTEX   0xE0002204L

//
// MessageId: NV_RET_CANNOT_CREATE_EVTQEVENT
//
// MessageText:
//
//  Unable to create event for the event queue%0
//
#define NV_RET_CANNOT_CREATE_EVTQEVENT   0xE0002205L

//
// MessageId: NV_RET_NO_MEM
//
// MessageText:
//
//  An operation could not be completed due to insufficient memory or an error occurred while attempting to allocate memory%0
//
#define NV_RET_NO_MEM                    0xE0002206L

//
// MessageId: NV_RET_INVALID_EVENT_QUEUE
//
// MessageText:
//
//  Invalid event queue%0
//
#define NV_RET_INVALID_EVENT_QUEUE       0xE0002207L

//
// MessageId: NV_RET_EVTQMUTEX_ABANDONED
//
// MessageText:
//
//  The event queue synchronization mutex was abandoned by another thread%0
//
#define NV_RET_EVTQMUTEX_ABANDONED       0xE0002208L

//
// MessageId: NV_RET_NO_ASSOC_EVTQ
//
// MessageText:
//
//  No event queue has been associated with channel: %1!s!%0
//
#define NV_RET_NO_ASSOC_EVTQ             0xE0002209L

//
// MessageId: NV_RET_INVALID_FILEINDEX
//
// MessageText:
//
//  The NV_IO_CONTROL FileIndex field is out of range for the file%0
//
#define NV_RET_INVALID_FILEINDEX         0xE000220AL

//
// MessageId: NV_RET_ZERO_FILELENGTH
//
// MessageText:
//
//  The NV_IO_CONTROL FileLength field is zero%0
//
#define NV_RET_ZERO_FILELENGTH           0xE000220BL

//
// MessageId: NV_RET_COMPAT_NOTSUPPORTED
//
// MessageText:
//
//  Compatibility mode is not supported in this implementation%0
//
#define NV_RET_COMPAT_NOTSUPPORTED       0xE000220CL

//
// MessageId: NV_RET_UNKNOWN_IOTRANS_HANDLE
//
// MessageText:
//
//  The handle in the IO_TRANS structure was not obtained through the NewVoice API file open functions%0
//
#define NV_RET_UNKNOWN_IOTRANS_HANDLE    0xE000220DL

//
// MessageId: NV_RET_CANNOT_CREATE_MUTEX
//
// MessageText:
//
//  Mutex creation failed%0
//
#define NV_RET_CANNOT_CREATE_MUTEX       0xE000220EL

//
// MessageId: NV_EXCEPTION
//
// MessageText:
//
//  An exception occurred during the API call%0
//
#define NV_EXCEPTION                     0xE000220FL

//
// MessageId: NV_RET_INVALID_FILENAME
//
// MessageText:
//
//  Invalid filename%0
//
#define NV_RET_INVALID_FILENAME          0xE0002210L

//
// MessageId: NV_RET_VERSION_MISMATCH
//
// MessageText:
//
//  Version mismatch%0
//
#define NV_RET_VERSION_MISMATCH          0xE0002211L

//
// MessageId: NV_RET_MISSING_TERMINATOR
//
// MessageText:
//
//  A terminating entry was not found in an array argument.
//
#define NV_RET_MISSING_TERMINATOR        0xE0002212L


//
// MessageId: NV_RET_NO_FAXBD
//
// MessageText:
//
//  Fax Bandwith is not enough
//
#define NV_RET_NO_FAXBD					0xE0002213L


//
// MessageId: NV_RET_CMDBIT_IS_SET
//
// MessageText:
//
//  Command bit is set
//
#define NV_RET_CMDBIT_IS_SET			0xE0002214L


//
// MessageId: NV_RET_INTERRUPT_FAILED
//
// MessageText:
//
//  Interrupt did't occur
//
#define NV_RET_INTERRUPT_FAILED			0xE0002215L


//
// MessageId: NV_RET_INCOMPATIBLE_BUFFERSIZE
//
// MessageText:
//
//  Buffer size is not mutiple of 65 for MGSM610
//
#define NV_RET_INCOMPATIBLE_BUFFERSIZE			0xE0002216L


