#ifndef TFENGINE_COMMUNICATION_H
#define TFENGINE_COMMUNICATION_H

#include "..\TFServer\communication_hub.h"
#include "..\BoardDLLs\Board_communication.h"
#include "..\TFServer\ivr_defs.h"
#include "..\inet\Socket.h"

class TFEngineCommunicationPacket : public CommunicationPacket
{
   public:

      enum TF_ENGINE_EVENT
      {
         SRV_EVT_TERMINATE,         // Thread must terminate now.
         SRV_EVT_SD_NORMAL,         // Monitor requests shutdown when safe,
         SRV_EVT_SD_ABORT,          // Monitor requests Engine shutdown now.
         SRV_EVT_RELOAD_NORMAL,     // Monitor requests Engine cycle when safe.
         SRV_EVT_RELOAD_ABORT,      // Monitor requests Engine to cycle now.

         // Request for engine to start streaming
         SRV_EVT_START_STREAM,

         // Request for engine to stop streaming
         SRV_EVT_STOP_STREAM,

         // Request for engine to pause streaming
         SRV_EVT_PAUSE_STREAM,

         // Request for board, boardMessage must be a valid message
         SRV_EVT_BOARD_EVENT
      };
      
      // An appID of 0 indicates broadcast (all TFEngines)
      TFEngineCommunicationPacket(int appID, TF_ENGINE_EVENT event);

      // Application ID of the destination engine
      int appID;
      TF_ENGINE_EVENT event;

      // Pointer to identify TFTCPClient for sending SRV_EVT_START_STREAM, SRV_EVT_STOP_STREAM,
      // or SRV_EVT_PAUSE_STREAM messages
      TCPSocket* client;

      // Format requested by a SRV_EVT_START_STREAM event
      SOUND_FORMAT format;

      // TCP/IP address of client making SRV_EVT_START_STREAM, SRV_EVT_STOP_STREAM,
      // or SRV_EVT_PAUSE_STREAM request
      ACE_INET_Addr destination;

      BoardCommunicationPacket* boardMessage;
};

// Message destination for engine
// Note that messages must be TFEngineCommunicationPackets
// Messages for engines must use TF_ENGINE_MANAGER as their destination
// Messages with a appID of 0 indicate a broadcast to all engines
class TFEngineCommunicationDestination : public CommunicationDestination<int>
{
   public:

      TFEngineCommunicationDestination(int appID);

      // Identified by engine application ID
      int ID(void);

   protected:
   
      int appID;
};

#endif