#include "VCE_List.h"
#include "step.h"
#include "steps.h"

Step::Step(STEPREC& stepRecord) : Link()
{
   cStep = stepRecord.current;
   nStep = stepRecord.next;
   failure = tStep = fStep = thStep = timeout = busy = 0;
}

Step::~Step(void)
{
}

Step* Step::Create(STEPREC& stepRecord)
{
   switch(stepRecord.op)
   {
      case TTCLEARBUF:               return(new StepFlushDigits(stepRecord));
      case VOXBEEP:                  return(new StepBeep(stepRecord));
      case SQLBEGINTR:               return(new StepSQLBeginTrans(stepRecord));
      case SQLDISCON:                return(new StepSQLDisconnect(stepRecord));
      case FAXQUEUECLR:              return(new StepFaxClear(stepRecord));
      case FAXQUEUEADD:              return(new StepFaxAdd(stepRecord));
      case FAXRECEIVE:               return(new StepFaxReceive(stepRecord));
      case FAXSEND:                  return(new StepFaxSend(stepRecord));
      case PROCEND:                  return(new StepProcEnd(stepRecord));
      case PROCSTART:                return(new StepProcStart(stepRecord));
      case VOXVOLUME:                return(new StepVolume(stepRecord));
      case VOXSPEAK:                 return(new StepAudioSpeak(stepRecord));
      case SETGLOBAL:                return(new StepSetGlobal(stepRecord));
      case SETLANGUAGE:              return(new StepSetLanguage(stepRecord));
      case SIPREGISTER:              return(new SIPRegister(stepRecord));
      case SETVOIPMEDIA:             return(new SetVoIPMedia(stepRecord));
      case SQLBEGINST:               return(new StepSQLBeginStmt(stepRecord));
      case SQLLOCK:                  return(new StepSQLLock(stepRecord));
      case SQLENDST:                 return(new StepSQLEndStmt(stepRecord));
      case SQLENDTR:                 return(new StepSQLEndTrans(stepRecord));
      case GOTOLABEL:                return(new StepAppGoto(stepRecord));
      case RUNBASICSCRIPT:           return(new StepRunBASICScript(stepRecord));
      case RUNPROGRAM:               return(new StepAppExec(stepRecord));
      case FLASH:                    return(new StepFlashHook(stepRecord));
      case TONE:                     return(new StepGenerateTones(stepRecord));
      case SETREORDERTONE:           return(new StepSetReorderTone(stepRecord));
      case WAIT:                     return(new StepWait(stepRecord));
      case RELOADON:                 return(new StepReloadOn(stepRecord));
      case RELOADOFF:                return(new StepReloadOff(stepRecord));
      case SRBARGEINON:              return(new StepSRBargeInOn(stepRecord));
      case SRBARGEINOFF:             return(new StepSRBargeInOff(stepRecord));
      case CREATEDIR:                return(new StepCreateDir(stepRecord));
      case FILEOPEN:                 return(new StepFileOpen(stepRecord));
      case FILEWRITE:                return(new StepFileWrite(stepRecord));
      case BLOCKCALL:                return(new StepBlockCalls(stepRecord));
      case UNBLOCKCALL:              return(new StepUnblockCalls(stepRecord));
      case WAITFORCALL:              return(new StepWaitForCall(stepRecord));
      case PLACECALL:                return(new StepPlaceCall(stepRecord));
      case TRANSFERCALL:             return(new StepTransferCall(stepRecord));
      case SWITCH:                   return(new StepSwitch(stepRecord));
      case SWITCHDISCON:             return(new StepSwitchDiscon(stepRecord));
      case TTPENDING:                return(new StepDigitPending(stepRecord));
      case VOXDEL:                   return(new StepAudioDelete(stepRecord));
      case VOXPLAY:                  return(new StepAudioPlay(stepRecord));
      case VOXCOPY:                  return(new StepAudioCopy(stepRecord));
      case REORDERTONEDETECTIONOFF:  return(new ReorderToneDetectionOff(stepRecord));
      case REORDERTONEDETECTIONON:   return(new ReorderToneDetectionOn(stepRecord));
      case TEXTSPEECH:               return(new StepTextSpeech(stepRecord));
      case VOXVARSET:                return(new StepVoxVarSet(stepRecord));
      case VARSET:                   return(new StepVarSet(stepRecord));
      case VARSETPRIV:               return(new StepVarSetPrivate(stepRecord));
      case REGISTRYREAD:             return(new StepRegistryRead(stepRecord));
      case VARRANDOM:                return(new StepVarRandom(stepRecord));
      case SQLCONNECT:               return(new StepSQLConnect(stepRecord));
      case SQLFETCH:                 return(new StepSQLFetch(stepRecord));
      case MESSAGESTEP:              return(new StepMessage(stepRecord));
      case SAVELOG:                  return(new StepSaveLog(stepRecord));
      case RUNFLOWCHART:             return(new StepAppCall(stepRecord));
      case RUNTAP:                   return(new StepRunTAP(stepRecord));
      case SCANDIR:                  return(new StepScanDir(stepRecord));
      case SENDEMAIL:                return(new StepSendEmail(stepRecord));
      case VOCRECGETDIGITS:          return(new StepVocRecGetDigits(stepRecord));
      case VOCRECGETYN:              return(new StepVocRecGetYN(stepRecord));
      case VOCRECGETSTRING:          return(new StepVocRecGetString(stepRecord));
      case VOCPASSREG:               return(new StepVocPassReg(stepRecord));
      case VOCPASSAUTH:              return(new StepVocPassAuth(stepRecord));
      case SRCLEARAUDIO:             return(new StepSRClearAudio(stepRecord));
      case SRFLAGRESULT:             return(new StepSRFlagResult(stepRecord));
      case MENU:                     return(new StepGetDigit(stepRecord));
      case TTGETLETTER:              return(new StepGetLetter(stepRecord));
      case TTGETSTRING:              return(new StepGetDigits(stepRecord));
      case VARCMP:                   return(new StepVarCompare(stepRecord));
      case CASE:                     return(new StepCase(stepRecord));
      case VARMATH:                  return(new StepVarMath(stepRecord));
      case VARDATEINFO:              return(new StepVarDateInfo(stepRecord));
      case VOCRECINIT:               return(new StepVocRecInit(stepRecord));
      case VOCRECRELEASE:            return(new StepVocRecRelease(stepRecord));
      case HANG_UP:                  return(new StepHangup(stepRecord));
   	case UDPIPBIND:                return(new StepUDPIPBind(stepRecord));
   	case UDPIPCLOSE:               return(new StepUDPIPClose(stepRecord));
		case UDPIPSEND:                return(new StepUDPIPSend(stepRecord));
   	case UDPIPRECEIVE:             return(new StepUDPIPReceive(stepRecord));
      case PARSE:                    return(new StepParse(stepRecord));
      case VOXREC:                   return(new StepAudioRecord(stepRecord));
      case DICTATE:                  return(new StepDictate(stepRecord));
      case STARTRECORD:              return(new StepAudioStartRecord(stepRecord));
      case STOPRECORD:               return(new StepAudioStopRecord(stepRecord));
      case CALLTRIGGER:              return(new StepCallTrigger(stepRecord));
      case FATALSTEP:                return(new StepFatal(stepRecord));
   	case TCPIPCONNECT:             return(new StepTCPIPConnect(stepRecord));
   	case TCPIPDISCONNECT:          return(new StepTCPIPDisconnect(stepRecord));
		case TCPIPSEND:                return(new StepTCPIPSend(stepRecord));
   	case TCPIPRECEIVE:             return(new StepTCPIPReceive(stepRecord));
   	case HTTP:                     return(new StepHTTP(stepRecord));
      case SIMPULLREQUEST:           return(new StepSimpullXMLReq(stepRecord));
      case SIMPULLPARMS:             return(new StepSimpullParms(stepRecord));
      case XMLLOAD:                  return(new StepXMLLoad(stepRecord));
      case XMLEND:                   return(new StepXMLEnd(stepRecord));
      case XMLWRITE:                 return(new StepXMLWrite(stepRecord));
      case XMLFETCH:                 return(new StepXMLFetch(stepRecord));
      case DLLFUNCTION:              return(new StepDLLFunction(stepRecord));
      case HASH:                     return(new StepHash(stepRecord));
      case ENCRYPT:                  return(new StepEncrypt(stepRecord));
      case DECRYPT:                  return(new StepDecrypt(stepRecord));
      case PICKUP:                   return(new StepPickup(stepRecord));
      case FINCALC:                  return(new StepFinCalc(stepRecord));
      case CONFOPEN:                 return(new StepConfOpen(stepRecord));
      case CONFCLOSE:                return(new StepConfClose(stepRecord));
      case CONFJOIN:                 return(new StepConfJoin(stepRecord));
      case CONFLEAVE:                return(new StepConfLeave(stepRecord));
      case PERMITSTREAMING:          return(new StepPermitStreaming(stepRecord));
      case SETBOARDPARM:             return(new StepSetBoardParameter(stepRecord));
      case LABEL:
      case COMMENTS:
      case SPACEPOINT:               return(new StepNoOp(stepRecord));
      default:                       return(NULL);
   }
}

const char* Step::GetLabel(void) const
{
   return(label.c_str());
}

// -------------------------------------------------------------------------
//   Name:        Step::GetNextLabel
//
//   Purpose:     The 'nextLabel' field is populated by a derived class and is
//                used for processing steps out of sequence.
// -------------------------------------------------------------------------
const char* Step::GetNextLabel(void) const
{
   return(nextLabel.c_str());
}

const char* Step::GetNextTAMName(void) const
{
   return(nextTAMName.c_str());
}

void Step::SetLabel(const char* s)
{
   label = s == NULL ? "" : s;
}

int Step::GetCurrentStep(void) const
{
   return(cStep);
}

int Step::GetNextStep(void) const
{
   return(nStep);
}

int Step::GetFailureStep(void) const
{
   return(failure);
}

int Step::GetTrueStep(void) const
{
   return(tStep);
}

int Step::GetFalseStep(void) const
{
   return(fStep);
}

int Step::GetThresholdStep(void) const
{
   return(thStep);
}

int Step::GetTimeoutStep(void) const
{
   return(timeout);
}

int Step::GetNoVoiceStep(void) const
{
   return(noVoice);
}

int Step::GetSilenceStep(void) const
{
   return(silence);
}

int Step::GetBusyStep(void) const
{
   return(busy);
}

void Step::SetNextStep(int stepID)
{
   nStep = stepID;
}

void Step::SetCurrentStep(int stepID)
{
   cStep = stepID;
}

void Step::SetFailureStep(int stepID)
{
   failure = stepID;
}

void Step::SetTrueStep(int stepID)
{
   tStep = stepID;
}

void Step::SetFalseStep(int stepID)
{
   fStep = stepID;
}

void Step::SetThresholdStep(int stepID)
{
   thStep = stepID;
}

void Step::SetTimeoutStep(int stepID)
{
   timeout = stepID;
}

void Step::SetNoVoiceStep(int stepID)
{
   noVoice = stepID;
}

void Step::SetSilenceStep(int stepID)
{
   silence = stepID;
}

void Step::SetBusyStep(int stepID)
{
   busy = stepID;
}

/* -------------------------------------------------------------------------
   Function:    Step::SetString

   Notes:       - This could be put into a generic string handling file.
   ------------------------------------------------------------------------- */
IV_STATUS Step::SetString (const char *src, char **dest)
{
   if (!dest)
      return IVR_FAIL;

   if (*dest) {
      delete [] *dest;
      *dest = 0;
   }

   if (src && *src) {
      *dest = new char [strlen (src) + 1];
      strcpy (*dest, src);
   }

   return IVR_SUCCESS;
}

void Step::SetNextTAMName(const char* value)
{
   nextTAMName = value == NULL ? "" : value;
}

void Step::SetNextLabel(const char* value)
{
   nextLabel = value == NULL ? "" : value;
}

IV_STATUS Step::Status(PIVRINFO pInfo) const
{
   if(pInfo == NULL)
   {
      return(IVR_FAIL);
   }

   if(pInfo->ivBoard == NULL)
   {
      pInfo->ivLog->Print("      ERROR: Board not initialised\n");
      return(IVR_FAIL);
   }

   return(IVR_SUCCESS);
}

IV_STATUS Step::PortRequired(PIVRINFO pInfo) const
{
	if(pInfo->iPort < 1)
   {
      pInfo->ivLog->Print("      ERROR: %s requires a port\n", GetStepName());
      return(IVR_FATAL);
   }

   return(IVR_SUCCESS);
}

IV_STATUS Step::GetScriptCodeFileNameAndFormat(PIVRINFO pInfo, const char* audioCode, char*& fileName, SOUND_FORMAT& format) const
{
   const char* scriptCodeFileName;

   if(pInfo->hSimulator != NULL)
   {
      scriptCodeFileName = pInfo->voiceList->GetSimulatorFileName(audioCode);
   }
   else
   {
      scriptCodeFileName = pInfo->voiceList->GetTelephonyFileName(audioCode);
   }

   if(scriptCodeFileName == NULL)
   {
      pInfo->ivLog->Print("      ERROR: Script code '%s' is not in the Voice Script Manager\n", audioCode);
      return(IVR_FATAL);
   }

   pInfo->ivLog->Print("      Script code '%s' corresponds to file '%s'\n", audioCode, scriptCodeFileName);

   if((fileName = pInfo->variables->Parse(scriptCodeFileName)) == NULL)
   {
      pInfo->ivLog->Print("      ERROR: '%s' is undefined\n", scriptCodeFileName);
      return(IVR_FATAL);
   }

   if(strcmp(fileName, scriptCodeFileName) != 0)
   {
      pInfo->ivLog->Print("      File '%s' evaluates to file '%s'\n", scriptCodeFileName, fileName);
   }

   format = (pInfo->hSimulator != NULL) ? TF_WAV_8KHZ_16BIT : pInfo->voiceList->GetFormat(audioCode);

   return(IVR_SUCCESS);
}

Steps::Steps(void) : Head()
{
}

Steps::~Steps(void)
{
}

Step* Steps::FindStep(int stepNo)
{
   Step* temp;

   temp = (Step*) First();

   while(temp != NULL)
   {
      if(temp->GetCurrentStep() == stepNo)
      {
         return(temp);
      }
      temp = (Step*) temp->Suc();
   }

   return(NULL);
}
