#include "..\BoardDLLs\brd_generic.h"
#include "step_SetBoardParameter.h"

StepSetBoardParameter::StepSetBoardParameter(STEPREC& stepRecord) : Step(stepRecord)
{
   parameterName = stepRecord.s1.arr;
   parameterValue = stepRecord.s2.arr;
}

StepSetBoardParameter::~StepSetBoardParameter(void)
{
}

const char* StepSetBoardParameter::GetStepName(void) const
{
	return("Set HW Parameter");
}

IV_STATUS StepSetBoardParameter::Execute(PIVRINFO pInfo)
{
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   string parsedParameterName;
   int iParameterValue;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((value = pInfo->variables->ParseParameter("Parameter name", parameterName.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedParameterName = value;
      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("Parameter value", parameterValue.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iParameterValue = atoi(value);
      delete[] value;
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }
   
   pInfo->ivLog->Print("      Set hardware parameter %s to %i\n", parsedParameterName.c_str(), iParameterValue);

   return(pInfo->ivBoard->SetParameter(parsedParameterName.c_str(), iParameterValue));
}
// StepSetBoardParameter::Execute

