#include "..\inet\Socket.h"
#include "..\escutil\iv_str.h"        // HexStr2CharStr
#include "step_TCPIPSend.h"

StepTCPIPSend::StepTCPIPSend(STEPREC& stepRecord) : Step(stepRecord)
{
   socketHandle = stepRecord.s1.arr;
   message = stepRecord.s2.arr;
   hexString = (bool) stepRecord.u2;
}

StepTCPIPSend::~StepTCPIPSend(void)
{
}

const char* StepTCPIPSend::GetStepName(void) const
{
	return("TCP Send");
}

IV_STATUS StepTCPIPSend::Execute(PIVRINFO pInfo)
{
   IV_STATUS status = IVR_SUCCESS;
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   TCPSocket* socket;
   unsigned int stringLength;
   int ret;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((value = pInfo->variables->ParseParameter("Socket handle", socketHandle.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      ACE_HANDLE handle = (ACE_HANDLE) atoi(value);
      delete[] value;

      if((socket = TCPSocket::GetSocket(handle)) == NULL)
      {
         pInfo->ivLog->Print("      ERROR: '%d' is not a valid socket handle\n", handle);
         bParmsValid = false;
      }
   }

   if((value = pInfo->variables->ParseParameter("Message", message.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else if(((stringLength = strlen(value)) > 0) && hexString)
   {
      stringLength /= 2;
      HexStr2CharStr(value, (unsigned char*) value, stringLength);
   }

   if(!bParmsValid)
   {
      delete[] value;
      return(IVR_FATAL);
   }

   if(stringLength == 0)
   {
      pInfo->ivLog->Print("      WARNING: Empty message will not be sent\n");
   }
   else
   {
      pInfo->ivLog->Print("      Sending message: '%s'\n", value);

      if((ret = socket->SendBytes(value, stringLength)) != Socket::SUCCESS)
      {
         if(ret == WSAENOTCONN)
         {
            pInfo->ivLog->Print("      ERROR: Socket not connected\n");
         }
         else
         {
            pInfo->ivLog->Print("      ERROR: Failed to send message\n");
            pInfo->ivLog->Print("         %s\n", Socket::WSAErrorString(ret));
         }

         status = IVR_FAIL;
      }
   }

   delete[] value;

   return(status);
 }
