#include "step_simpullparms.h"

StepSimpullParms::StepSimpullParms(STEPREC& stepRecord) : Step(stepRecord)
{
   PID = stepRecord.s1.arr;
   UID = stepRecord.s2.arr;
   secret = stepRecord.s3.arr;
   machine = stepRecord.s4.arr;
   defaultTimeout = stepRecord.s5.arr;
   defaultRetries = stepRecord.s6.arr;
}

StepSimpullParms::~StepSimpullParms(void)
{
}

const char* StepSimpullParms::GetStepName(void) const
{
	return("Simpull Parameters");
}

// -------------------------------------------------------------------------
//   Name:        StepSimpullParms::Execute
//
//   Purpose:     Set global parameters required by Simpull XML API
//                Values are assigned into Global TeleFlow variables:
//                @SIMPULL_*
// ------------------------------------------------------------------------- 
IV_STATUS StepSimpullParms::Execute(PIVRINFO pInfo)
{
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   char* value;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   // --- PID:
   if(!PID.empty())
   {
      if((value = pInfo->variables->ParseParameter("PID", PID.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         bParmsValid &= pInfo->variables->Set("@SIMPULL_PID", value);
         delete[] value;
      }
   }
   else if(!pInfo->variables->IsVariable("@SIMPULL_PID"))
   {
      if(!pInfo->variables->Set("@SIMPULL_PID", "engenic"))
      {
         pInfo->ivLog->Print("      ERROR: Unable to intialize global variable '@SIMPULL_PID'\n");
         bParmsValid = false;
      }
   }

   // --- UID:
   if(!UID.empty())
   {
      if((value = pInfo->variables->ParseParameter("UID", UID.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         bParmsValid &= pInfo->variables->Set("@SIMPULL_UID", value);
         delete[] value;
      }
   }
   else if(!pInfo->variables->IsVariable("@SIMPULL_UID"))
   {
      if(!pInfo->variables->Set("@SIMPULL_UID", "0"))
      {
         pInfo->ivLog->Print("      ERROR: Unable to intialize global variable '@SIMPULL_UID'\n");
         bParmsValid = false;
      }
   }

   // --- SECRET:
   if(!secret.empty())
   {
      if((value = pInfo->variables->ParseParameter("Secret", secret.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         bParmsValid &= pInfo->variables->Set("@SIMPULL_SECRET", value);
         delete[] value;
      }
   }
   else if(!pInfo->variables->IsVariable("@SIMPULL_SECRET"))
   {
      if(!pInfo->variables->Set("@SIMPULL_SECRET", "N/A"))
      {
         pInfo->ivLog->Print("      ERROR: Unable to intialize global variable '@SIMPULL_SECRET'\n");
         bParmsValid = false;
      }
   }

   // --- Machine:
   if(!machine.empty())
   {
      if((value = pInfo->variables->ParseParameter("Machine", machine.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         bParmsValid &= pInfo->variables->Set("@SIMPULL_MACHINE", value);
         delete[] value;
      }
   }
   else if(!pInfo->variables->IsVariable("@SIMPULL_MACHINE"))
   {
      if(!pInfo->variables->Set("@SIMPULL_MACHINE", ""))
      {
         pInfo->ivLog->Print("      ERROR: Unable to intialize global variable '@SIMPULL_MACHINE'\n");
         bParmsValid = false;
      }
   }

   if(!defaultTimeout.empty())
   {
      if((value = pInfo->variables->ParseParameter("Default initial connect timeout", defaultTimeout.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         bParmsValid &= pInfo->variables->Set("@SIMPULL_CONTIMEOUT", value);
         delete[] value;
      }
   }
   else if(!pInfo->variables->IsVariable("@SIMPULL_CONTIMEOUT"))
   {
      if(!pInfo->variables->Set("@SIMPULL_CONTIMEOUT", "500"))
      {
         pInfo->ivLog->Print("      ERROR: Unable to intialize global variable '@SIMPULL_CONTIMEOUT'\n");
         bParmsValid = false;
      }
   }

   // --- Default Initial Connection Retries:
   if(!defaultRetries.empty())
   {
      if((value = pInfo->variables->ParseParameter("Default initial connect retries", defaultRetries.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         bParmsValid &= pInfo->variables->Set("@SIMPULL_CONRETRIES", value);
         delete[] value;
      }
   }
   else if(!pInfo->variables->IsVariable("@SIMPULL_CONRETRIES"))
   {
      if(!pInfo->variables->Set("@SIMPULL_CONRETRIES", "2"))
      {
         pInfo->ivLog->Print("      ERROR: Unable to intialize global variable '@SIMPULL_CONRETRIES'\n");
         bParmsValid = false;
      }
   }

   // --- Done:
   return(bParmsValid ? IVR_SUCCESS : IVR_FATAL);
}
