#include <stdlib.h>             // Needed for "atoi".
#include "..\escutil\odbc_db.h"
#include "step_sqlfetch.h"

StepSQLFetch::StepSQLFetch(STEPREC& stepRecord) : Step(stepRecord)
{
   sqlHandle = stepRecord.s1.arr;
   odbcConnection = stepRecord.s2.arr;
}

StepSQLFetch::~StepSQLFetch(void)
{
}

const char* StepSQLFetch::GetStepName(void) const
{
	return("SQL Fetch");
}

IV_STATUS StepSQLFetch::Execute(PIVRINFO pInfo)
{
   char* valueConnectHandle = NULL;
   char* valueStmtHandle = NULL;
   SQL* stmt;
   SQLList *connection;
   IV_STATUS status;
   bool bParmsValid = true;         // Evaluate all parameters before exiting

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if(pInfo->odbcManager == NULL)
   {
      pInfo->ivLog->Print("      ERROR: SQL not initialized.\n");
      return(IVR_FATAL);
   }

   if((valueStmtHandle = pInfo->variables->ParseParameter("Statement Handle", sqlHandle.c_str())) == NULL)
   {
      bParmsValid = false;
   }

   // Parse ODBC connection handle
   if((valueConnectHandle = pInfo->variables->ParseParameter("ODBC Connection", odbcConnection.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      if(strcmp(valueConnectHandle,"") == 0)
      {
         pInfo->ivLog->Print("      Using default ODBC handle\n");
      }

      // Find ODBC connection by its handle
      if((connection = pInfo->odbcManager->FindConnection(valueConnectHandle)) == NULL)
      {
         pInfo->ivLog->Print("      ERROR: No current connection matching handle '%s'\n", valueConnectHandle);
         bParmsValid = false;
      }
   }

   if(!bParmsValid)
   {
      delete[] valueConnectHandle;  // safe to delete, pointer was initialized to NULL
      delete[] valueStmtHandle;     //   "
      return(IVR_FATAL);
   }

   // --- Parameters validated - execution begins here ---

   if((stmt = connection->FindSQL(valueStmtHandle)) == NULL)
   {
      pInfo->ivLog->Print("      ERROR: No SQL statement defined\n");
      status = IVR_FATAL;
   }
   else
   {
      status = stmt->Fetch(pInfo);
   }

   delete[] valueConnectHandle;
   delete[] valueStmtHandle;

   return(status);
}
