#include <stdio.h>             // Needed for file open, read & write.
#include <stdlib.h>             // Needed for "atoi".
#include "step_textspeech.h"
#include "..\BoardDLLs\brd_generic.h"

StepTextSpeech::StepTextSpeech(STEPREC& stepRecord) : Step(stepRecord)
{
   bInt = (bool) stepRecord.u1;
   text = stepRecord.s6.arr;
   serverIP = stepRecord.s1.arr;
   serverIPPort = stepRecord.s2.arr;
}

StepTextSpeech::~StepTextSpeech(void)
{
}

const char* StepTextSpeech::GetStepName(void) const
{
	return("Speak Text");
}

IV_STATUS StepTextSpeech::Execute(PIVRINFO pInfo)
{
   IV_STATUS status = IVR_SUCCESS;	// status returned from this module
   string    parsedText;						// parsed version of text to be converted
   string    parsedServerIP;					// parsed version of TTS server IP
	char*     value;			   		// temporary string for conversion purposes
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
	int       serverIPport;				// parsed version of TTS server IP port

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

	// Parse server IP field
   if((value = pInfo->variables->ParseParameter("TTS server", serverIP.c_str(), "127.0.0.1")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
	   parsedServerIP = value;
      delete[] value;
   }

	// Parse server IP port field
   if((value = pInfo->variables->ParseParameter("TTS server port", serverIPPort.c_str(), "7000")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      serverIPport = atoi(value);
      delete[] value;

   	if(serverIPport <= 0)
   	{
	   	pInfo->ivLog->Print("      ERROR: Invalid IP port!\n");
         bParmsValid = false;
   	}
   }

	// Parse text
   if((value = pInfo->variables->ParseParameter("Text", text.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedText = value;
      delete[] value;
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   if(pInfo->portLicense == UNLICENSED)
   {
      pInfo->ivLog->Print ("      THIS PORT IS UNLICENSED\n");
      status = IVR_NOTSUPPORTED;
   }

   if(status == IVR_SUCCESS)
   {
   	status = pInfo->ivBoard->TextToSpeech(parsedServerIP.c_str(),
                                            serverIPport,
                                            parsedText.c_str(),
                                            bInt,
                                            atoi(pInfo->variables->Get("@VOLUME")));
   }

   return(status);
}
//StepTextSpeech::Execute

