#include "step_varcompare.h"

StepVarCompare::StepVarCompare(STEPREC& stepRecord) : Step(stepRecord)
{
   varA = stepRecord.s1.arr;
   varB = stepRecord.s3.arr;
   switch(stepRecord.u1)
   {
      case 1: iFn = FN_EQ;    break;
      case 2: iFn = FN_NE;    break;
      case 3: iFn = FN_GT;    break;
      case 4: iFn = FN_LT;    break;
      case 5: iFn = FN_GE;    break;
      case 6: iFn = FN_LE;    break;
      case 7: iFn = FN_NULL;  break;

      case 8:
      {
         iFn = FN_VALID;

         if(strcmp(stepRecord.s2.arr, "Integer") == 0)
         {
            iType = VAR_INTEGER;
         }
         else if(strcmp(stepRecord.s2.arr, "Floating point") == 0)
         {
            iType = VAR_FLOAT;
         }
         else if(strcmp(stepRecord.s2.arr, "Date") == 0)
         {
            iType = VAR_DATE;
         }
         else if(strcmp(stepRecord.s2.arr, "Time") == 0)
         {
            iType = VAR_TIME;
         }

         break;
      }

      case 9: iFn = FN_LUHN;  break;

      default:
      {
//@@@ Should be invalid compare type      
      }
   }
}

StepVarCompare::~StepVarCompare(void)
{
}

const char* StepVarCompare::GetStepName(void) const
{
	return("Compare");
}

IV_STATUS StepVarCompare::Execute (PIVRINFO pInfo)
{
   IV_STATUS status;
   bool rslt;
   char* valueA;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((valueA = pInfo->variables->ParseParameter("First Value", varA.c_str(), "")) == NULL)
   {
      if((iFn == FN_NULL) || (iFn == FN_VALID) || (iFn == FN_LUHN))
      {
         return(IVR_FATAL);
      }
      
      bParmsValid = false;
   }

   switch(iFn)
   {
      case FN_NULL:
      {
         pInfo->ivLog->Print ("         %s is empty?\n", varA.c_str());
         pInfo->ivLog->Print ("         (%s):%s is empty?\n", VarTypeString(DetermineType(valueA)), valueA);
         // status = VarCmp(pInfo, valueA, "", iFn, rslt); 2003-05-21 JSH pInfo no longer used
         status = VarCmp(valueA, "", iFn, rslt);
         break;
      }

      case FN_VALID:
      {
         pInfo->ivLog->Print ("         %s is valid?\n", varA.c_str());
         pInfo->ivLog->Print ("         (%s):%s is valid?\n", VarTypeString(DetermineType(valueA)), valueA);
         status = VarValid(pInfo, valueA, iType, rslt);
         break;
      }

      case FN_LUHN:
      {
         pInfo->ivLog->Print ("         %s is valid LUHN number?\n", varA.c_str());
         pInfo->ivLog->Print ("         (%s):%s is valid LUHN number?\n", VarTypeString(DetermineType(valueA)), valueA);
         status = VarLUHN(pInfo, valueA, rslt);
         break;
      }

      default:
      {
         char* valueB;
         const char* fnString;

         if((valueB = pInfo->variables->ParseParameter("Second Value", varB.c_str(), "")) == NULL)
         {
            bParmsValid = false;
         }

         if(!bParmsValid)
         {
            delete[] valueA;
            return(IVR_FATAL);
         }

         switch(iFn)
         {
            case FN_EQ: fnString = "="; break;
            case FN_NE: fnString = "!="; break;
            case FN_LT: fnString =  "<"; break;
            case FN_GT: fnString = ">"; break;
            case FN_LE: fnString = "<="; break;
            case FN_GE: fnString = ">="; break;
         }

         pInfo->ivLog->Print ("         %s %s %s?\n", varA.c_str(), fnString, varB.c_str());
         pInfo->ivLog->Print ("         (%s):%s %s (%s):%s?\n", VarTypeString(DetermineType(valueA)), valueA,
                                                fnString, VarTypeString(DetermineType(valueB)), valueB);
         // status = VarCmp(pInfo, valueA, valueB, iFn, rslt);  2003-05-21 JSH: pInfo no longer req'd
         status = VarCmp(valueA, valueB, iFn, rslt);

         delete[] valueB;
      }
   }

   delete[] valueA;

   if(status == IVR_SUCCESS)
   {
      return(rslt ? IVR_TRUE : IVR_FALSE);
   }

   return(status);
}
// StepVarCompare::Execute
