// -------------------------------------------------------------------------
//   Notes:       - It would be better if when loading TAMs the application
//                  searched for the file using a path environment variable.
// -------------------------------------------------------------------------
#include <string>
#include <string.h>
#include <dir.h>
#include "tam.h"
#include "step_tones.h"
#include "txtfile.h"

/* -------------------------------------------------------------------------
   Name:        TAM::TAM                                       (constructor)
   ------------------------------------------------------------------------- */
TAM::TAM (Log *outLog, bool logDisabled, const char *procName, int portnum) : Link()
{
   char   szFile[MAXFILE];

   ivLog = outLog;
   myState = good;
   pszName = 0;
   pszShortName = 0;
   iPort = portnum;
   this->logDisabled = logDisabled;

   if (!procName || !*procName) {
      myState = bad;
      return;
   }

   pszName = new char [strlen(procName) + 1];
   strcpy (pszName, procName);

   fnsplit (pszName, 0, 0, szFile, 0);
   pszShortName = new char [strlen(szFile) + 1];
   strcpy (pszShortName, szFile);

   steps = new Steps();
   if(LoadSteps() != IVR_SUCCESS)
   {
      myState = bad;
   }
}
/*TAM::TAM*/

/* -------------------------------------------------------------------------
   Name:        TAM::~TAM                                       (destructor)
   ------------------------------------------------------------------------- */
TAM::~TAM()
{
   delete steps;
   delete [] pszName;
   delete [] pszShortName;
}
/*TAM::~TAM*/

const char* TAM::GetName(void) const
{
   return(pszName);
}

const char* TAM::GetShortName(void) const
{
   return(pszShortName);
}

void TAM::SetTAMID(int value)
{
   iTAMID = value;
}

int TAM::GetTAMID(void) const
{
   return(iTAMID);
}

TAM::state TAM::GetState(void) const
{
   return(myState);
}

/* -------------------------------------------------------------------------
   Name:        TAM::LoadSteps

   Notes:       - The field 'next' must contain a value greater than 0 to be
                  connected to a next step.
   ------------------------------------------------------------------------- */
IV_STATUS TAM::LoadSteps(void)
{
   FILE      *fp;
   IV_STATUS  status;
   STEPREC    stepRec;

   TxtSetupStepRecMemory (&stepRec);

   if (IvOpenFile (ivLog, &fp, pszName) != IVR_SUCCESS) {
      myState = bad;
      return IVR_FAIL;
   }

   status = TxtGenericHeader (ivLog, fp, MOD_TYPE);
   if (status != IVR_SUCCESS) {
      myState = bad;
      ivLog->Print("\nERROR: Could not read from module file '%s'.\n", pszName);
      IvCloseFile (ivLog, &fp, pszName);
      return status;
   }

   status = TxtFetchStep (fp, &stepRec);
   if (status != IVR_SUCCESS) {
      myState = bad;
      ivLog->Print("\nERROR: Could not read from module file '%s'.\n", pszName);
      IvCloseFile (ivLog, &fp, pszName);
      return status;
   }

   while(!feof(fp))
   {
      Step* step;

      // ******************************** STEPS THAT MODIFY OTHER STEPS ****
      if((stepRec.op >= Step::IVRTTOTHER) && (stepRec.op <= Step::APPFALSE))
      {
         step = steps->FindStep(stepRec.parent);

         if(step != NULL)
         {
            switch(stepRec.op)
            {
               case Step::APPBUSY:       step->SetBusyStep(stepRec.next);       break;
               case Step::APPCHECK:      step->SetTrueStep(stepRec.next);       break;
               case Step::APPEX:         step->SetFalseStep(stepRec.next);      break;
               case Step::APPFAILURE:    step->SetFailureStep(stepRec.next);    break;
               case Step::APPFALSE:      step->SetFalseStep(stepRec.next);      break;
               case Step::APPNOVOICE:    step->SetNoVoiceStep(stepRec.next);    break;
               case Step::APPSILENCE:    step->SetSilenceStep(stepRec.next);    break;
               case Step::APPTHRESHOLD:  step->SetThresholdStep(stepRec.next);  break;
               case Step::APPTIMEOUT:    step->SetTimeoutStep(stepRec.next);    break;
               case Step::APPTRUE:       step->SetTrueStep(stepRec.next);       break;
               case Step::IVRTT0:
               case Step::IVRTT1:
               case Step::IVRTT2:
               case Step::IVRTT3:
               case Step::IVRTT4:
               case Step::IVRTT5:
               case Step::IVRTT6:
               case Step::IVRTT7:
               case Step::IVRTT8:
               case Step::IVRTT9:
               case Step::IVRTTA:
               case Step::IVRTTB:
               case Step::IVRTTC:
               case Step::IVRTTD:
               case Step::IVRTTOTHER:
               case Step::IVRTTPOUND:
               case Step::IVRTTSTAR:
               {
                  if((strcmp(step->GetStepName(), "Menu") == 0) || (strcmp(step->GetStepName(), "Case") == 0)
                     || (strcmp(step->GetStepName(), "Get Letter") == 0))
                  {
                     ((StepTones*) step)->SetTT (stepRec.op, stepRec.next);
                  }
                  else
                  {
                     ivLog->Print("ERROR: TT Action step (%d) linked to non-tone step (%d)\n", stepRec.current, stepRec.parent);
		               ivLog->Print("       flowchart: '%s', step number: %d \n", GetShortName(), stepRec.current);
                     ivLog->Print("This application may have been created by a version of"
                        " TFDesigner that is incompatible with this version of TFServer.\n");
                     return(IVR_NOTSUPPORTED);
                  }
                  break;
               }
            }
         }
         else
         {
            if(stepRec.current >= stepRec.parent)
            {
               ivLog->Print("WARNING: TT Action step (%d) with missing parent step (%d)\n", stepRec.current, stepRec.parent);
            }
            else
            {
               ivLog->Print("WARNING: TT Action step (%d) added before parent step (%d)\n", stepRec.current, stepRec.parent);
            }
            ivLog->Print("       Flowchart: '%s', Step number: %d \n", GetShortName(), stepRec.current);
            ivLog->Print("  This TT Action step will not be linked to its parent step!\n");
         }
      }
      else
      {
         if((step = Step::Create(stepRec)) == NULL)
         {
            ivLog->Print("ERROR: An unknown step type (%d) has been encountered\n", stepRec.op);
            ivLog->Print("       flowchart: '%s', step number: %d \n", GetShortName(), stepRec.current);
            ivLog->Print("This application may have been created by a version of"
               " TFDesigner that is incompatible with this version of TFServer.\n");
            return(IVR_NOTSUPPORTED);
         }

         switch((Step::OP_CODE) stepRec.op)
         {
            case Step::PROCSTART:
            {
               // This step is added to the beginning since it is the first step in the TAM
               // The order of steps is otherwise unimportant
               steps->AddFirst(step);
               break;
            }

            case Step::VOCRECINIT:
            {
               // Abuse of SetNextTAMName so that StepVocRecInit knows the TAM it's in
               step->SetNextTAMName(GetName());
               steps->AddLast(step);
               break;
            }

            default:
            {
               steps->AddLast(step);
            }
         }
      }

      TxtFetchStep(fp, &stepRec);
   }

   TxtFreeStepRecMemory(&stepRec);

   return(IvCloseFile(ivLog, &fp, pszName));
}
// TAM::LoadSteps

/* -------------------------------------------------------------------------
   Name:        TAM::GetFirstStep

   Purpose:     Finds the first step in the TAM.

   Notes:       - I think this is the first time I needed to know what the
                  OP code was for the step.
   ------------------------------------------------------------------------- */
Step* TAM::GetFirstStep(void)
{
   if(steps == NULL)
   {
      return(NULL);
   }

   return((Step*) steps->First());
}
/*TAM::GetFirstStep*/

/* -------------------------------------------------------------------------
   Name:        TAM::GetStep

   Purpose:     Find the specific numbered step in the TAM.
   ------------------------------------------------------------------------- */
Step *TAM::GetStep (int cStep)
{
   Step  *temp;

   if (!steps)
      return 0;

   temp = (Step*)steps->First();
   while (temp) {
      if (cStep == temp->GetCurrentStep())
         return temp;
      temp = (Step*)temp->Suc();
   }
   return 0;
}
/*TAM::GetStep*/

/* -------------------------------------------------------------------------
   Name:        TAM::GetLabelStep

   Purpose:     Find the specific labelled step in the TAM.
   ------------------------------------------------------------------------- */
Step *TAM::GetLabelStep (const char *label)
{
   Step  *temp;

   if (!steps)
      return 0;

   temp = (Step*)steps->First();
   while (temp) {
      if (temp->GetLabel() && !stricmp (temp->GetLabel(), label))
         return temp;
      temp = (Step*)temp->Suc();
   }
   return 0;
}
/*TAM::GetLabelStep*/

bool TAM::IsLogDisabled(void) const
{
   return(logDisabled);
}

