#include <winsock2.h>
#include <windows.h>

#include "TFEngine_communication.h"
#include "wait.h"
#include "brd_generic.h"

/* -------------------------------------------------------------------------
   Name:        Wait

   Purpose:     Wait for increments of 1/1000 seconds.

   Questions:   - Does the timeout parameter passed in need to be changed
                  from one format to another?

   Notes:       - When shutdown code is implemented, need to break down the
                  waiting into smaller cycles (start with 1000) and check
                  for monitor events between cycles.
                - Due to event checking, the timeout won't be 100% accurate.
   ------------------------------------------------------------------------- */
IV_STATUS Wait(PIVRINFO pInfo, int iTimeout)
{
   HANDLE          hWaitTimer;
   HANDLE          waitObjs[2];
   LARGE_INTEGER   waitTimeout;
   IV_STATUS status = IVR_SUCCESS;

   hWaitTimer = CreateWaitableTimer (0, false, 0);

   // waitTimeout is measured in 100 nanosecond intervals (10^-7 = .0000001).
   // To convert to milliseconds times by 10,000 (10^-3 = 10^-7 x 10^4).

	// Sean change to prevent overflow and allow waits longer than 214748 ms (2^31 / 10000)
   // was: waitTimeout.QuadPart = -(LONGLONG)(iTimeout * 10000);
   waitTimeout.QuadPart = -(LONGLONG) iTimeout * 10000;
   SetWaitableTimer (hWaitTimer, &waitTimeout, 0, 0, 0, false);
   waitObjs[0] = pInfo->messages->GetMessageSignal();
   waitObjs[1] = hWaitTimer;
   while(status == IVR_SUCCESS)
   {
      switch(WaitForMultipleObjects(2, waitObjs, false, INFINITE))
      {
         case WAIT_FAILED:
         case WAIT_TIMEOUT:
         {
            status = IVR_FATAL;
            break;
         }

         case WAIT_OBJECT_0:
         {
            status = pInfo->ivBoard->ProcessSystemEvent();
            break;
         }

         case WAIT_OBJECT_0 + 1:
         {
            status = IVR_TIMEOUT;
            break;
         }
      }
   }

   CloseHandle(hWaitTimer);

   return(status == IVR_TIMEOUT ? IVR_SUCCESS : status);
}
/*Wait*/
