#include <vcl.h>
#include <dos.h>        				// _argc, _argv
#include <stdio.h>						// FILE, fopen, fgetc, fgets, feof, fclose
#include <algorithm>
using std::max;
#include <string>
using std::string;
#pragma hdrstop

#include "..\TFServer\ivr_defs.h" 	// TFLINELIST_REG_KEY, TELEFLOW_APP_NAME_MAX
#include "..\escutil\help_fns.h"    // for link to Help file entry
#include "linelist_savechanges.h"	// FormSaveChanges
#include "linelist_rep.h"				// FormReplicate
#include "linelist_about.h"			// AboutBox
#include "linelist_main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormLineList *FormLineList;
//---------------------------------------------------------------------------

__fastcall TFormLineList::TFormLineList(TComponent* Owner)
	: TForm(Owner)
{
	// Load window settings, column widths and history list from registry
   ReadRegistry();

	// Clear all data out of the grid
	ClearGrid();

	// Set all editor menus and variables for no open file
   EditorActive(false);

	// Initialise App grid
   gridAppList->ColWidths[0] = 10;
	gridAppList->Cells[APP_NAME][0] = "Name";
	gridAppList->Cells[APP_FILE_NAME][0] = "Filename";
 	gridAppList->Cells[APP_PROTOCOL][0] = "Protocol";
	gridAppList->Cells[APP_RESTART_COUNT][0] = "Restart Count";
	gridAppList->Cells[APP_NO_LIMIT][0] = "No limit";
	gridAppList->Cells[APP_LOG_OUTPUT][0] = "Log output";
	gridAppList->Cells[APP_APPEND_FILE][0] = "Append to file";
	gridAppList->Cells[APP_PORT][0] = "@PORT";
	gridAppList->Cells[APP_P1][0] = "@P1";
	gridAppList->Cells[APP_P2][0] = "@P2";
	gridAppList->Cells[APP_P3][0] = "@P3";
	gridAppList->Cells[APP_P4][0] = "@P4";
	gridAppList->Cells[APP_P5][0] = "@P5";
	gridAppList->Cells[APP_P6][0] = "@P6";
	gridAppList->Cells[APP_P7][0] = "@P7";
	gridAppList->Cells[APP_P8][0] = "@P8";
	gridAppList->Cells[APP_P9][0] = "@P9";

   // If we have been passed a file name as the first argument then load it
   if(_argc > 1)
   {
   	LoadTAL(_argv[1]);
   }
}
//---------------------------------------------------------------------------

__fastcall TFormLineList::~TFormLineList(void)
{
   EditorActive(false);

	// Save window settings, column widths and history list to registry
	WriteRegistry();
}

//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuHelpAboutClick(TObject *Sender)
{
	AboutBox->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::ComboBoxProtocolChange(TObject *Sender)
{
	// Add a new row if we are editing the last one
   if(gridAppList->Row == gridAppList->RowCount - 1)
	{
   	EditBlankRow();
	}

	// Update the grid
  	gridAppList->Cells[APP_PROTOCOL][gridAppList->Row] = ComboBoxProtocol->Text;

	// The file has now changed
	fileChanged = true;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::CheckBoxNoLimitClick(TObject *Sender)
{
	// Are we changing checkbox or just updating from cell?
//   if(gridAppList->Cells[APP_NO_LIMIT][gridAppList->Row] != (CheckBoxNoLimit->Checked ? "X" : " "))
	{
	   // Add a new row if we are editing the last one
  		if((gridAppList->Row == gridAppList->RowCount - 1) && validEdit)
		{
   	 	EditBlankRow();
	  	}

		// Update the grid
	  	gridAppList->Cells[APP_NO_LIMIT][gridAppList->Row] = CheckBoxNoLimit->Checked ? "X" : " ";

		// The file has now changed
		fileChanged = validEdit;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::CheckBoxAppendFileClick(TObject *Sender)
{
	// Are we changing checkbox or just updating from cell?
//   if(gridAppList->Cells[APP_APPEND_FILE][gridAppList->Row] != (CheckBoxAppendFile->Checked ? "X" : " "))
	{
	   // Add a new row if we are editing the last one
  		if((gridAppList->Row == gridAppList->RowCount - 1) && validEdit)
		{
   	 	EditBlankRow();
	  	}

		// Update the grid
	  	gridAppList->Cells[APP_APPEND_FILE][gridAppList->Row] = CheckBoxAppendFile->Checked ? "X" : " ";

		// The file has now changed
		fileChanged = validEdit;
   }
}
//---------------------------------------------------------------------------

/* -------------------------------------------------------------------------
   Name:        TFormLineList::DetermineTALVersion

   Purpose:     Reads the version number from the .TAL file.  If no header
                is found, the file is assumed to be in first version format.
   ------------------------------------------------------------------------- */
int __fastcall TFormLineList::DetermineTALVersion(char* pszFileNm)
{
   int    version;
   FILE*  fTAL;
   char   szLine[1024];
   char   szVersion[4];   // Format expected: "XXX"

	// Unable to open file.
   if((fTAL = fopen (pszFileNm, "r")) == NULL)
   {
      return(0);
   }

   fgets(szLine, 1024, fTAL);
   if(!strncmp(szLine, "TAL Version: ", 13))
   {
      strncpy (szVersion, &(szLine[13]), 4);
      szVersion[3] = 0;

      version = atoi (szVersion);

      if (version < 2)
      {
         version = 0;       // ERROR: Headers weren't included until version 2.
      }
   }
   else
   {
      version = 1;
   }

   fclose(fTAL);
   return(version);
}
//---------------------------------------------------------------------------

/* -------------------------------------------------------------------------
   Name:        TFormAppBuild::LoadTAL

   Purpose:     Loads in the TeleFlow Application List (TAL) file into the
                memory structure passed in.
   ------------------------------------------------------------------------- */
bool __fastcall TFormLineList::LoadTAL(char* pszFileNm)
{
   FILE* fTAL;
   char  szLine[1024];
   int   iOffset;        // Offset into s.
   int   ch;             // Character read in from file.
   int   iFieldNo;       // Current field being read from file.
   int   version = DetermineTALVersion(pszFileNm);

   int iAppCount = 0;

   if((version < 1) || (version > 4))
   {
      return(false);
   }

   try
   {
		EditorActive(true);

      // Unable to open file.
      if((fTAL = fopen(pszFileNm, "r")) == NULL)
      {
      	return(false);
      }

   	// Clear the grid of any previous data
      ClearGrid();

      // First line of file must contain version information.
      fgets(szLine, 1024, fTAL);

      ch = fgetc(fTAL);
      iFieldNo = 1;
      iOffset = 0;

      while(!feof (fTAL))
      {
        // ----- Found end of field
         if((ch == '\t') || (ch == '\r') || (ch == '\n'))
         {
            szLine[iOffset] = 0;

            // Version filter
            // - current version is now 4 - TF Feb.19/02
            switch(version)
            {
            	case 1:
               {
               	switch(iFieldNo)
                  {
                  	case 5:	iFieldNo = 6;	break;
                  	case 7: iFieldNo = 11; break;
                  }
               }

               case 2:
               {
               	switch(iFieldNo)
                  {
                  	case 5: iFieldNo = 6; break;
                  	case 10: iFieldNo = 11; break;
                  }
               }

               case 3:
               {
               	switch(iFieldNo)
                  {
                  	case 10: iFieldNo = 11; break;
                  }
               }
            }

            switch(iFieldNo)
            {
            	case 1:  // ----------- FIELD 1: --------------------------- iAppID
               {
               	iAppCount++;
            	   gridAppList->RowCount++;
   					// No longer loading IDs
                  break;
               }

               case 2:  // ----------- FIELD 2: -------------------- pszDebugParam
               {
            	   // No longer loading debug parameters
   					break;
               }

               case 3:  // ----------- FIELD 3: ------------------------- pszAppNm
			   	{
                 // Truncate names longer than 30 characters
            	   if(strlen(szLine) > TELEFLOW_APP_NAME_MAX)
                  {
                     szLine[TELEFLOW_APP_NAME_MAX] = '\0';
                  }
                  gridAppList->Cells[APP_NAME][iAppCount] = szLine;
                  break;
               }

               case 4:  // ----------- FIELD 4: ------------------------ pszFileNm
	   			{
               	gridAppList->Cells[APP_FILE_NAME][iAppCount] = szLine;
                  break;
               }

               case 5:  // ----------- FIELD 5: ---------------------- pszProtocol
	   			{
               	gridAppList->Cells[APP_PROTOCOL][iAppCount] = szLine;
                  break;
               }

               case 6:  // ----------- FIELD 6: -------------------- iRestartCount
	   			{
               	gridAppList->Cells[APP_RESTART_COUNT][iAppCount] = szLine;
                  break;
               }

               case 7:  // ----------- FIELD 7: ----------------- bRestartInfinite
	   			{
               	if (szLine[0] == 'Y')
                 	{
                  	gridAppList->Cells[APP_NO_LIMIT][iAppCount] = "X";
					   }
                  else
                  {
                  	gridAppList->Cells[APP_NO_LIMIT][iAppCount] = " ";
			   		}
                  break;
               }

               case 8:  // ----------- FIELD 8: ----------------------- bLogOutput
               {
               	if (szLine[0] == 'Y')
                 	{
                  	gridAppList->Cells[APP_LOG_OUTPUT][iAppCount] = "All events";
   					}
                  else
                 	{
                  	gridAppList->Cells[APP_LOG_OUTPUT][iAppCount] = "Errors only";
				   	}
                  break;
               }

               case 9:  // ----------- FIELD 9: ----------------------- bLogAppend
               {
               	if (szLine[0] == 'Y')
                 	{
                  	gridAppList->Cells[APP_APPEND_FILE][iAppCount] = "X";
					   }
                  else
                 	{
                  	gridAppList->Cells[APP_APPEND_FILE][iAppCount] = " ";
			   		}
                  break;
               }

               case 10:  // ----------- FIELD 10: ----------------------- @PORT
               {
                 	gridAppList->Cells[APP_PORT][iAppCount] = szLine;
                  break;
               }

               default: // ----------- FIELDS 11-19: ----- Optional params (max 9)
           	   {
                 gridAppList->Cells[iFieldNo - 10 + APP_PORT][iAppCount] = (strcmp(szLine, "(null)")) ? szLine : "";
	   			}
		   	}

            // Reset variables to receive next field.
            iOffset = 0;
            if((ch == '\r') || (ch == '\n'))
            {
			   	// Fields not in earlier version files
				   switch(version)
               {
	   				case 1:
                  {
                  	gridAppList->Cells[APP_NO_LIMIT][iAppCount] = "X";
                  	gridAppList->Cells[APP_LOG_OUTPUT][iAppCount] = "Errors only";
                  	gridAppList->Cells[APP_APPEND_FILE][iAppCount] = "X";
                     // Fall through
                  }

                  case 2:
                  {
                  	gridAppList->Cells[APP_PROTOCOL][iAppCount] = "lps0";
                     // Fall through
                  }

                  case 3:
                  {
		         		if(gridAppList->Cells[APP_P1][iAppCount].Length() != 0)
                     {
				   			gridAppList->Cells[APP_PORT][iAppCount] = gridAppList->Cells[APP_P1][iAppCount];
                     }
                     else
                     {
                       	gridAppList->Cells[APP_PORT][iAppCount] = 0;
                     }
                  }
               }

            	iFieldNo = 1;
            }
            else
            {
            	++iFieldNo;
            }
         }
         else
         {
            // Build up field data until the end of the field is found.
            szLine[iOffset] = (char) ch;
            ++iOffset;
         }

         ch = fgetc(fTAL);
      } // while

      fclose(fTAL);

      if(TALFileName != NULL)
      {
   		// delete previous string
        	delete[] TALFileName;
      }

      TALFileName = new char[strlen(pszFileNm) + 1];
      strcpy(TALFileName, pszFileNm);

		// Add newly loaded file's name to the file name history
	   HistoryAdd();

		Caption = "TeleFlow Line List - " + (AnsiString) pszFileNm;

   	return(true);
   }

   catch (Exception &e)
   {
      Application->MessageBox(e.Message.c_str(), "LoadTAL", MB_OK | MB_ICONERROR);
      return(false);
   }
}
//---------------------------------------------------------------------------

bool __fastcall TFormLineList::SaveTAL(char* pszFileNm)
{
	// If we don't have a name yet we need to call Save As
	if(TALFileName == NULL)
	{
		return(SaveAs());
   }

	FILE* fTAL;

   if((fTAL = fopen(pszFileNm, "w")) == NULL)
   {
   	char  s[255];
      sprintf(s, "Failed to write to file '%s'", pszFileNm);
      Application->MessageBox(s, "Write Error", MB_OK | MB_ICONERROR);
      return(false);
   }

   fprintf(fTAL, "TAL Version: 004\n");

   // This loop assumes that row 0 is a fixed row for the column titles
   // and that the last row is blank
   for(int row = 1; row < gridAppList->RowCount - 1; row++)
   {
   	int pvars;

   	fprintf(fTAL,	"%d"           // AppID
                     "\t-ZA"        // Now always "-ZA"
                     "\t%s"         // APP_NAME
                     "\t%s"         // APP_FILE_NAME
                     "\t%s"         // APP_PROTOCOL
                     "\t%d"         // APP_RESTART_COUNT
                     "\t%c"         // APP_NO_LIMIT
                     "\t%c"         // APP_LOG_OUTPUT
                     "\t%c",        // APP_APPEND_FILE
               1000 + row,
               gridAppList->Cells[APP_NAME][row].c_str(),
               gridAppList->Cells[APP_FILE_NAME][row].c_str(),
               gridAppList->Cells[APP_PROTOCOL][row].c_str(),
               gridAppList->Cells[APP_RESTART_COUNT][row].ToInt(),
               (gridAppList->Cells[APP_NO_LIMIT][row] == "X") ? 'Y' : 'N',
               (gridAppList->Cells[APP_LOG_OUTPUT][row] == "All events") ? 'Y' : 'N',
               (gridAppList->Cells[APP_APPEND_FILE][row] == "X") ? 'Y' : 'N');

		// We only want to save up to the last non-empty P variable
		for(pvars = APP_P9; (pvars >= APP_PORT) && (gridAppList->Cells[pvars][row]== ""); pvars--);

      for(int col = APP_PORT; col <= pvars; col++)
      {	fprintf (fTAL, "\t%s", (gridAppList->Cells[col][row] != "") ? gridAppList->Cells[col][row].c_str() : "(null)");
      }
      fprintf (fTAL, "\n");
   }
   fclose(fTAL);
	fileChanged = false;

	// Add the recently saved file name to the history list
   HistoryAdd();

	return(true);
}

//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuFileOpenClick(TObject *Sender)
{
	// If the file has been changed, check to see if we should save the changes
   // and abort if we cancel
	if(fileChanged && !SaveChanges())
	{
		return;
   }

   dlgOpen->FileName = (TALFileName == NULL) ? "" : TALFileName;
   dlgOpen->Title = "Open TeleFlow Application List File";
   dlgOpen->Filter = "TeleFlow Application List File (*.tal)|*.tal|"
                     "All Files (*.*)|*.*";
   dlgOpen->Options.Clear();

   if(dlgOpen->Execute())
   {
   	LoadTAL(dlgOpen->FileName.c_str());
		ActiveControl = gridAppList;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuFileExitClick(TObject *Sender)
{
	Close();
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuFileSaveClick(TObject *Sender)
{
	if(ValidCells())
	{
		SaveTAL(TALFileName);
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::ClearGrid(void)
{
  // Clear all the rows except the column title row
   for(int counter = 1; counter < gridAppList->RowCount; counter++)
   {
   	gridAppList->Rows[counter]->Clear();
   }

   // Reduce the grid to a column title row and a one blank row
   gridAppList->RowCount = 2;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::EditBlankRow(void)
{
	// Save ActiveControl state since adding a row to the grid also switches the
   // active control to the grid
   TWinControl* activeControl = ActiveControl;

   gridAppList->RowCount++;

   // Default settings for a new row
   gridAppList->Cells[APP_PROTOCOL][gridAppList->RowCount - 2] = "lps0";
   gridAppList->Cells[APP_RESTART_COUNT][gridAppList->RowCount - 2] = "0";
	gridAppList->Cells[APP_NO_LIMIT][gridAppList->RowCount - 2] = "X";
	gridAppList->Cells[APP_LOG_OUTPUT][gridAppList->RowCount - 2] = "Error only";
	gridAppList->Cells[APP_APPEND_FILE][gridAppList->RowCount - 2] = "X";

   // Restore the active control
   ActiveControl = activeControl;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuFileCloseClick(TObject *Sender)
{
	// If the file has been changed, check to see if we should save the changes
   // and abort if we cancel
	if(fileChanged && !SaveChanges())
	{
		return;
   }

	EditorActive(false);
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::InsertRows(int row, int number)
{
	int counter;
	gridAppList->RowCount += number;

   // Move rows down and don't copy into blank row at end
 	for(counter = gridAppList->RowCount - 2; counter > row + number - 1; counter--)
   {
   	gridAppList->Rows[counter] = gridAppList->Rows[counter - number];
   }

	// Set vacated rows to defaults
	for(; counter >= row; counter--)
   {
      // Clear row
      gridAppList->Rows[counter]->Clear();

	   // Default settings for a new row
   	gridAppList->Cells[APP_PROTOCOL][counter] = "lps0";
	   gridAppList->Cells[APP_RESTART_COUNT][counter] = "0";
		gridAppList->Cells[APP_NO_LIMIT][counter] = "X";
		gridAppList->Cells[APP_LOG_OUTPUT][counter] = "Error only";
		gridAppList->Cells[APP_APPEND_FILE][counter] = "X";
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::DeleteRows(int row, int number)
{
   // Don't delete blank row at end
	if(row < gridAppList->RowCount - number)
	{
   	// Move rows after deletion row up one
    	for(int counter = row; counter < gridAppList->RowCount - number - 1; counter++)
	   {
      	gridAppList->Rows[counter] = gridAppList->Rows[counter + number];
   	}

		gridAppList->RowCount -= number;

 		// Clear blank row
	   gridAppList->Rows[gridAppList->RowCount - 1]->Clear();
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::ReplicateRows(int copyRow, int iterations, bool append)
{
	char* anotherAppName;
	char* appName;
	int nameNumber = 0;
   int portNumber = 0;
   int maxRow = copyRow;
   int pasteRow, leadingZeros;
	unsigned int index = 0;
   bool numberFound = false;

	// Create a copy of the application name
	appName = new char[gridAppList->Cells[APP_NAME][copyRow].Length() + 1];
	strcpy(appName, gridAppList->Cells[APP_NAME][copyRow].c_str());

   // Find integer at end of application name?
	if(strlen(appName) > 0)
   {
		for(index = strlen(appName) - 1; isdigit(appName[index]); index--);
   	index++;
   }

	appName[index] = '\0';

	// Search each instance of application name followed by a number and find largest number
	for(int counter = 1; counter < gridAppList->RowCount - 1; counter++)
   {
   	// Create a copy of application name for this row
	  	anotherAppName = new char[gridAppList->Cells[APP_NAME][counter].Length() + 1];
		strcpy(anotherAppName, gridAppList->Cells[APP_NAME][counter].c_str());

		// Don't process empty strings
	 	if(strlen(anotherAppName) > 0)
	 	{
         unsigned int index2;

			for(index2 = strlen(anotherAppName) - 1; isdigit(anotherAppName[index2]); index2--);
		   index2++;
     		anotherAppName[index2] = '\0';

		 	// The truncated names match
   	  	if(!strcmp(anotherAppName, appName))
	   	{
       		// Get a new copy of the application name for this row and extract the number
	     		strcpy(anotherAppName, gridAppList->Cells[APP_NAME][counter].c_str());

            if(strlen(anotherAppName) > index)
	         {
   	      	numberFound = true;

	   	     	if(atoi(anotherAppName + index) > nameNumber)
		   	   {
      	   	 	nameNumber = atoi(anotherAppName + index);
     	   			maxRow = counter;
		        	}
   	      }
	   	}
      }

     	delete[] anotherAppName;
   }

   delete[] appName;

	// If there is no number than pretend it's 1 (so the next one's 2)
   if(!numberFound)
   {
		nameNumber = 1;
   }

	// Does the app have a number other than 0 for APP_PORT
	if(atoi(gridAppList->Cells[APP_PORT][copyRow].c_str()))
	{
   	// Find the largest number used for APP_PORT so far
  		for(int counter = 0; counter < gridAppList->RowCount - 1; counter++)
	   {
      	portNumber = max(portNumber, atoi(gridAppList->Cells[APP_PORT][counter].c_str()));
   	}
   }

	if(append)
   {
   	pasteRow = gridAppList->RowCount - 1;
   	for(int counter = 0; counter < iterations; counter++)
      {
      	EditBlankRow();
      }
   }
   else
   {
   	pasteRow = copyRow + 1;
		if(maxRow > copyRow)
      {
      	maxRow += iterations;
      }
   	InsertRows(pasteRow, iterations);
   }

	// Create copy of application name with largest number
  	appName = new char[gridAppList->Cells[APP_NAME][maxRow].Length() + 1];
	strcpy(appName, gridAppList->Cells[APP_NAME][maxRow].c_str());

	// Find out how many leading zeros there are
   for(leadingZeros = 0; appName[index + leadingZeros] == '0'; leadingZeros++);

	// Null terminate after leading zeros
   index += leadingZeros;
   appName[index] = '\0';

	for(int counter = 0; counter < iterations; counter++)
   {
      // Use up another leading zero if the number gets longer and there are any
      // zeros left to use
		if((IntToStr(nameNumber).Length() < IntToStr(++nameNumber).Length()) &&
      	(leadingZeros > 0))
      {
      	leadingZeros--;
         appName[--index] = '\0';
      }

		// Copy application name base
	   anotherAppName = new char[strlen(appName) + IntToStr(nameNumber).Length() + 1];
      strcpy(anotherAppName, appName);

		// Add the largest number plus 1 to application name
		strcat(anotherAppName, IntToStr(nameNumber).c_str());

		// Copy original row
		gridAppList->Rows[pasteRow] = gridAppList->Rows[copyRow];

		// Paste new name
		gridAppList->Cells[APP_NAME][pasteRow] = anotherAppName;

		// Use the next port number
      if(portNumber)
		{
      	gridAppList->Cells[APP_PORT][pasteRow] = ++portNumber;
      }

      pasteRow++;

      delete[] anotherAppName;
   }

   delete[] appName;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuEditInsertBlankClick(TObject *Sender)
{
 	InsertRows(gridAppList->Row, 1);
	fileChanged = true;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuEditDeleteClick(TObject *Sender)
{
   DeleteRows(gridAppList->Row, 1);
	fileChanged = true;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuEditCopyClick(TObject *Sender)
{
	clipBoard->Rows[0] = gridAppList->Rows[gridAppList->Row];
 	validClipboard = true;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuEditPasteClick(TObject *Sender)
{
	if(validClipboard)
   {
		// Add new row if we are pasting to blank row
	   if(gridAppList->Row == gridAppList->RowCount - 1)
   	{
      	EditBlankRow();
	   }

		gridAppList->Rows[gridAppList->Row] = clipBoard->Rows[0];
		fileChanged = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuEditInsertClick(TObject *Sender)
{
	if(validClipboard)
	{
    	InsertRows(gridAppList->Row, 1);
		gridAppList->Rows[gridAppList->Row] = clipBoard->Rows[0];
		fileChanged = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuFileNewClick(TObject *Sender)
{
	// If the file has been changed, check to see if we should save the changes
   // and abort if we cancel
	if(fileChanged && !SaveChanges())
	{
		return;
   }

	EditorActive(true);
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::FormResize(TObject *Sender)
{
	// Make sure Windows refreshes the graphics... sometimes garbage get left behind
	Invalidate();
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::EditorActive(bool enable)
{
	// Change state of grid
	gridAppList->Visible = enable;

	// Clean up grid
	ClearGrid();

	// Reset variables and flags
   editCol = 0;
   editRow = 0;
   fileChanged = false;
 	validClipboard = false;
	TABFlag = false;

   if(TALFileName != NULL)
   {
   	delete[] TALFileName;
      TALFileName = NULL;
   }

	// Change state of menus
   mnuFileSave->Enabled = enable;
   mnuFileSaveAs->Enabled = enable;
   mnuFileClose->Enabled = enable;
	mnuEdit->Visible = enable;
   mnuTools->Visible = enable;

	// Hide special cell graphics
	ButtonSelectTAP->Visible = false;
	BitBtnProtocol->Visible = false;
  	ComboBoxProtocol->Visible = false;
  	ComboBoxLogErrors->Visible = false;
  	CheckBoxNoLimitPanel->Visible = false;
   CheckBoxAppendFilePanel->Visible = false;

	if(enable)
   {
		// Start with the cell in the upper left corner active
	   ActiveControl = gridAppList;
      gridAppList->Row = 1;
      gridAppList->Col = 1;
   }

	// Set title bar to default
	Caption = "TeleFlow Line List";
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::gridAppListMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
	int     iCol;
   int     iRow;
   TPoint  ScreenOrigin;
   TPoint  ClientPoint;

   // Highlight the row under the mouse by moving to this row.
   gridAppList->MouseToCell(X, Y, iCol, iRow);

   // Do nothing if they click on the column headings
   if(iRow < 1)
	{
   	return;
   }

	// Select row we clicked on
   gridAppList->Row = iRow;

   if((iCol == 0) || (Button == mbRight))
   {
   	gridAppList->Options >> goEditing;
   	gridAppList->Options << goRowSelect;

		// Make the row actually show up as selected since it is
   	if(gridAppList->Row > 1)
	   {
      	gridAppList->Row--;
   	   gridAppList->Row++;
	   }
   	else if(gridAppList->Row != gridAppList->RowCount - 1)
	   {
      	gridAppList->Row++;
   	   gridAppList->Row--;
	   }
   }
	// Left click not in row buttons
   else
   {
	   // We should not be able to edit these cells directly
   	if((iCol == APP_PROTOCOL) || (iCol == APP_NO_LIMIT) ||
   		(iCol == APP_LOG_OUTPUT) || (iCol == APP_APPEND_FILE))
	   {
   		gridAppList->Options >> goEditing;
	   }
   	else
	   {
   		gridAppList->Options << goEditing;
	   }

   	gridAppList->Options >> goRowSelect;

      // Edit cell if we already have it selected
      if((gridAppList->Col == iCol) && (gridAppList->Row == iRow) && (Button == mbLeft))
      {
         gridAppList->EditorMode = true;
      }
      else
      {
   		// Select column we clicked on
         gridAppList->Col = iCol;

		   // Make the cell only show up as selected since it is
      	if(gridAppList->Row > 1)
	      {
         	gridAppList->Row--;
   	      gridAppList->Row++;
   	   }
      	else if(gridAppList->Row != gridAppList->RowCount - 1)
	      {
      	   gridAppList->Row++;
      	   gridAppList->Row--;
	      }

   	   gridAppListDrawSpecialCell(iCol, iRow);
      }
   }

	// Sometimes garbage is left behind coming from a row select
	gridAppList->Invalidate();

   // Display a popup menu if the user clicks the right mouse button.
   if(Button == mbRight)
   {
      // Convert the Grid X, Y values to screen X, Y values.
      ClientPoint.x = X;
      ClientPoint.y = Y;
      ScreenOrigin = gridAppList->ClientToScreen(ClientPoint);

      // Display menu (usually to the right of the mouse cursor).
      popMenuEdit->Popup(ScreenOrigin.x, ScreenOrigin.y);
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuEditReplicateClick(TObject *Sender)
{
	// Default to one row
	FormReplicate->efNewRows->Text = "1";

   FormReplicate->ShowModal();

   if(FormReplicate->ModalResult == mrOk)
   {
   	ReplicateRows(gridAppList->Row, atoi(FormReplicate->efNewRows->Text.c_str()),
   		FormReplicate->RadioButtonAppend->Checked);
		fileChanged = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuFileSaveAsClick(TObject *Sender)
{
	SaveAs();
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuEditCutClick(TObject *Sender)
{
	clipBoard->Rows[0] = gridAppList->Rows[gridAppList->Row];
	DeleteRows(gridAppList->Row, 1);
	fileChanged = true;
 	validClipboard = true;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::ComboBoxLogErrorsChange(TObject *Sender)
{
	// Add a new row if we are editing the last one
   if(gridAppList->Row == gridAppList->RowCount - 1)
	{
      int temp = ComboBoxLogErrors->ItemIndex;

   	EditBlankRow();

     	ComboBoxLogErrors->ItemIndex = temp;
	}

  	gridAppList->Cells[APP_LOG_OUTPUT][gridAppList->Row] = ComboBoxLogErrors->Text;
	fileChanged = true;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::gridAppListDrawSpecialCell(int ACol, int ARow)
{
	ButtonSelectTAP->Visible = false;
	BitBtnProtocol->Visible = false;
  	ComboBoxProtocol->Visible = false;
   CheckBoxNoLimitPanel->Visible = false;
	BitBtnLogErrors->Visible = false;
  	ComboBoxLogErrors->Visible = false;
   CheckBoxAppendFilePanel->Visible = false;

  	if(ARow != 0)
	{
   	switch(ACol)
		{
  			case APP_NAME:
			{
  	   	   break;
      	}

	      case APP_FILE_NAME:
  		   {
				// Draw file browsing button
     			ButtonSelectTAP->Left = gridAppList->CellRect(ACol, ARow).Right - 18;
     			ButtonSelectTAP->Top = gridAppList->CellRect(ACol, ARow).Top + 3;
	         ButtonSelectTAP->Visible = true;
  		   	break;
     		}

      	case APP_PROTOCOL:
   	   {
				// Draw pulldown menu button
     			BitBtnProtocol->Left = gridAppList->CellRect(ACol, ARow).Right - 15;
     			BitBtnProtocol->Top = gridAppList->CellRect(ACol, ARow).Top + 6;
	         BitBtnProtocol->Visible = true;
  	   		break;
      	}

	      case APP_RESTART_COUNT:
			{
     			break;
	      }

	      case APP_NO_LIMIT:
			{
				// Draw checkbox panel
         	CheckBoxNoLimitPanel->Left = gridAppList->CellRect(ACol, ARow).Left + 3;
   			CheckBoxNoLimitPanel->Top = gridAppList->CellRect(ACol, ARow).Top + 3;
				CheckBoxNoLimitPanel->Visible = true;
            ActiveControl = CheckBoxNoLimit;
     			break;
	      }

  		   case APP_LOG_OUTPUT:
     		{
				// Draw pulldown menu button
     			BitBtnLogErrors->Left = gridAppList->CellRect(ACol, ARow).Right - 15;
     			BitBtnLogErrors->Top = gridAppList->CellRect(ACol, ARow).Top + 6;
	         BitBtnLogErrors->Visible = true;
     			break;
	      }

	      case APP_APPEND_FILE:
			{
				// Draw checkbox panel
         	CheckBoxAppendFilePanel->Left = gridAppList->CellRect(ACol, ARow).Left + 3;
   			CheckBoxAppendFilePanel->Top = gridAppList->CellRect(ACol, ARow).Top + 3;
				CheckBoxAppendFilePanel->Visible = true;
            ActiveControl = CheckBoxAppendFile;
     			break;
	      }

	      case APP_PORT:
 		   case APP_P1:
 		   case APP_P2:
     		case APP_P3:
	      case APP_P4:
  		   case APP_P5:
     		case APP_P6:
      	case APP_P7:
   	   case APP_P8:
  	   	case APP_P9:
			{
  	   		break;
      	}
	      default:;
  		}
	}
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::gridAppListDrawCell(TObject *Sender,
      int ACol, int ARow, TRect &Rect, TGridDrawState State)
{
	// Draw check boxes in proper columns on valid rows
   if((ARow != 0) && (ARow != gridAppList->RowCount - 1) &&
   	((ACol == APP_NO_LIMIT) || (ACol == APP_APPEND_FILE)))
	{
      imgCheckBox->Draw(gridAppList->Canvas,
   		gridAppList->CellRect(ACol, ARow).Left + 1,
         gridAppList->CellRect(ACol, ARow).Top + 1,
         (gridAppList->Cells[ACol][ARow] == "X"), true);
   }

	if((ACol == gridAppList->Col) && (ARow == gridAppList->Row))
   {
   	gridAppListDrawSpecialCell(ACol, ARow);
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::gridAppListSetEditText(TObject *Sender,
      int ACol, int ARow, const AnsiString Value)
{
	if(ARow == gridAppList->RowCount - 1)
   {
   	EditBlankRow();
   }

	gridAppListDrawSpecialCell(ACol, ARow);

   fileChanged = true;
}
//---------------------------------------------------------------------------

// The next several functions are necessary to make comboboxes appear to be a
//	part of the grid while TABing and SHIFT-TABing
void __fastcall TFormLineList::gridAppListKeyUp(TObject *Sender,
      WORD &Key, TShiftState Shift)
{
	if(TABFlag && (Key == VK_TAB))
   {
   	if(Shift.Contains(ssShift))
   	{
      	gridAppList->Col--;
      }
      else
   	{
      	gridAppList->Col++;
      }

		TABFlag = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::CheckBoxAppendFileHeadKeyUp(
      TObject *Sender, WORD &Key, TShiftState Shift)
{
	switch(Key)
	{
   	case VK_UP:
   	{
      	if(gridAppList->Row > 1)
      	{
         	gridAppList->Row--;
         }
         break;
      }
		case VK_TAB:
      case VK_LEFT:
      default:
		{
      	gridAppList->Col--;
	   }
   }

   ActiveControl = gridAppList;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::CheckBoxAppendFileTailKeyUp(
      TObject *Sender, WORD &Key, TShiftState Shift)
{
	switch(Key)
	{
   	case VK_DOWN:
   	{
      	if(gridAppList->Row < gridAppList->RowCount - 1)
      	{
         	gridAppList->Row++;
         }
         break;
      }
		case VK_TAB:
      case VK_RIGHT:
      default:
		{
      	gridAppList->Col++;
	   }
   }

   ActiveControl = gridAppList;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::CheckBoxNoLimitHeadKeyUp(TObject *Sender,
      WORD &Key, TShiftState Shift)
{
	switch(Key)
	{
   	case VK_UP:
   	{
      	if(gridAppList->Row > 1)
      	{
         	gridAppList->Row--;
         }
         break;
      }
		case VK_TAB:
      case VK_LEFT:
      default:
		{
      	gridAppList->Col--;
	   }
   }

   ActiveControl = gridAppList;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::CheckBoxNoLimitTailKeyUp(TObject *Sender,
      WORD &Key, TShiftState Shift)
{
	switch(Key)
	{
   	case VK_DOWN:
   	{
      	if(gridAppList->Row < gridAppList->RowCount - 1)
      	{
         	gridAppList->Row++;
         }
         break;
      }
		case VK_TAB:
      case VK_RIGHT:
      default:
		{
      	gridAppList->Col++;
	   }
   }

   ActiveControl = gridAppList;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::CheckBoxNoLimitExit(TObject *Sender)
{
	// So we don't create blank rows when reading a blank into CheckBoxNoLimit
	validEdit = false;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::CheckBoxAppendFileExit(TObject *Sender)
{
	// So we don't create blank rows when reading a blank into CheckBoxAppendFile
	validEdit = false;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::CheckBoxNoLimitEnter(TObject *Sender)
{
	CheckBoxNoLimit->Checked = (gridAppList->Cells[APP_NO_LIMIT][gridAppList->Row] == "X");
	validEdit = true;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::CheckBoxAppendFileEnter(TObject *Sender)
{
	CheckBoxAppendFile->Checked = (gridAppList->Cells[APP_APPEND_FILE][gridAppList->Row] == "X");
	validEdit = true;
}
//---------------------------------------------------------------------------

bool __fastcall TFormLineList::SaveChanges(void)
{
   AnsiString fileName;

	if(TALFileName == NULL)
   {
   	fileName = "file";
   }
   else
   {
		char* pfileName = strrchr(TALFileName, '\\');
	   fileName = (pfileName == NULL) ? TALFileName : pfileName;
   }

   FormSaveChanges->lblSaveChanges->Caption = "Save changes to " + fileName + "?";

	FormSaveChanges->ShowModal();
	if(FormSaveChanges->ModalResult == mrYes)
   {
   	return(ValidCells() && SaveTAL(TALFileName));
   }
   else
   {
	   return(FormSaveChanges->ModalResult == mrNo);
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::FormCloseQuery(TObject *Sender,
      bool &CanClose)
{
	CanClose = fileChanged ? SaveChanges() : true;
}
//---------------------------------------------------------------------------

bool __fastcall TFormLineList::ValidRestartCount(int ACol, int ARow)
{
	// Don't check if column 0, row 0, or last row
   // >= prevents dialog when shutting down grid
	if((ACol == 0) || (ARow == 0) || (ARow >= gridAppList->RowCount - 1))
   {
   	return(true);
   }

	if(gridAppList->Cells[ACol][ARow] !=  AnsiString(atoi(gridAppList->Cells[ACol][ARow].c_str())))
   {
   	Application->MessageBox("Value must be an integer greater than 0.",
      	"Restart Count", MB_OK | MB_ICONERROR);

      // Smallest allowable value
   	gridAppList->Cells[ACol][ARow] = "0";

      return(false);
   }

   return(true);
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::gridAppListSelectCell(TObject *Sender,
      int ACol, int ARow, bool &CanSelect)
{
	CanSelect = true;

   // We are no longer at the cell we were last editting
   if((ACol != editCol) || (ARow != editRow))
   {
		if((editCol == APP_RESTART_COUNT) && !ValidRestartCount(editCol, editRow))
  	   {
			// We can't leave cell with invalid value
			CanSelect = false;
      }
		else
      {
			if(editCol == APP_NAME)
   	   {
      		ValidAppName(editCol, editRow);
	      }

         // Update editCol and editRow values
	    	editCol = ACol;
  			editRow = ARow;
      }
   }

   // We should not be able to edit these cells directly
   if((ACol == APP_PROTOCOL) || (ACol == APP_NO_LIMIT) ||
   	(ACol == APP_LOG_OUTPUT) || (ACol == APP_APPEND_FILE))
   {
   	gridAppList->Options >> goEditing;
   }
   else
   {
   	gridAppList->Options << goEditing;
   }

	// reset flag in case we got here by clicking
   TABFlag = false;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::gridAppListExit(TObject *Sender)
{
	// Last cell we were editting was a Restart Count cell with an invalid value
   if((editCol == APP_RESTART_COUNT) && !ValidRestartCount(editCol, editRow))
   {
   	// Go back to the invalid cell
		gridAppList->Col = editCol;
		gridAppList->Row = editRow;
      ActiveControl = gridAppList;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuEditClick(TObject *Sender)
{
   mnuEditSelectRow->Enabled = !gridAppList->Options.Contains(goRowSelect);
	mnuEditInsert->Enabled = (gridAppList->Options.Contains(goRowSelect) && validClipboard);
	mnuEditInsertBlank->Enabled = gridAppList->Options.Contains(goRowSelect);
	mnuEditDelete->Enabled = (gridAppList->Options.Contains(goRowSelect) &&
		(gridAppList->Row != gridAppList->RowCount - 1));
	mnuEditCopy->Enabled = (gridAppList->Options.Contains(goRowSelect) &&
		(gridAppList->Row != gridAppList->RowCount - 1));
	mnuEditCut->Enabled = (gridAppList->Options.Contains(goRowSelect) &&
		(gridAppList->Row != gridAppList->RowCount - 1));
	mnuEditPaste->Enabled = (gridAppList->Options.Contains(goRowSelect) && validClipboard);
	mnuEditReplicate->Enabled = (gridAppList->Options.Contains(goRowSelect) &&
		(gridAppList->Row != gridAppList->RowCount - 1));
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::popMenuEditPopup(TObject *Sender)
{
	popMenuEditInsert->Enabled = validClipboard;
   popMenuEditDelete->Enabled = (gridAppList->Row != gridAppList->RowCount - 1);
	popMenuEditCopy->Enabled = (gridAppList->Row != gridAppList->RowCount - 1);
	popMenuEditCut->Enabled = (gridAppList->Row != gridAppList->RowCount - 1);
	popMenuEditPaste->Enabled = validClipboard;
	popMenuEditReplicate->Enabled = (gridAppList->Row != gridAppList->RowCount - 1);
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuToolsRenumberClick(TObject *Sender)
{
	int ivrPort = 1;

	for(int row = 1; row < gridAppList->RowCount - 1; row++)
   {
		// Cell value is a number
   	if(atoi(gridAppList->Cells[APP_PORT][row].c_str()))
      {
         // Replace cell value with next available port
			gridAppList->Cells[APP_PORT][row] = ivrPort++;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::ReadRegistry(void)
{  LONG  ret;
   HKEY  hkey;
   DWORD dwLen;
   DWORD dwValue;
   DWORD dwType;
   char  szKey[14];  // Capable of holding "FileHistory %d" for up to 9 rows
   char* pszValue;

   try
   {
   	ret = RegOpenKeyEx (HKEY_LOCAL_MACHINE, TFLINELIST_REG_KEY, 0, KEY_READ, &hkey);
      if(ret != ERROR_SUCCESS)
      {
      	return;
      }

      ret = RegQueryValueEx(hkey, "Top", NULL, &dwType, (LPBYTE) &dwValue, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	Top = dwValue;
      }

      ret = RegQueryValueEx(hkey, "Left", NULL, &dwType, (LPBYTE) &dwValue, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	Left = dwValue;
      }

      ret = RegQueryValueEx(hkey, "Height", NULL, &dwType, (LPBYTE) &dwValue, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	Height = dwValue;
      }

      ret = RegQueryValueEx(hkey, "Width", NULL, &dwType, (LPBYTE) &dwValue, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	Width = dwValue;
      }

      for(int col = 1; col < gridAppList->ColCount; col++)
      {
      	sprintf(szKey, "ColWidth %d", col);

         ret = RegQueryValueEx(hkey, szKey, NULL, &dwType, (LPBYTE) &dwValue, &dwLen);
         if(ret == ERROR_SUCCESS)
         {
         	gridAppList->ColWidths[col] = dwValue;
         }
      }

      for(int row = 1; row < fileHistory->RowCount; row++)
      {
      	sprintf(szKey, "FileHistory %d", row);

         // Get length of string
         ret = RegQueryValueEx(hkey, szKey, NULL, &dwType, NULL, &dwLen);

         if(ret == ERROR_SUCCESS)
         {
         	// Copy the string
            pszValue = new char[dwLen + 1];
            ret = RegQueryValueEx(hkey, szKey, NULL, &dwType, pszValue, &dwLen);

            if(ret == ERROR_SUCCESS)
            {
            	fileHistory->Cells[0][row] = pszValue;
            }

            delete[] pszValue;
         }
      }

   	RegCloseKey(hkey);
   }

   catch(Exception &e)
   {  Application->MessageBox(
         e.Message.c_str(),
         "ReadRegistry",
         MB_OK | MB_ICONERROR);
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::WriteRegistry(void)
{  LONG  ret;
   HKEY  hkey;
   DWORD dwDisposition;
   DWORD dwValue;
   DWORD dwLen;
   char szKey[14];  // Capable of holding "FileHistory %d" for up to 9 rows
   char szValue[255];

   try
   {
   	ret = RegCreateKeyEx(
         HKEY_LOCAL_MACHINE,      // Handle of an open key.
         TFLINELIST_REG_KEY,   // Address of subkey name.
         0,                       // Reserved.
         0,                       // Address of class string.
         REG_OPTION_NON_VOLATILE, // Special options flag.
         KEY_WRITE,               // Desired security access.
         0,                       // Address of key security structure.
         &hkey,                   // Address of buffer for opened handle.
         &dwDisposition);         // Address of disposition value buffer.

      if(ret != ERROR_SUCCESS)
      {
      	Application->MessageBox(
            "Failed to create registry key.",
            "Registry Save Failed",
            MB_ICONERROR | MB_OK);
      }
      else
      {
         ret &= RegSetValueEx(hkey, "Top", 0, REG_DWORD, (LPBYTE) &Top, sizeof(DWORD));

         ret &= RegSetValueEx(hkey, "Left", 0, REG_DWORD, (LPBYTE) &Left, sizeof(DWORD));

         ret &= RegSetValueEx(hkey, "Height", 0, REG_DWORD, (LPBYTE) &Height, sizeof(DWORD));

         ret &= RegSetValueEx(hkey, "Width", 0, REG_DWORD, (LPBYTE) &Width, sizeof(DWORD));

         for(int col = 1; col < gridAppList->ColCount; col++)
         {
         	sprintf(szKey, "ColWidth %d", col);
         	dwValue = gridAppList->ColWidths[col];
            ret &= RegSetValueEx(hkey, szKey, 0, REG_DWORD, (LPBYTE) &dwValue, sizeof(DWORD));
         }

         for(int row = 1; row < fileHistory->RowCount; row++)
         {
         	sprintf(szKey, "FileHistory %d", row);
            sprintf(szValue, "%s", fileHistory->Cells[0][row].c_str());
            dwLen = strlen(szValue) + 1;
            ret &= RegSetValueEx(hkey, szKey, 0, REG_SZ, szValue, dwLen);;
         }

         if(ret != ERROR_SUCCESS)
         {
         	Application->MessageBox(
               "Failed to write host entry.",
               "Registry Save Failed",
               MB_ICONERROR | MB_OK);
         }
      }

      RegCloseKey(hkey);
   }

   catch(Exception &e)
   {
      Application->MessageBox(
         e.Message.c_str(),
         "ReadRegistry",
         MB_OK | MB_ICONERROR);
   }
}
//---------------------------------------------------------------------------

bool __fastcall TFormLineList::SaveAs(void)
{
	if(!ValidCells())
   {
   	return(false);
   }

   dlgSave->FileName = TALFileName;
   dlgSave->Title = "Save TeleFlow Application List File As";
   dlgSave->Filter = "TeleFlow Application List File (*.tal)|*.tal|"
                     "All Files (*.*)|*.*";
   dlgSave->Options.Clear();

   if(dlgSave->Execute())
   {
		Caption = "TeleFlow Line List - " + dlgSave->FileName;

      if(TALFileName != NULL)
      {
      	delete[] TALFileName;
      }

      TALFileName = new char[dlgSave->FileName.Length() + 1];
      strcpy(TALFileName, dlgSave->FileName.c_str());
   	return(SaveTAL(dlgSave->FileName.c_str()));
   }
   else
   {
   	return(false);
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuEditSelectRowClick(TObject *Sender)
{
	// Don't allow editing
	gridAppList->Options >> goEditing;

	// Highlight rows
   gridAppList->Options << goRowSelect;

	// Make the row actually show up as selected since it is
   if(gridAppList->Row > 1)
   {
   	gridAppList->Row--;
      gridAppList->Row++;
   }
   else if(gridAppList->Row != gridAppList->RowCount - 1)
   {
   	gridAppList->Row++;
      gridAppList->Row--;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::HistoryAdd(void)
{
	int index = fileHistory->Cols[0]->IndexOf(TALFileName);

	// file not in list already
	if(index == -1)
   {
		// find next empty row
   	for(index = 1; (index < fileHistory->RowCount) &&
      	(fileHistory->Cells[0][index] != ""); index++);

		// all rows filled so use last row
   	if(index == fileHistory->RowCount)
      {
      	index--;
      }
   }

	// bubble file up
  	for(int row = index; row > 1; row--)
   {
		fileHistory->Cells[0][row] = fileHistory->Cells[0][row - 1];
   }

   fileHistory->Cells[0][1] = TALFileName;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuFileClick(TObject *Sender)
{
	N4->Visible = (fileHistory->Cells[0][1] != "");
	mnuFileHistory1->Caption = fileHistory->Cells[0][1];
   mnuFileHistory1->Visible = (fileHistory->Cells[0][1] != "");
	mnuFileHistory2->Caption = fileHistory->Cells[0][2];
   mnuFileHistory2->Visible = (fileHistory->Cells[0][2] != "");
	mnuFileHistory3->Caption = fileHistory->Cells[0][3];
   mnuFileHistory3->Visible = (fileHistory->Cells[0][3] != "");
	mnuFileHistory4->Caption = fileHistory->Cells[0][4];
   mnuFileHistory4->Visible = (fileHistory->Cells[0][4] != "");
	mnuFileHistory5->Caption = fileHistory->Cells[0][5];
   mnuFileHistory5->Visible = (fileHistory->Cells[0][5] != "");
	mnuFileHistory6->Caption = fileHistory->Cells[0][6];
   mnuFileHistory6->Visible = (fileHistory->Cells[0][6] != "");
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuFileHistory1Click(TObject *Sender)
{
	if(fileChanged && !SaveChanges())
	{
		return;
   }

	LoadTAL(fileHistory->Cells[0][1].c_str());
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuFileHistory2Click(TObject *Sender)
{
	if(fileChanged && !SaveChanges())
	{
		return;
   }

	LoadTAL(fileHistory->Cells[0][2].c_str());
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuFileHistory3Click(TObject *Sender)
{
	if(fileChanged && !SaveChanges())
	{
		return;
   }

	LoadTAL(fileHistory->Cells[0][3].c_str());
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuFileHistory4Click(TObject *Sender)
{
	if(fileChanged && !SaveChanges())
	{
		return;
   }

	LoadTAL(fileHistory->Cells[0][4].c_str());
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuFileHistory5Click(TObject *Sender)
{
	if(fileChanged && !SaveChanges())
	{
		return;
   }

	LoadTAL(fileHistory->Cells[0][5].c_str());
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::mnuFileHistory6Click(TObject *Sender)
{
	if(fileChanged && !SaveChanges())
	{
		return;
   }

	LoadTAL(fileHistory->Cells[0][6].c_str());
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::BitBtnProtocolEnter(TObject *Sender)
{
	int ACol = gridAppList->Col;
   int ARow = gridAppList->Row;

	ComboBoxProtocol->Left = gridAppList->CellRect(ACol, ARow).Left + 3;
	ComboBoxProtocol->Top = gridAppList->CellRect(ACol, ARow).Top + 3;
   ComboBoxProtocol->Width = gridAppList->CellRect(ACol, ARow).Right
  		- gridAppList->CellRect(ACol, ARow).Left;
   ComboBoxProtocol->Visible = true;
	if(ComboBoxProtocol->Items->IndexOf(gridAppList->Cells[ACol][ARow]) != -1)
	{
   	ComboBoxProtocol->ItemIndex = ComboBoxProtocol->Items->IndexOf(gridAppList->Cells[ACol][ARow]);
   }
   else
   {
   	ComboBoxProtocol->ItemIndex = ComboBoxProtocol->Items->IndexOf("lps0");
   }
   ActiveControl = ComboBoxProtocol;
	ComboBoxProtocol->DroppedDown = true;
   BitBtnProtocol->Visible = false;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::BitBtnLogErrorsEnter(TObject *Sender)
{
	int ACol = gridAppList->Col;
   int ARow = gridAppList->Row;

	ComboBoxLogErrors->Left = gridAppList->CellRect(ACol, ARow).Left + 3;
	ComboBoxLogErrors->Top = gridAppList->CellRect(ACol, ARow).Top + 3;
	ComboBoxLogErrors->Width = gridAppList->CellRect(ACol, ARow).Right
 		- gridAppList->CellRect(ACol, ARow).Left;
  	ComboBoxLogErrors->Visible = true;
	if(ComboBoxLogErrors->Items->IndexOf(gridAppList->Cells[ACol][ARow]) != -1)
	{	ComboBoxLogErrors->ItemIndex = ComboBoxLogErrors->Items->IndexOf(gridAppList->Cells[ACol][ARow]);
   }
   else
   {	ComboBoxLogErrors->ItemIndex = ComboBoxLogErrors->Items->IndexOf("Errors only");
   }
  	ActiveControl = ComboBoxLogErrors;
	ComboBoxLogErrors->DroppedDown = true;
	BitBtnLogErrors->Visible = false;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::ComboBoxProtocolEnter(TObject *Sender)
{
	// Let grid know that we just came from another control and should take the
   // appropriate navigation action
	TABFlag = true;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::ComboBoxLogErrorsEnter(TObject *Sender)
{
	// Let grid know that we just came from another control and should take the
   // appropriate navigation action
	TABFlag = true;
}
//---------------------------------------------------------------------------

bool __fastcall TFormLineList::ValidCells(void)
{
   int badCell = 0;

   for(int row = 1; row < gridAppList->RowCount - 1; row++)
	{
   	// Check for valid entries in all cells of this row
		if(gridAppList->Cells[APP_FILE_NAME][row] == "")
		{	badCell = APP_FILE_NAME;
      }
      if(gridAppList->Cells[APP_NAME][row] == "")
      {	badCell = APP_NAME;
      }

      // Move to bad cell and warn user
      if(badCell)
      {	gridAppList->Col = badCell;
      	gridAppList->Row = row;
	   	Application->MessageBox("This field must have a value. TAL file has not been written to disk.",
   	   	"Save TAL halted", MB_OK | MB_ICONERROR);
         return(false);
      }
   }

   return(true);
}
//---------------------------------------------------------------------------

bool __fastcall TFormLineList::ValidAppName(int ACol, int ARow)
{
	// Don't check if column 0, row 0, or last row
   // >= prevents dialog when shutting down grid
	if((ACol == 0) || (ARow == 0) || (ARow >= gridAppList->RowCount - 1))
   {
   	return(true);
   }

	if(gridAppList->Cells[ACol][ARow].Length() >  TELEFLOW_APP_NAME_MAX)
   {
      char* messageString = new char[100];
		sprintf(messageString,
      	"Application name can be at most %d characters and will be truncated at the server",
         TELEFLOW_APP_NAME_MAX);
   	Application->MessageBox(messageString, "Application Name", MB_OK | MB_ICONWARNING);
      delete messageString;

      return(false);
   }

   return(true);
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::gridAppListEnter(TObject *Sender)
{
  	editCol = gridAppList->Col;
	editRow = gridAppList->Row;
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::ButtonSelectTAPMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
	// Does the application file name look like a file?
   if(gridAppList->Cells[APP_FILE_NAME][gridAppList->Row].Pos("."))
   {
		// Seperate out the file name and the directory name
		char* directory = new char[gridAppList->Cells[APP_FILE_NAME][gridAppList->Row].Length() + 1];
      char* file;

      strcpy(directory, gridAppList->Cells[APP_FILE_NAME][gridAppList->Row].c_str());
      if((file = strrchr(directory, '\\')) == NULL)
      {
         dlgOpen->FileName = directory;
         dlgOpen->InitialDir = "";
      }
      else
      {
         file[0] = '\0';
         dlgOpen->FileName = file + 1;
         dlgOpen->InitialDir = directory;
      }

      delete[] directory;
   }
   else
   {
		// Assume that since there was no extension that the application file name
      // is instead a directory
   	dlgOpen->FileName = "";
      dlgOpen->InitialDir = gridAppList->Cells[APP_FILE_NAME][gridAppList->Row];
   }

   dlgOpen->Title = "Select TeleFlow Application File";
   dlgOpen->Filter = "TeleFlow Application File (*.tap)|*.tap|"
                     "All Files (*.*)|*.*";
   dlgOpen->Options.Clear();
   dlgOpen->Options << ofFileMustExist << ofHideReadOnly << ofNoChangeDir;

   if(dlgOpen->Execute())
   {
	   gridAppList->Cells[APP_FILE_NAME][gridAppList->Row] = dlgOpen->FileName;
		fileChanged = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormLineList::LineListHelp1Click(TObject *Sender)
{
   // function in ESCUTIL: help_fns.cpp, opens help file and links to topic requested
   HelpLink("ServerConfig_LineList.htm");
}
//---------------------------------------------------------------------------

