#include <stdio.h>
#include <process.h>

#include <string.h>
#include <string>
using std::string;

#include "ivr_defs.h"	// TELEFLOW_APP_NAME_MAX
#include "license_read.h"  // TeleFlow license information
#include "..\TFEngine\brd_load.h"
#include "srvr_eng.h"
#include "debugopt.h"
#include "TFApplicationsManager.h"

#include "tf_registry.h"

TFApplicationsManager::TFApplicationsManager(CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID>* commHub,
   TCPClientManager* tcpClientManager, FileLog& sysLog, HINSTANCE boardLibrary)
   : commHub(commHub), tcpClientManager(tcpClientManager), sysLog(sysLog), boardLibrary(boardLibrary)
{
}

TFApplicationsManager::~TFApplicationsManager(void)
{
   RemoveApplications();
}

void TFApplicationsManager::AddApplication(int appID, TFApplication* application)
{
   if(application == NULL)
   {
      return;
   }

   mutex.Get();

   if(applications.count(appID) != 0)
   {
      delete applications[appID];
   }

   applications[appID] = application;
   Add(applications[appID]->info.messages);

   mutex.Release();
}

bool TFApplicationsManager::Empty(void)
{
   bool result;

   mutex.Get();
   result = applications.empty();
   mutex.Release();

   return(result);
}

// -------------------------------------------------------------------------
// Name:        IVRApps::LoadTAL
//
// Purpose:     Calls the correct file loader for the file format.
// -------------------------------------------------------------------------
bool TFApplicationsManager::LoadTAL(void)
{
   char* value;
   string installationDirectory;
   string logDirectory;
   string TALFileName;
   DWORD logLines;
   DWORD maxLogFileSize;
   DWORD logSRAudio;
   TFLICENSESTRUCT tfLicense;
   FILE* fTAL;
   char szLine[1024];   // String to hold field read in from file.
   char szVersion[4];   // Format expected: "XXX"
   int iOffset;         // Offset into szLine.
   int ch;              // Character read in from file.
   int iFieldNo;        // Current field being read from file.
   int appID;
   bool bHaveApp = false;
   int version;
   int iPortsInTAL = 0;  // Counts the total ports in TAL file regardless of iPort identifying value
   int iLowestSIPPort = MAX_LINES + 1;  // Find the actual starting point of a Port using 'sip0' protocol
   int iHighestNonSIPPort = 0;  // Used to find highest traditional telephony port

   // get registry information
   HKEY  hkey;
   DWORD dwLen;
   DWORD dwType;

   tfLicense = ReadLicense(&sysLog);

   if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, TFSERVER_REG_KEY, NULL, KEY_READ, &hkey) != ERROR_SUCCESS)
   {
      sysLog.Print("ERROR: Failed to open registry\n");
      return(false);
   }

   // **** InstallationPath ***************************************************
   if(RegQueryValueEx(hkey, "InstallDirectory", NULL, &dwType, NULL, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Print("ERROR: Failed to find InstallDirectory key in registry\n");
      return(false);
   }

   value = new char[dwLen + 1];
   if(RegQueryValueEx(hkey, "InstallDirectory", NULL, &dwType, (unsigned char*) value, &dwLen) != ERROR_SUCCESS)
   {
      delete[] value;
      sysLog.Print("ERROR: Failed to load InstallDirectory from registry\n");
      return(false);
   }
   else
   {
      installationDirectory = value;
      delete[] value;
   }

   // **** LogDirectory *******************************************************
   if(RegQueryValueEx(hkey, "LogDirectory", NULL, &dwType, NULL, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Print("ERROR: Failed to find LogDirectory key in registry\n");
      return(false);
   }

   value = new char[dwLen + 1];
   if(RegQueryValueEx(hkey, "LogDirectory", NULL, &dwType, (unsigned char*) value, &dwLen) != ERROR_SUCCESS)
   {
      delete[] value;
      sysLog.Print("ERROR: Failed to load LogDirectory from registry\n");
      return(false);
   }
   else
   {
      logDirectory = value;
      delete[] value;
   }

   // **** AppListFileName ****************************************************
   if(RegQueryValueEx(hkey, "AppListFileName", NULL, &dwType, NULL, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Print("ERROR: Failed to find AppListFileName key in registry\n");
      return(false);
   }

   value = new char[dwLen + 1];
   if(RegQueryValueEx(hkey, "AppListFileName", NULL, &dwType, (unsigned char*) value, &dwLen) != ERROR_SUCCESS)
   {
      delete[] value;
      sysLog.Print("ERROR: Failed to load AppListFileName from registry\n");
      return(false);
   }
   else
   {
      TALFileName = value;
      delete[] value;
   }

   // **** LogLines ***********************************************************
   dwLen = sizeof(DWORD);
   if(RegQueryValueEx(hkey, "LogLines", NULL, &dwType, (LPBYTE) &logLines, &dwLen) != ERROR_SUCCESS)
   {
      logLines = 200;
   }
   else if(logLines < 50)
   {
      logLines = 50;
   }
   else if(logLines > 20000)
   {
      logLines = 20000;
   }

   // **** MaxLogFileSize *****************************************************
   dwLen = sizeof(DWORD);
   if(RegQueryValueEx(hkey, "MaxLogFileSize", NULL, &dwType, (LPBYTE) &maxLogFileSize, &dwLen) != ERROR_SUCCESS)
   {
      maxLogFileSize = 500;
   }

   // **** LogSRAudio *********************************************************
   dwLen = sizeof(DWORD);
   if(RegQueryValueEx(hkey, "LogSRAudio", NULL, &dwType, (LPBYTE) &logSRAudio, &dwLen) != ERROR_SUCCESS)
   {
      // Off by default
      logSRAudio = 0;
   }

   mutex.Get();

	// Unable to open file
   if((fTAL = fopen(TALFileName.c_str(), "r")) == NULL)
   {
      sysLog.Print("ERROR: Unable to open TeleFlow Application List file\n");
      sysLog.Print("    '%s'.  Please ensure that the Active\n", TALFileName.c_str());
      sysLog.Print("    TeleFlow Application List file is correctly specified in TFConfig.\n");
      mutex.Release();
      return(false);
   }

   fgets(szLine, 1024, fTAL);
   if(strncmp(szLine, "TAL Version: ", 13) == 0)
   {
      strncpy(szVersion, &(szLine[13]), 4);
      szVersion[3] = '\0';

      version = atoi(szVersion);
      if(version < 2)
      {
         sysLog.Print("ERROR: Invalid version! Headers weren't included until version 2\n");
         mutex.Release();
         return(false);
      }

      ch = fgetc(fTAL);
      iOffset = 0;
   }
   else
   {
      version = 1;
      ch = '\n';
      iOffset = strlen(szLine);
   }

   iFieldNo = 1;

	while(!feof(fTAL))
	{
      if(ch == '\t' || ch == '\r' || ch == '\n') // ----- Found end of field
      {
         szLine[iOffset] = '\0';

         // Version filter
         // - current version is now 4 - TF Feb.19/02
         switch(version)
         {
         	case 1:
            {
            	switch(iFieldNo)
               {
               	case 5:	iFieldNo = 6;	break;
               	case 7: iFieldNo = 11; break;
               }
            }

            case 2:
            {
            	switch(iFieldNo)
               {
               	case 5: iFieldNo = 6; break;
               	case 10: iFieldNo = 11; break;
               }
            }

            case 3:
            {
            	switch(iFieldNo)
               {
               	case 10: iFieldNo = 11; break;
               }
            }
         }

         switch(iFieldNo)
         {
            case 1:  // ----------- FIELD 1: --------------------------- iAppID
            {
               appID = atoi(szLine);
               // Create new application
               applications[appID] = new TFApplication(appID, new GUILog(maxLogFileSize, logLines));
               break;
            }

            case 2:  // ----------- FIELD 2: -------------------- pszDebugParam
            {
               if((szLine[0] == '-') && (szLine[1] == 'Z'))
               {
                  for(int index = 0; szLine[index] != '\0'; index++)
                  {
                     applications[appID]->info.ivLog->SetDebug(szLine[index]);
                  }
               }
               applications[appID]->info.ivLog->SetDebug(1);  // Allow "true" to be passed to Debug.
               SetupDebugOptions(applications[appID]->info.ivLog);
               break;
            }

            case 3:  // ----------- FIELD 3: ------------------------- pszAppNm
            {
					// Do not copy more than allowable TELEFLOW_APP_NAME_MAX characters as defined in ivr_defs.h
               applications[appID]->info.pszAppNm = new char[TELEFLOW_APP_NAME_MAX + 1];
               strncpy(applications[appID]->info.pszAppNm, szLine, TELEFLOW_APP_NAME_MAX);
               applications[appID]->info.pszAppNm[TELEFLOW_APP_NAME_MAX] = '\0';
               break;
            }

            case 4:  // ----------- FIELD 4: ------------------------ pszFileNm
            {
               applications[appID]->info.pszAppFile = new char[iOffset + 1];
               strcpy(applications[appID]->info.pszAppFile, szLine);
               break;
            }

            case 5:  // ----------- FIELD 5: ---------------------- pszProtocol
         	{
               applications[appID]->info.pszProtocol = new char[iOffset + 1];
               strcpy(applications[appID]->info.pszProtocol, szLine);
               break;
            }

            case 6:  // ----------- FIELD 6: -------------------- iRestartCount
         	{
               applications[appID]->info.restartMax = atoi(szLine);
               bHaveApp = true;
               break;
            }

            case 7:  // ----------- FIELD 7: ----------------- bRestartInfinite
         	{
               applications[appID]->info.bRestartInfinite = (szLine[0] == 'Y') ? true : false;
               break;
            }

            case 8:  // ----------- FIELD 8: ----------------------- bLogOutput
            {
               applications[appID]->info.bLogOutput = (szLine[0] == 'Y') ? true : false;
               break;
            }

            case 9:  // ----------- FIELD 9: ----------------------- bLogAppend
            {
               applications[appID]->info.bLogAppend = (szLine[0] == 'Y') ? true : false;
               break;
            }

				case 10: // ----------- FIELD 10: ---------------------- iPort
            {
               int iPort = atoi(szLine);

               applications[appID]->info.iPort = iPort;
               if(iPort > 0)
               {
                  iPortsInTAL++;
                  if(iPortsInTAL <= tfLicense.numports)
                  {
                     applications[appID]->info.portLicense = tfLicense.licensetype;
                  }
               }

               // Update the point (by port number) at which the traditional telephony ports end
               // and VoIP ports start, or are allowed to start
               if(stricmp(applications[appID]->info.pszProtocol,"sip0") == 0)
               {
                  if(iPort < iLowestSIPPort)
                  {
                     iLowestSIPPort = iPort;
                  }
               }
               else
               {
                  if(iPort > iHighestNonSIPPort)
                  {
                     iHighestNonSIPPort = iPort;
                     iLowestSIPPort = MAX_LINES + 1;  // not allowed to be lower than highest traditional port
                  }
               }

               break;
            }

            default: // ----------- FIELDS 11 to 10 + MAX_PARAMETERS: ----- Optional params (max MAX_PARAMETERS)
            {
               if(applications[appID]->info.iParamCount < MAX_PARAMETERS)
               {
                  applications[appID]->info.pszParams[iFieldNo - 11] = new char[iOffset + 1];
                  strcpy(applications[appID]->info.pszParams[iFieldNo - 11], szLine);
                  applications[appID]->info.iParamCount++;
               }
            }
         }

         // Reset variables to receive next field.
         iOffset = 0;
         if((ch == '\r') || (ch == '\n'))
         {
            iFieldNo = 1;

            if(bHaveApp)
            {
     	         if(!logDirectory.empty())
               {
        	         applications[appID]->info.pszLogDir = new char[logDirectory.length() + 1];
           	      strcpy(applications[appID]->info.pszLogDir, logDirectory.c_str());
              	}
               else
               {
                 	applications[appID]->info.pszLogDir = new char[2];
                  strcpy(applications[appID]->info.pszLogDir, "\\");
  	            }

              	applications[appID]->info.pszInstallationPath = new char[installationDirectory.length() + 1];
               strcpy(applications[appID]->info.pszInstallationPath, installationDirectory.c_str());

					// Fields not in earlier version files
					switch(version)
               {
						case 1:
                  {
			            applications[appID]->info.bRestartInfinite = false;
      			      applications[appID]->info.bLogOutput = true;
         	   		applications[appID]->info.bLogAppend = true;
                     // Fall through
                  }
                  case 2:
                  {
							applications[appID]->info.pszProtocol = new char[5];
							strcpy(applications[appID]->info.pszProtocol, "lps0");
                     // Fall through
                  }
                  case 3:
                  {
							if(applications[appID]->info.pszParams[0])
                     {
								applications[appID]->info.iPort = atoi(applications[appID]->info.pszParams[0]);
                     }
                     else
                     {
                     	applications[appID]->info.iPort = 0;
                     }
                  }
               }

					// Make all @P variables exist
        	      for(; applications[appID]->info.iParamCount < MAX_PARAMETERS; applications[appID]->info.iParamCount++)
					{
              		applications[appID]->info.pszParams[applications[appID]->info.iParamCount] = new char[1];
                  strcpy(applications[appID]->info.pszParams[applications[appID]->info.iParamCount], "");
               }

        	      if(applications[appID]->info.bLogOutput)
               {
           	      strcpy(szLine, logDirectory.c_str());
                 	strcat(szLine, applications[appID]->info.pszAppNm);
  	               applications[appID]->info.ivLog->Open(szLine, applications[appID]->info.bLogAppend, true);
     	         }

               applications[appID]->info.loadingMutex = &mutex;
               applications[appID]->info.boardLibrary = boardLibrary;
               applications[appID]->info.logSRAudio = logSRAudio != 0;
               applications[appID]->info.commHub = commHub;
               Add(applications[appID]->info.messages);
               applications[appID]->info.tcpClientManager = tcpClientManager;
               bHaveApp = false;
            }
            else
            {
               delete applications[appID];
               applications.erase(appID);
            }
         }
         else
         {
            ++iFieldNo;
         }
      }
      else
      {
         // Build up field data until the end of the field is found.
         szLine[iOffset] = (char) ch;
         ++iOffset;
      }
      ch = fgetc(fTAL);
   } // while

   fclose(fTAL);

   // Report expected board usage according to TAL file
   sysLog.Print("   Total Ports in TAL: %i\n", iPortsInTAL);
   sysLog.Print("   Traditional telephony to Port %i\n", iHighestNonSIPPort);
   if(iLowestSIPPort <= MAX_LINES)
   {
      sysLog.Print("   VoIP (SIP) starts at Port %i\n", iLowestSIPPort);
   }
   else
   {
      sysLog.Print("   VoIP (SIP) not used\n");
   }

   for(map<int, TFApplication*>::iterator app = applications.begin(); app != applications.end(); app++)
   {
      // Each application's IVRINFO structure has to know these values
      // in order for the board query to work properly
      app->second->info.iLastNonSIPPort = iHighestNonSIPPort;  // needed by board query

      if(CreateInstance != NULL)
      {
         app->second->info.ivBoard = CreateInstance(&app->second->info);
      }
      app->second->info.variables->SetBoard(app->second->info.ivBoard);
   }

   mutex.Release();

   if(applications.empty())
   {
      sysLog.Print("ERROR: No applications were loaded from the TeleFlow Application List file\n");
      sysLog.Print("   '%s'.   Please verify\n", TALFileName.c_str());
      sysLog.Print("   the Active TeleFlow Application List file in TFLineList.\n");
      return(false);
   }

   return(true);
}

void TFApplicationsManager::PrintRunningApplications(void)
{
   mutex.Get();
   for(map<int, TFApplication*>::iterator app = applications.begin(); app != applications.end(); app++)
   {
      sysLog.Print("Engine thread %d is still running\n", app->first);
      sysLog.Print("      Port %d Application '%s'\n", app->second->info.iPort, app->second->info.pszAppFile);
      printf("Engine thread %d is still running\n", app->first);
      printf("      Port %d Application '%s'\n", app->second->info.iPort, app->second->info.pszAppFile);
   }
   mutex.Release();
}

void TFApplicationsManager::RemoveApplication(int appID)
{
   mutex.Get();
   if(applications.count(appID) != 0)
   {
      Remove(appID);
      delete applications[appID];
      applications.erase(appID);
   }
   mutex.Release();
}

void TFApplicationsManager::RemoveApplications(void)
{
   mutex.Get();
   for(map<int, TFApplication*>::iterator app  = applications.begin(); app != applications.end(); app++)
   {
      Remove(app->first);
      delete app->second;
   }
   applications.clear();
   mutex.Release();
}

bool TFApplicationsManager::StartApplication(int appID)
{
   unsigned threadId;
   bool result = false;

   mutex.Get();
   if(applications.count(appID) == 0)
   {
      sysLog.Print("   Application %d does not exist\n", appID);
   }
   else if(_beginthreadex(NULL, 0, ServerEngineMain, &applications[appID]->info, 0, &threadId) == (unsigned long) -1)
   {
      sysLog.Print("   Failed to create Engine thread\n");
      sysLog.Print("      Port %d Application '%s'\n", applications[appID]->info.iPort, applications[appID]->info.pszAppFile);
   }
   else
   {
      result = true;
   }
   mutex.Release();

   return(result);
}

bool TFApplicationsManager::StartApplications(void)
{
   unsigned threadId;
   bool result = true;

   // This is the same mutex that is used in the Engine threads as *loadingMutex
   mutex.Get();
   for(map<int, TFApplication*>::iterator app = applications.begin(); app != applications.end(); app++)
   {
      sysLog.Print("   AppID %i %s: %s, Port %i, protocol '%s'\n", app->second->info.iAppID,
                   app->second->info.pszAppNm, app->second->info.pszAppFile,
                   app->second->info.iPort, app->second->info.pszProtocol );
      //
      if(_beginthreadex(NULL, 0, ServerEngineMain, &app->second->info, 0, &threadId) == (unsigned long) -1)
      {
         sysLog.Print("   Failed to create Engine thread\n");
         sysLog.Print("      Port %d Application '%s'\n", app->second->info.iPort, app->second->info.pszAppFile);
         result = false;
      }
      Sleep(10);  // sleep between launching each thread
   }
   mutex.Release();

   return(result);
}

static unsigned int _stdcall SimulatorEngine(void* params)
{
   PIVRINFO pInfo = (PIVRINFO) params;
   TFServerCommunicationPacket::TF_SERVER_EVENT result;

   // Run the engine and send the result in a message to the server so that the
   // simulator knows when the thread has actually finished, can clean up and
   // start new TFEngines
   result = EngineMain(pInfo);
   pInfo->commHub->SendMessage(TFServerCommunicationPacket::TF_SERVER, new TFServerCommunicationPacket(result, pInfo->iAppID));

   return(0);
};

bool TFApplicationsManager::StartSimulation(int appID)
{
   unsigned threadId;
   bool result = false;

   mutex.Get();
   if(applications.count(appID) == 0)
   {
      sysLog.Print("   Application %d does not exist\n", appID);
   }
   else
   {
      applications[appID]->info.bRestart = true;
      applications[appID]->info.bTerminate = false;
      applications[appID]->info.reloadRequested = false;
      applications[appID]->info.shutdownRequested = false;
      applications[appID]->info.safeToShutdown = true;
      applications[appID]->info.appFlags.status = lsEmpty;

      if(_beginthreadex(NULL, 0, SimulatorEngine, &applications[appID]->info, 0, &threadId) == (unsigned long) -1)
      {
         sysLog.Print("   Failed to create Engine thread\n");
         sysLog.Print("      Port %d Application '%s'\n", applications[appID]->info.iPort, applications[appID]->info.pszAppFile);
      }
      else
      {
         result = true;
      }
   }
   mutex.Release();

   return(result);
}

void TFApplicationsManager::StopApplication(int appID)
{
   commHub->SendMessage(TFServerCommunicationPacket::TF_ENGINE_MANAGER, new TFEngineCommunicationPacket(appID, TFEngineCommunicationPacket::SRV_EVT_TERMINATE));
}

void TFApplicationsManager::StopApplications(void)
{
   commHub->SendMessage(TFServerCommunicationPacket::TF_ENGINE_MANAGER, new TFEngineCommunicationPacket(0, TFEngineCommunicationPacket::SRV_EVT_TERMINATE));
}

