/* -------------------------------------------------------------------------
   File:        SRVR_ENG.CPP

   Warning:     - All files must be compiled with 4-byte alignment.

   Questions:   - Why do all files need to be compiled with 4-byte
                  alignment?

   DDTK:        Include the following files:
                   loadddtk.cpp
                   ddtk_db.cpp

   BDE32:       Include the following files:
                   newdb.cpp
                   \Borland\BDE\TUTIL32.LIB

                The project path must include:
                   include: $(BCB)\include\vcl

   Copyright (c) 1993-2001 enGenic. All rights reserved.
   ------------------------------------------------------------------------- */
#include "ivr_defs.h"	// TELEFLOW_APP_NAME_MAX

#include <windows.h>

#include "tap.h"
#include "initglobals.h"
#include "debugopt.h"
#include "srvr_eng.h"
#include "TFServer_communication.h"
#include "..\TFEngine\TFEngine_communication.h"

/* -------------------------------------------------------------------------
   Name:        ServerEngineMain

   Purpose:     Starts an instance of the Engine running.  This thread
                should not terminate unless the Service is shutting down.

   Steps:       Loop
                   Try
                      Initialize variables.
                      Run the Engine.
                   except
                      Flag Engine as crashed.
                      ** These should never happen.
                   finally
                      Destroy variables.
                   end try
                   if (!bRestart)
                      Wait for events
                end loop

   Notes:       - If a shutdown or reload event arrives, store the shutdown
                  event into PIVRINFO structure.  The Engine will shutdown
                  when appropriate.
                - If there is a serious problem starting the Engine, the
                  primary thread will not be informed that the Engine didn't
                  start.  Therefore this thread will not try and start
                  again.
   ------------------------------------------------------------------------- */
unsigned _stdcall ServerEngineMain(void* arg)
{
   PIVRINFO pInfo = (PIVRINFO) arg;

   // Unable to start Engine thread.  Don't inform server since this
   // the Server simply try and restart it.
   if(arg == NULL)
   {
      return(0);
   }

   // Continuously reload Engine until shutdown.
   for(;;)
   {
      EngineMain((PIVRINFO) arg);

      // - If the application terminates (normally or by crashing) it will
      //   restart unless the user chose to shutdown the application or the
      //   primary thread informed the Engine to terminate.
      // - If an exception occurs, the application will terminate.
      // - I've put a limit on the number of times the application can restart
      //   if it terminates with errors.
      // - If not told to terminate, the Engine thread will wait until it is
      //   told to reload or told to terminate.
      if(pInfo->bTerminate)
      {
         TFServerCommunicationPacket* message = new TFServerCommunicationPacket(TFServerCommunicationPacket::SRV_EVT_ENG_TERMINATED, pInfo->iAppID);

         // Send a message to the primary thread before returning.
         pInfo->commHub->SendMessage(TFServerCommunicationPacket::TF_SERVER, message);
         return(0);
      }

      if(pInfo->reloadRequested)
      {
         pInfo->bRestart = true;
      }
      
      while(!pInfo->bRestart)
      {
         DWORD ret;
         TFEngineCommunicationPacket* message = NULL;
         TFEngineCommunicationPacket::TF_ENGINE_EVENT event;

         ret = WaitForSingleObject(pInfo->messages->GetMessageSignal(), INFINITE);

         if(ret == WAIT_OBJECT_0)
         {
            message = (TFEngineCommunicationPacket*) pInfo->messages->ReceiveMessage();
            event = message->event;
            delete message;
         }

         if((ret != WAIT_OBJECT_0) || (message == NULL))
         {
            TFServerCommunicationPacket* message = new TFServerCommunicationPacket(TFServerCommunicationPacket::SRV_EVT_ENG_FATAL_ERROR, pInfo->iAppID);

            // Send a message to the primary thread before returning.
            pInfo->commHub->SendMessage(TFServerCommunicationPacket::TF_SERVER, message);
            return(0);
         }

         if((event == TFEngineCommunicationPacket::SRV_EVT_RELOAD_NORMAL) || (event == TFEngineCommunicationPacket::SRV_EVT_RELOAD_ABORT))
			{
            pInfo->bRestart = true;
            pInfo->restartCnt = 0;
         }

         if(event == TFEngineCommunicationPacket::SRV_EVT_TERMINATE)
         {
            TFServerCommunicationPacket* message = new TFServerCommunicationPacket(TFServerCommunicationPacket::SRV_EVT_ENG_TERMINATED, pInfo->iAppID);

            pInfo->bTerminate = true;

            // Send a message to the primary thread before returning.
            pInfo->commHub->SendMessage(TFServerCommunicationPacket::TF_SERVER, message);

            // Time to terminate
            return(0);
         }
      }
   } // for (;;)
}
/*ServerEngineMain*/
