#include <fstream>
using std::ofstream;

#include <io.h>      // access
#include <dir.h>     // mkdir

#include "Voice.h"

Voice::Voice(unsigned int applid, Log* ivLog, BoardGeneric* board)
   : applid(applid), ivLog(ivLog), ivBoard(board)
{
   isPendingSRAudio = false;
   isPendingSRResult = false;
   isReceivingSRAudio = false;
   nextSRAudioBufferIndex = 0;
   SRAudioBeginBufferIndex = 0;
   SRAudioLength = 0;
   // srResource set in SRAcquire
   // telephonyPort set in SRAcquire
   // logSRAudio set in SRAcquire
   // logDir set in SRAcquire
   // currentTAMName updated by SRUpdateTAMInfo
   // lastPlayedStep updated by SRCapturePlayStats
   // lastExecPlayStep updated by SRCapturePlayStats
   // lastSRStep updated by SRCaptureStartStats
   // novoiceTimeout updated by SRCaptureStartStats
   // endvoiceTimeout updated by SRCaptureStartStats
   // thresholdConf updated by SRCaptureStartStats
   // endvoiceTimeout updated by SRCaptureStartStats
   // retainAudio updated by SRCaptureStartStats
   // grammarType updated by SRCaptureStartStats
   // grammarBase updated by SRCaptureStartStats
   // grammarBaseData updated by SRCaptureStartStats
   // grammarAdd updated by SRCaptureStartStats
   // SRHandle updated by SRCreateHandle
   // SRHeard updated by SRGetResult or SRCaptureResult
   // SRAdjusted updated by SRCaptureResult
   // SRConfidence updated by SRCaptureResult
   // SRReturn updated by SRCaptureResult
   // termDigit updated by SRCaptureResult
}

Voice::~Voice(void)
{
}

IV_STATUS Voice::SRAcquire(const char*, int, const char*, const char*, bool)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS Voice::SRAddGrammar(Voice::SR_GRAMMAR, const char*)
{
   return(IVR_NOTSUPPORTED);
}

void Voice::SRCapturePlayStats(const char* pszAppFile,
   int iStepNum, const char* pszStepName, const char* pszScriptcode,
   const char* pszFileName)
{
   const float  minHeardDuration = 0.700;    // seconds

   lastExecPlayStep.stepStat.TAPName  = pszAppFile;
   lastExecPlayStep.stepStat.TAMName  = currentTAMName;
   lastExecPlayStep.stepStat.stepNum  = iStepNum;
   lastExecPlayStep.stepStat.stepName = pszStepName;

   if(pszScriptcode != NULL)
   {
      lastExecPlayStep.script         = pszScriptcode;
   }
   else
   {
      lastExecPlayStep.script         = "";
   }

   lastExecPlayStep.file              = pszFileName;
   lastExecPlayStep.completed         = ivBoard->GetPlayCompleted();
   lastExecPlayStep.played_sec        = ivBoard->GetPlayElapsedTime();

   if((lastExecPlayStep.completed) || (lastExecPlayStep.played_sec >= minHeardDuration))
   {
      lastPlayedStep = lastExecPlayStep;
   }
}

// Capture SR result for SpeechTrainer
void Voice::SRCaptureResult(const char cDig,               // touch tone digit
                                    const char* pszConcept,        // result of recognition
                                    unsigned int iConfidence,      // confidence
                                    IV_STATUS status)              // result
{
   // Capture result statistics
   termDigit = cDig;
   SRAdjusted = (pszConcept == NULL) ? "" : pszConcept;
   if(SRHeard.empty())
   {
      SRHeard = SRAdjusted;
   }
   SRConfidence = iConfidence;
   SRReturn = status;

   // Failure, timeout, or touch tone result
   if((SRReturn == IVR_FAIL) || (SRReturn == IVR_TIMEOUT) || (cDig != '\0'))
   {
      SRAudioStartTime.Update();
      SRAudioEndTime.Update();
   }

   // Write the output file
   if(logSRAudio)
   {
      SRWriteData();    // report stats to file
   }
}

void Voice::SRCaptureStartStats(const char* pszAppFile,   // TAP name
                                   int iStepNum,                  // step number
                                   const char* pszStepName,       // Step name
                                   int iNoVoiceTimeout,           // no voice t/o
                                   unsigned int iEndVoiceTimeout, // end speech t/o
                                   unsigned int iThresholdConf,   // threshold confidence
                                   bool bRetainAudio,             // retainAudio
                                   Voice::SR_GRAMMAR srgType,     // grammar type
                                   const char* pszBaseGrammar,    // base grammar name/file
                                   const char* pszAddGrammar)     // additional grammar text
{
   string pathedDataName;

   SRInvalidateStatistics();
   SRCreateHandle();

   pathedDataName = SRGetTrainerPath() + SRGetTrainerDataName();
   
   lastSRStep.TAPName  = pszAppFile;
   lastSRStep.TAMName  = currentTAMName;
   lastSRStep.stepNum  = iStepNum;
   lastSRStep.stepName = pszStepName;

   novoiceTimeout = iNoVoiceTimeout;
   endvoiceTimeout = iEndVoiceTimeout;
   thresholdConf = iThresholdConf;
   retainAudio = bRetainAudio;

   grammarType = srgType;

   switch(grammarType)
   {
      case Voice::YES_NO:
      {
         baseGrammarIdentifier = "Internal:YES_NO";
         break;
      }

      case Voice::DIGITS:
      {
         baseGrammarIdentifier = "Internal:DIGITS";
         break;
      }

      case Voice::FILE:
      {
         if(pszBaseGrammar != NULL)
         {
            baseGrammarIdentifier = pszBaseGrammar;
            baseGrammarFileName = SRGetBaseGrammarFileName();

            if(!CopyFile(pszBaseGrammar, (SRGetTrainerPath() + baseGrammarFileName).c_str(), false))
            {
               ivLog->Print("      WARNING: Unable to copy grammar file '%s' to '%s'\n", pszBaseGrammar, (SRGetTrainerPath() + baseGrammarFileName).c_str());
               baseGrammarFileName.clear();
            }
         }
         else
         {
            baseGrammarIdentifier.clear();
            baseGrammarFileName.clear();
         }

         break;
      }

      case Voice::TEXT:
      {
         baseGrammarIdentifier.clear();

         if(pszBaseGrammar != NULL)
         {
            ofstream grammarFile;

            baseGrammarFileName = SRGetBaseGrammarFileName();
            grammarFile.open((SRGetTrainerPath() + baseGrammarFileName).c_str(), std::ios::app);

            if(grammarFile.is_open())
            {
               grammarFile.write(pszBaseGrammar, strlen(pszBaseGrammar));
               grammarFile.close();
            }
            else
            {
               ivLog->Print("      WARNING: Unable to write grammar file '%s'\n", (SRGetTrainerPath() + baseGrammarFileName).c_str());
               baseGrammarFileName.clear();
            }
         }
         else
         {
            baseGrammarFileName.clear();
         }

         break;
      }

      default:
      {
         baseGrammarIdentifier.clear();
         baseGrammarFileName.clear();
      }
   }

   if(pszAddGrammar != NULL)
   {
      ofstream grammarFile;

      additionalGrammarFileName = SRGetAdditionalGrammarFileName();
      grammarFile.open((SRGetTrainerPath() + additionalGrammarFileName).c_str(), std::ios::app);

      if(grammarFile.is_open())
      {
         grammarFile.write(pszAddGrammar, strlen(pszAddGrammar));
         grammarFile.close();
      }
      else
      {
         ivLog->Print("      WARNING: Unable to write grammar file '%s'\n", (SRGetTrainerPath() + additionalGrammarFileName).c_str());
         additionalGrammarFileName.clear();
      }
   }
   else
   {
      additionalGrammarFileName.clear();
   }

   // Results are undefined until audio is processed by recognizer
   SRHeard.clear();
   SRAdjusted.clear();
   SRConfidence = 0;
}

IV_STATUS Voice::SRClearAudio(void)
{
   isPendingSRAudio = false;
   isPendingSRResult = false;
   isReceivingSRAudio = false;
   SRAudioLength = 0;
   return(IVR_SUCCESS);
}

string Voice::SRCreateHandle(void)
{
   ESCTime sysTime;
   char name[28];
   string SRTrainerPath;

   sysTime.Update();

   // e.g. 2005-11-09_10.51.42.342_001
   sprintf(name, "%s_%03d", sysTime.GetFileNameDate(), telephonyPort);
   SRHandle = name;

   SRTrainerPath = SRGetTrainerPath();

   // Create folder under log directory
   if(access(SRTrainerPath.c_str(), 0))
   {
      if(mkdir(SRTrainerPath.c_str()) != 0)
      {
         ivLog->Print ("      WARNING: SpeechTrainer failed to create folder '%s' \n", SRTrainerPath.c_str());
      }
   }

   return(SRHandle);
}

string Voice::SRGetAudioFileName(string handle) const
{
   // e.g. 2005-11-09_10.51.42.342_001.wav
   if(handle.empty())
   {
      handle = SRHandle;
   }

   return(handle + (string) ".wav");
}

string Voice::SRGetAdditionalGrammarFileName(string handle) const
{
   // e.g. 2005-11-09_10.51.42.342_001.grmr2
   if(handle.empty())
   {
      handle = SRHandle;
   }

   return(handle + (string) ".grmr2");
}

string Voice::SRGetBaseGrammarFileName(string handle) const
{
   // e.g. 2005-11-09_10.51.42.342_001.grmr1
   if(handle.empty())
   {
      handle = SRHandle;
   }

   return(handle + (string) ".grmr1");
}

unsigned int Voice::SRGetHandleLength(void) const
{
   return(27);
}

string Voice::SRGetTrainerDataName(string handle) const
{
   // e.g. 2005-11-09_10.51.42.342_001.csv
   if(handle.empty())
   {
      handle = SRHandle;
   }

   if(handle.length() != SRGetHandleLength())
   {
      return("");
   }

   return(handle.substr(0, 10) + handle.substr(23, 4) + (string) ".csv");
}

string Voice::SRGetTrainerPath(string handle) const
{
   // e.g. <logDir>\\ST_2005-11-09\\
   string trainerPath = logDir;

   if(handle.empty())
   {
      handle = SRHandle;
   }

   if(handle.length() != SRGetHandleLength())
   {
      return(trainerPath);
   }

   trainerPath += "ST_";
   trainerPath += handle.substr(0, 10);
   trainerPath += "\\";

   return(trainerPath);
}

IV_STATUS Voice::SREnd(void)
{
   return(IVR_NOTSUPPORTED);
}

// Flag a previous SR result for review in SpeechTrainer GUI application
bool Voice::SRFlagResult(const char* SRHandle)
{
   string fileName;
   string SRHandleString;
   string handleDate;
   string port;
   ofstream speechTrainerFile;

   if((SRHandle == NULL) || (strlen(SRHandle) != SRGetHandleLength()))
   {
      return(false);
   }

   // e.g. 2005-11-09_10.51.42.342_port_001
   SRHandleString = SRHandle;

   fileName = SRGetTrainerPath(SRHandleString) + SRGetTrainerDataName(SRHandleString);

   // Open file
   speechTrainerFile.open(fileName.c_str(), std::ios::app);
   if(speechTrainerFile.is_open())
   {
      // 43  SR Handle
      speechTrainerFile << ",,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,\"";
      speechTrainerFile << SRHandleString;
      speechTrainerFile << "\"\n";
      speechTrainerFile.close();
   }
   else
   {
      ivLog->Print ("      ERROR: SpeechTrainer failed to write to recognition file '%s'\n", fileName.c_str());
      return(false);
   }

   return(true);
}

const char* Voice::SRGetRecognitionHandle(void)
{
   return(SRHandle.c_str());
}

char* Voice::SRGetAudioBuffer(void)
{
   return(incomingSRAudio);
}

unsigned int Voice::SRGetAudioBufferLength(void)
{
   return(AUDIO_MAX);
}

unsigned int Voice::SRGetAudioChunkSize(void)
{
	switch(SRAudioFormat)
   {
   	case TF_WAV_8KHZ_16BIT:	return(3200);	      // 200 ms * 8 samples/ms * 2 bytes/sample
      case TF_WAV_8KHZ_8BIT_MULAW: return(1600);   // 200 ms * 8 samples/ms * 1 byte/sample
      case TF_WAV_8KHZ_8BIT_ALAW: return(1600);    // 200 ms * 8 samples/ms * 1 byte/sample
   	case TF_VCE_24: return(620);	               // 200 ms * 62 bytes per 20 ms
      default: return(0);
   }
}

char* Voice::SRGetNextAudioChunk(void)
{
   char* audioBuffer = incomingSRAudio + nextSRAudioBufferIndex;
   unsigned int bufferSize = SRGetAudioChunkSize();

   if(isReceivingSRAudio)
   {
      if(nextSRAudioBufferIndex > SRAudioBeginBufferIndex)
      {
         SRAudioLength = nextSRAudioBufferIndex - SRAudioBeginBufferIndex;
      }
      else
      {
         SRAudioLength = nextSRAudioBufferIndex + AUDIO_MAX - SRAudioBeginBufferIndex;
      }
   }

   // Make sure we have enough room to write an audio chunk at the new address
   // or reset to beginning of buffer
   if((nextSRAudioBufferIndex + bufferSize) > AUDIO_MAX)
   {
      nextSRAudioBufferIndex = 0;
   }

   // Blank out the buffer before we submit it, board might not use it
   // before it reports a speech-end event
   // @@@ This will not work for all audio formats!!!
   memset(audioBuffer, 0, bufferSize);

   nextSRAudioBufferIndex += bufferSize;

   return(audioBuffer);
}

IV_STATUS Voice::SRGetResult(char*, unsigned int, unsigned int&)
{
   return(IVR_NOTSUPPORTED);
}

void Voice::SRInvalidateStatistics(void)
{
   // initialise properties to empty
   retainAudio = false;
   SRHeard.clear();
   SRAdjusted.clear();
   SRConfidence = 0;
   SRReturn = IVR_NO_VOICE;   // start with no-voice as nothing has happened yet
   termDigit = '\0';
   baseGrammarIdentifier.clear();
   baseGrammarFileName.clear();
   additionalGrammarFileName.clear();
   SRHandle.clear();
}

bool Voice::SRIsPendingAudio(void)
{
   return(isPendingSRAudio);
}

bool Voice::SRIsPendingResult(void)
{
   return(isPendingSRResult);
}

bool Voice::SRIsReceivingAudio(void)
{
   return(isReceivingSRAudio);
}

IV_STATUS Voice::SRLoadGrammar(Voice::SR_GRAMMAR, const char*)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS Voice::SRRelease(void)
{
   return(IVR_NOTSUPPORTED);
}

void Voice::SRSetAudioFormat(SOUND_FORMAT audioFormat)
{
   SRAudioFormat = audioFormat;
}

void Voice::SRSetAudioLength(unsigned int length)
{
   if(isReceivingSRAudio)
   {
      // Align audio length to chunk boundary
      length += SRGetAudioChunkSize() - SRAudioLength % SRGetAudioChunkSize();

      // Maximum audio length is buffer length
      SRAudioLength = length > AUDIO_MAX ? AUDIO_MAX : length;
   }
}

void Voice::SRSetAudioStart(const char* start)
{
   if((start >= incomingSRAudio) && (start < incomingSRAudio + AUDIO_MAX))
   {
      SRAudioBeginBufferIndex = start - incomingSRAudio;
   }
}

IV_STATUS Voice::SRStartAudio(void)
{
   SRClearAudio();
   isReceivingSRAudio = true;
   SRAudioStartTime.Update();
   SRAudioBeginBufferIndex = nextSRAudioBufferIndex;
   return(IVR_SUCCESS);
}

IV_STATUS Voice::SRStopAudio(void)
{
   if(isReceivingSRAudio)
   {
      isReceivingSRAudio = false;
      isPendingSRResult = true;
      isPendingSRAudio = SRAudioLength > 0;
      SRAudioEndTime.Update();
      if(SRAudioBeginBufferIndex + SRAudioLength > AUDIO_MAX)
      {
         memcpy(outgoingSRAudio, incomingSRAudio + SRAudioBeginBufferIndex, AUDIO_MAX - SRAudioBeginBufferIndex);
         memcpy(outgoingSRAudio + AUDIO_MAX - SRAudioBeginBufferIndex, incomingSRAudio, SRAudioBeginBufferIndex + SRAudioLength - AUDIO_MAX);
      }
      else
      {
         memcpy(outgoingSRAudio, incomingSRAudio + SRAudioBeginBufferIndex, SRAudioLength);
      }
   }

   return(IVR_SUCCESS);
}

void Voice::SRUpdateTAMInfo(const char* pTAMName)
{
   currentTAMName = (pTAMName != NULL) ? pTAMName : "";
}

IV_STATUS Voice::SRValidateAudio(const char* audiobuf, unsigned int audiobuflen)
{
   // Doesn't really do anything yet.  Will be necessary for streaming

   // Audio is outside incomingSRAudio
   if((audiobuf < incomingSRAudio) || (audiobuf + audiobuflen >= incomingSRAudio + AUDIO_MAX))
   {
      return(IVR_FAIL);
   }

   return(IVR_SUCCESS);
}

IV_STATUS Voice::SRWriteAudioToFile(const char* audioBuf, unsigned int bytes)
{
   string fileName = SRGetTrainerPath() + SRGetAudioFileName();
   ofstream audioFile(fileName.c_str(), std::ios::binary);

   if(!audioFile.is_open())
   {
      ivLog->Print("      ERROR: Failed to open SR audio file '%s'\n", fileName.c_str());
      return(IVR_FAIL);
   }
   else
   {
      ivLog->Print("      SR audio logged to '%s'\n", fileName.c_str());
   }

   if((SRAudioFormat == TF_WAV_8KHZ_16BIT) || (SRAudioFormat == TF_WAV_8KHZ_8BIT_MULAW))
   {
      char wavHeader[44] =
      {
         0x52, 0x49, 0x46, 0x46,    // "RIFF"
         0x24, 0x00, 0x00, 0x00,    // # of bytes after "RIFF" header (to be filled in later)
         0x57, 0x41, 0x56, 0x45,    // "WAVE"
         0x66, 0x6D, 0x74, 0x20,    // "fmt"
         0x10, 0x00, 0x00, 0x00,    // 16 more bytes in chunk
         0x00, 0x00,                // compression code (to be filled in later)
         0x01, 0x00,                // mono
         0x40, 0x1F, 0x00, 0x00,    // 8000 samples per second
         0x00, 0x00, 0x00, 0x00,    // bytes per second (to be filled in later)
         0x00, 0x00,                // block size (to be filled in later)
         0x00, 0x00,                // bits per sample (to be filled in later)
         0x64, 0x61, 0x74, 0x61,    // "data"
         0x00, 0x00, 0x00, 0x00     // # of bytes in audio chunk (to be filled in later)
      };
      unsigned int chunkSize;       // Data size adjusted for header offsets

      switch(SRAudioFormat)
      {
         // Microsoft Pulse Code Modulation (PCM)
         case TF_WAV_8KHZ_16BIT:
         {
            // PCM/uncompressed compression code
            wavHeader[20] = 0x01;

            // Least significant word of 16000 bytes per second
            wavHeader[28] = 0x80;

            // Most significant word of 16000 bytes per second
            wavHeader[29] = 0x3E;

            // 2 bytes block size
            wavHeader[32] = 0x02;

            // 16 bits per sample
            wavHeader[34] = 0x10;

            break;
         }
         // ITU G.711 uLaw
         case TF_WAV_8KHZ_8BIT_MULAW:
         {
            // ITU G.711 uLaw compression code
            wavHeader[20] = 0x07;

            // Least significant word of 8000 bytes per second
            wavHeader[28] = 0x40;

            // Most significant word of 8000 bytes per second
            wavHeader[29] = 0x1F;

            // 1 byte block size
            wavHeader[32] = 0x01;

            // 8 bits per sample
            wavHeader[34] = 0x08;

            break;
         }
      }

      // Calculate audio header...

      // Adjust wav header for RIFF size
      chunkSize = bytes + 40;
      wavHeader[7] = (unsigned char) (chunkSize >> 24);
      wavHeader[6] = (unsigned char) ((chunkSize >> 16) - (wavHeader[7] << 8));
      wavHeader[5] = (unsigned char) ((chunkSize >> 8) - (wavHeader[7] << 16) - (wavHeader[6] << 8));
      wavHeader[4] = (unsigned char) (chunkSize - (wavHeader[7] << 24) - (wavHeader[6] << 16) - (wavHeader[5] << 8));

      // Adjust wav header for audio data size
      chunkSize = bytes;
      wavHeader[43] = (unsigned char) (chunkSize >> 24);
      wavHeader[42] = (unsigned char) ((chunkSize >> 16) - (wavHeader[43] << 8));
      wavHeader[41] = (unsigned char) ((chunkSize >> 8) - (wavHeader[43] << 16) - (wavHeader[42] << 8));
      wavHeader[40] = (unsigned char) (chunkSize - (wavHeader[43] << 24) - (wavHeader[42] << 16) - (wavHeader[41] << 8));

      // Write audio wav header, then write audio data
      audioFile.write(wavHeader, sizeof(wavHeader));
   }

   audioFile.write(audioBuf, bytes);
   audioFile.close();

   return(IVR_SUCCESS);
}

// --------------------------------------------------------------------------
// Write SR trainer data to .csv file
// --------------------------------------------------------------------------
void Voice::SRWriteData(void)
{
   string      fileName;
   char        portstr[4];
   char        tempstr[12];
   ESCTime currentTime;
   ofstream    trainerFile;

   currentTime.Update();
   sprintf(portstr, "%.3d", telephonyPort);

   fileName = SRGetTrainerPath() + SRGetTrainerDataName();

   // Open file
   trainerFile.open(fileName.c_str(), std::ios::app);
   if(trainerFile.is_open())
   {
      trainerFile << "\"";                  // quote

      //  1  Call Start Date
      trainerFile << ivBoard->GetCallStartTime().GetDate();
      trainerFile << "\",";                // quote comma

      //  2  Call Start Time
      trainerFile << ivBoard->GetCallStartTime().GetTime();
      trainerFile << ",";                  // comma

      //  3  Port number
      trainerFile << portstr;
      trainerFile << ",\"";                // comma quote

      //  4  Application log file name
      trainerFile << appName << "_" << currentTime.GetDate("-") << ".log";
      trainerFile << "\",\"";              // quote comma quote

      //  5  PlayedStepTAP
      trainerFile << lastPlayedStep.stepStat.TAPName;
      trainerFile << "\",\"";              // quote comma quote

      //  6  PlayedStepTAM
      trainerFile << lastPlayedStep.stepStat.TAMName;
      trainerFile << "\",";                // quote comma

      //  7  PlayedStepNum
      trainerFile << lastPlayedStep.stepStat.stepNum;
      trainerFile << ",\"";                // comma quote

      //  8  PlayedStepName
      trainerFile << lastPlayedStep.stepStat.stepName;
      trainerFile << "\",\"";              // quote comma quote

      //  9  PlayedScript
      trainerFile << lastPlayedStep.script;
      trainerFile << "\",\"";              // quote comma quote

      // 10  PlayedFile
      trainerFile << lastPlayedStep.file;
      trainerFile << "\",\"";              // quote comma quote

      // 11  PlayedComplete
      trainerFile << ((lastPlayedStep.completed) ? "COMPLETED" : "INTERRUPTED");
      trainerFile << "\",";                // quote comma

      // 12  Playedto
      sprintf(tempstr, "%.3f", lastPlayedStep.played_sec);
      trainerFile << tempstr;
      trainerFile << ",\"";                // comma quote

      // 13  ExecStepTAP
      trainerFile << lastExecPlayStep.stepStat.TAPName;
      trainerFile << "\",\"";              // quote comma quote

      // 14  ExecStepTAM
      trainerFile << lastExecPlayStep.stepStat.TAMName;
      trainerFile << "\",";                // quote comma

      // 15  ExecStepNum
      trainerFile << lastExecPlayStep.stepStat.stepNum;
      trainerFile << ",\"";                // comma quote

      // 16   ExecStepName
      trainerFile << lastExecPlayStep.stepStat.stepName;
      trainerFile << "\",\"";              // quote comma quote

      // 17   ExecScript
      trainerFile << lastExecPlayStep.script;
      trainerFile << "\",\"";              // quote comma quote

      // 18   ExecFile
      trainerFile << lastExecPlayStep.file;
      trainerFile << "\",\"";              // quote comma quote

      // 19   ExecResult
      trainerFile << ((lastExecPlayStep.completed) ? "COMPLETED" : "INTERRUPTED");
      trainerFile << "\",";                // quote comma

      // 20  ExecTo
      sprintf(tempstr, "%.3f", lastExecPlayStep.played_sec);
      trainerFile << tempstr;
      trainerFile << ",\"";                // comma quote

      // 21  SRStepTAP
      trainerFile << lastSRStep.TAPName;
      trainerFile << "\",\"";              // quote comma quote

      // 22  SRStepTAM
      trainerFile << lastSRStep.TAMName;
      trainerFile << "\",";                // quote comma

      // 23  SRStepNum
      trainerFile << lastSRStep.stepNum;
      trainerFile << ",\"";                // comma quote

      // 24  SRStepName
      trainerFile << lastSRStep.stepName;
      trainerFile << "\",";                // quote comma

      // 25  NoVoiceTO
      trainerFile << novoiceTimeout;
      trainerFile << ",";                  // comma

      // 26  EndVoiceTO
      trainerFile << endvoiceTimeout;
      trainerFile << ",";                  // comma

      // 27  ThresholdConf
      trainerFile << thresholdConf;
      trainerFile << ",";                  // comma

      // 28  retainAudio
      trainerFile << retainAudio;
      trainerFile << ",\"";                // comma quote

      // 29  RecognizerParameters
      trainerFile << srResource;
      trainerFile << "\",\"";              // quote comma quote

      // 30  GrammarName
      trainerFile << baseGrammarIdentifier;
      trainerFile << "\",\"";              // quote comma quote

      // 31  GrammarBase
      trainerFile << baseGrammarFileName;
      trainerFile << "\",\"";              // quote comma quote

      // 32  GrammarAdd
      trainerFile << additionalGrammarFileName;
      trainerFile << "\",\"";              // quote comma quote

      // 33  SpeechStartDate
      trainerFile << SRAudioStartTime.GetDate();
      trainerFile << "\",";             // quote comma

      // 34  SpeechStartTime
      trainerFile << SRAudioStartTime.GetTime();
      trainerFile << ",\"";             // comma quote

      // 35  SpeechEndDate
      trainerFile << SRAudioEndTime.GetDate();
      trainerFile << "\",";             // quote comma

      // 36  SpeechEndTime
      trainerFile << SRAudioEndTime.GetTime();
      trainerFile << ",\"";             // comma quote

      // 37  SRAudioFile
      // Non-touch tone result
      if(((SRReturn == IVR_SUCCESS) || (SRReturn == IVR_THRESHOLD)) && (termDigit == '\0'))
      {
         trainerFile << SRGetAudioFileName();
      }
      trainerFile << "\",\"";              // quote comma quote

      // 38  SRHeard
      trainerFile << SRHeard;
      trainerFile << "\",\"";              // quote comma quote

      // 39  SRAdjusted
      trainerFile << SRAdjusted;
      trainerFile << "\",";                // quote comma

      // 40  SRResultConf
      trainerFile << SRConfidence;
      trainerFile << ",\"";                // comma quote

      // 41  SRTermDigit
      sprintf(tempstr, "%c", termDigit);
      trainerFile << tempstr;
      trainerFile << "\",\"";              // quote comma quote

      // 42  SRReturn
      switch(SRReturn)
      {
         case IVR_SUCCESS :
            trainerFile << "SUCCESS";
            break;

         case IVR_FAIL :
            trainerFile << "FAILURE";
            break;

         case IVR_FATAL :
            trainerFile << "FATAL";
            break;

         case IVR_TIMEOUT :
            trainerFile << "TIMEOUT";
            break;

         case IVR_HANGUP :
            trainerFile << "HANGUP";
            break;

         case IVR_THRESHOLD :
            trainerFile << "THRESHHOLD";
            break;

         default:
            sprintf(tempstr, "STATUS_%i", SRReturn);
            trainerFile << tempstr;
      }
      trainerFile << "\",\"";              // quote comma quote

      // 43  SR Handle
      trainerFile << SRHandle;
      trainerFile << "\"\n";               // quote newline

      trainerFile.close();
   }
   else
   {
      ivLog->Print ("      WARNING: SpeechTrainer failed to write to recognition file '%s'\n", fileName.c_str());
   }
}

IV_STATUS Voice::VAAcquire(const char*)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS Voice::VARelease(void)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS Voice::VAAuthorise(const char*, int, const char*, int)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS Voice::VAEnd(void)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS Voice::VARegister(const char*)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS Voice::VASubmit(int, const char*, int)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS Voice::VAStart(const char*, const char*, int)
{
   return(IVR_NOTSUPPORTED);
}
