#include <math.h>    // pow
#include <stdio.h>   // sprintf

#include "ESCTime.h"

ESCTime::ESCTime(void)
{
   Invalidate();
}

ESCTime::~ESCTime(void)
{
	// Do nothing
}

float ESCTime::GetSecondsSinceMidnight(void) const
{
   if(!valid)
   {
      return(-1);
   }

   return(sysTime.wHour*3600 + sysTime.wMinute*60 + sysTime.wSecond + ((float) sysTime.wMilliseconds)/1000);
}

const char* ESCTime::GetDate(const char* separator)
{
   char* tempstr;

   if(!valid)
   {
      return(NULL);
   }

   separator = separator == NULL ? "/" : separator;
   tempstr = new char[9 + 2*strlen(separator)];
   sprintf(tempstr, "%.4d%s%.2d%s%.2d", sysTime.wYear, separator, sysTime.wMonth, separator, sysTime.wDay);
   date = tempstr;
   delete[] tempstr;

   return(date.c_str());
}

int ESCTime::GetDay(void)
{
   return(valid ? sysTime.wDay : -1);
}

int ESCTime::GetDayOfWeek(void)
{
   return(valid ? sysTime.wDayOfWeek : -1);
}

const char* ESCTime::GetDayOfWeekName(void)
{
   const char* days[] = { "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
   return(valid ? days[sysTime.wDayOfWeek] : NULL);
}

const char* ESCTime::GetFileNameDate(void)
{
   if(!valid)
   {
      return(NULL);
   }

   fileDate = (string) GetDate("-") + (string) "_" + (string) GetTime(".");

   return(fileDate.c_str());
}

int ESCTime::GetHour(void)
{
   return(valid ? sysTime.wHour : -1);
}

int ESCTime::GetMillisecond(void)
{
   return(valid ? sysTime.wMilliseconds : -1);
}

int ESCTime::GetMinute(void)
{
   return(valid ? sysTime.wMinute : -1);
}

int ESCTime::GetMonth(void)
{
   return(valid ? sysTime.wMonth : -1);
}

const char* ESCTime::GetMonthName(void)
{
   const char* months[] = { "January", "February", "March", "April", "May", "June",
      "July", "August", "September", "October", "November", "December"};
   return(valid ? months[sysTime.wMonth - 1] : NULL);
}

int ESCTime::GetSecond(void)
{
   return(valid ? sysTime.wSecond : -1);
}

const char* ESCTime::GetShortDayOfWeekName(void)
{
   const char* days[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
   return(valid ? days[sysTime.wDayOfWeek] : NULL);
}

const char* ESCTime::GetShortMonthName(void)
{
   const char* months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
   return(valid ? months[sysTime.wMonth - 1] : NULL);
}

const char* ESCTime::GetTime(const char* separator, bool includeMilliseconds)
{
   char* tempstr;

   if(!valid)
   {
      return(NULL);
   }

   if(separator == NULL)
   {
      tempstr = new char[13];
      if(includeMilliseconds)
      {
         sprintf(tempstr, "%.2d:%.2d:%.2d.%.3d", sysTime.wHour, sysTime.wMinute, sysTime.wSecond, sysTime.wMilliseconds);
      }
      else
      {
         sprintf(tempstr, "%.2d:%.2d:%.2d", sysTime.wHour, sysTime.wMinute, sysTime.wSecond);
      }
   }
   else
   {
      tempstr = new char[10 + 3*strlen(separator)];
      if(includeMilliseconds)
      {
         sprintf(tempstr, "%.2d%s%.2d%s%.2d%s%.3d", sysTime.wHour, separator, sysTime.wMinute,
            separator, sysTime.wSecond, separator, sysTime.wMilliseconds);
      }
      else
      {
         sprintf(tempstr, "%.2d%s%.2d%s%.2d", sysTime.wHour, separator, sysTime.wMinute,
            separator, sysTime.wSecond);
      }
   }

   time = tempstr;
   delete[] tempstr;

   return(time.c_str());
}

int ESCTime::GetYear(void)
{
   return(valid ? sysTime.wYear : -1);
}

const char* ESCTime::GetHTTPDateTime(void)
{
   char* tempstr;

   if(!valid)
   {
      return(NULL);
   }

   // Compose as HTTP 1.1 Date format, eg: "Tue, 15 Nov 1994 08:12:31 GMT"

   // day short name
   formatDate = GetShortDayOfWeekName();
   formatDate += ", ";

   // day of month
   tempstr = new char[3];
   sprintf(tempstr, "%.2d", GetDay());
   formatDate += tempstr;
   delete[] tempstr;
   formatDate += " ";

   // month short name
   formatDate += GetShortMonthName();
   formatDate += " ";

   // year
   tempstr = new char[5];
   sprintf(tempstr, "%.4d", GetYear());
   formatDate += tempstr;
   delete[] tempstr;
   formatDate += " ";

   // time
   formatDate += GetTime(":", false);  // colon separator (":"), exclude milliseconds
   formatDate += " ";

   switch(timeRegion)
   {
      case GMT:
         formatDate += "GMT";
         break;

      case LOCAL:
      default:
         formatDate += "local";
   }

   return(formatDate.c_str());
}

void ESCTime::Invalidate(void)
{
   valid = false;
   timeRegion = INVALID;
}

float ESCTime::GetSecondsSince(const ESCTime& from) const
{
   SYSTEMTIME toSystemTime = sysTime;
   FILETIME fromFileTime;
   FILETIME toFileTime;
   double high;
   double low;
   double flTo;
   double flFrom;

   if(!valid)
   {
      GetLocalTime(&toSystemTime);
   }

   if(!from.valid)
   {
      return(0);
   }

   SystemTimeToFileTime(&(from.sysTime), &fromFileTime);
   SystemTimeToFileTime(&toSystemTime, &toFileTime);

   // Convert from 100 ns intervals to seconds
	flTo = toFileTime.dwLowDateTime;
	flFrom = fromFileTime.dwLowDateTime;
   low = (flTo - flFrom) / 10000000;

   // Convert to appropriate number of seconds
	flTo = toFileTime.dwHighDateTime;
	flFrom = fromFileTime.dwHighDateTime;
   high = (flTo - flFrom) * pow(256,sizeof(DWORD)) / 10000000;

   return((float)(high + low));
}

ESCTime::TIME_REGION ESCTime::GetRegion(void) const
{
   return(timeRegion);
}

float ESCTime::Update(TIME_REGION regionDesired)
{
   switch (regionDesired)
   {
      case GMT:
         GetSystemTime(&sysTime);
         break;

      case LOCAL:
      default:
         GetLocalTime(&sysTime);
         regionDesired = LOCAL;     // if LOCAL not specified, that's what you get
   }

   timeRegion = regionDesired;
   valid = true;
   return(GetSecondsSinceMidnight());
}

bool ESCTime::IsValid(void) const
{
   return(valid);
}
