#include <string.h>
#include <stdio.h> // Just for debugging

#include "..\3rdparty\c_md5\md5class.h"
#include "..\3rdparty\Blowfish\Blowfish.h"
#include "iv_str.h"
#include "security.h"

char* MD5Hash(const char* plainText, bool uppercase)
{
   if(plainText == NULL)
   {
      return(NULL);
   }

   CMD5 hash(plainText);
   char* hashText = new char[strlen(hash.getMD5Digest()) + 1];
   strcpy(hashText, hash.getMD5Digest());

   if(uppercase)
   {
      strupr(hashText);
   }

   return(hashText);
}

char* Encrypt(const char* plainText, const char* encryptionKey, BLOWFISH_PADDING padding, bool uppercase)
{
   if((plainText == NULL) || (encryptionKey == NULL))
   {
      return(NULL);
   }

   // Block of characters to encrypt; block lengths must always be a multiple of 8
   int paddingBytes = 8 - strlen(plainText)%8;
   int plainTextBlockLength = strlen(plainText) + paddingBytes;
	unsigned char* plainTextBlock = new unsigned char[plainTextBlockLength + 1];
   memset(plainTextBlock, 0, plainTextBlockLength + 1);
   strcpy((char*) plainTextBlock, plainText);

   // Pad block
   switch(padding)
   {
      case PKCS_5_PADDING:
      {
         memset(plainTextBlock + plainTextBlockLength - paddingBytes, paddingBytes, paddingBytes);
         break;
      }

      case ONES_AND_ZEROES_PADDING:
      {
         plainTextBlock[plainTextBlockLength - paddingBytes] = 8;
         break;
      }

      case NULL_PADDING:
      {
         break;
      }

      case SPACE_PADDING:
      {
         memset(plainTextBlock + plainTextBlockLength - paddingBytes, ' ', paddingBytes);
         break;
      }
   }

   // Set up encryption
   CBlowFish oBlowFish((unsigned char*) encryptionKey, strlen(encryptionKey));

   // Encrypt text
   unsigned char* encryptedTextBlock = new unsigned char[plainTextBlockLength + 1];
   memset(encryptedTextBlock, 0, plainTextBlockLength + 1);
	oBlowFish.Encrypt(plainTextBlock, encryptedTextBlock, plainTextBlockLength, CBlowFish::CBC);

   // Convert string to sequence of hex
   int encryptedHexLength = 2*plainTextBlockLength;
   char* encryptedHex = new char[encryptedHexLength + 1];
   memset(encryptedHex, 0, encryptedHexLength + 1);
   CharStr2HexStr(encryptedTextBlock, encryptedHex, plainTextBlockLength, uppercase);

   // Clean up
   delete[] plainTextBlock;
   delete[] encryptedTextBlock;

   return(encryptedHex);
}

char* Decrypt(const char* encryptedHex, const char* decryptionKey)
{
   if((encryptedHex == NULL) || (decryptionKey == NULL))
   {
      return(NULL);
   }

   // A properly enccrypted hex sequence will have a multiple of 8 characters
   if(strlen(encryptedHex)%8 != 0)
   {
      return(NULL);
   }

   // Convert sequence of hex to string
   int encryptedTextBlockLength = strlen(encryptedHex)/2;
	unsigned char* encryptedTextBlock = new unsigned char[encryptedTextBlockLength + 1];
   memset(encryptedTextBlock, 0, encryptedTextBlockLength + 1);
   HexStr2CharStr(encryptedHex, encryptedTextBlock, encryptedTextBlockLength);

   // Set up decryption
   CBlowFish oBlowFish((unsigned char*) decryptionKey, strlen(decryptionKey));

   // Decrypt text
   unsigned char* plainTextBlock = new unsigned char[encryptedTextBlockLength + 1];
   memset(plainTextBlock, 0, encryptedTextBlockLength + 1);
	oBlowFish.Decrypt(encryptedTextBlock, plainTextBlock, encryptedTextBlockLength, CBlowFish::CBC);

   // Remove block padding
   int index = encryptedTextBlockLength - 1;
   // ONES_AND_ZEROES_PADDING and NULL_PADDING
   if(plainTextBlock[index] == '\0')
   {
      for(index--; (plainTextBlock[index] == '\0') && (index > encryptedTextBlockLength - 9); index--);

      // ONES_AND_ZEROES_PADDING
      if(plainTextBlock[index] == 8)
      {
         plainTextBlock[index] = '\0';
      }
   }
   // SPACE_PADDING
   else if(plainTextBlock[index] == ' ')
   {
      for(index--; (plainTextBlock[index] == ' '); index--);
      index++;
      plainTextBlock[index] = '\0';
   }
   // PKCS_5_PADDING
   else
   {
      index -= (plainTextBlock[index] - 1);
      plainTextBlock[index] = '\0';
   }

   // Clean up
   delete[] encryptedTextBlock;

   return((char*) plainTextBlock);
}
