#include <stdlib.h>                 // atoi
#include <ctype.h>                  // isdigit

#include "..\escutil\iv_str.h"      // FindCaseInsensitiveString
#include "HTTPResponse.h"

HTTPResponse::HTTPResponse(void)
{
}

HTTPResponse::~HTTPResponse(void)
{
}

HTTPResponse::STATUS_CODE HTTPResponse::GetStatusCode(void) const
{
   string::size_type endOfResponseLine = this->text.find("\r\n");
   string::size_type index;

   if(endOfResponseLine == string::npos)
   {
      return(INVALID_STATUS);
   }

   // Find space between version and status code
   if((index = text.substr(0, endOfResponseLine).find(' ')) == string::npos)
   {
      return(INVALID_STATUS);
   }

   index++;

   if((endOfResponseLine - index < 3) || !isdigit(text[index]) || !isdigit(text[index + 1]) || !isdigit(text[index + 2]))
   {
      return(INVALID_STATUS);
   }

   return((STATUS_CODE) atoi(text.substr(index, 3).c_str()));
}

const char* HTTPResponse::GetReasonPhrase(void)
{
   string::size_type endOfResponseLine = this->text.find("\r\n");
   string::size_type index;

   if(endOfResponseLine == string::npos)
   {
      return("");
   }

   // Find space between version and status code
   if((index = text.substr(0, endOfResponseLine).find(' ')) == string::npos)
   {
      return("");
   }

   // Find space between status code and reason phrase
   if((index = text.substr(0, endOfResponseLine).find(' ', index + 1)) == string::npos)
   {
      return("");
   }

   // Move to reason phrase
   index++;

   // Get reason phrase
   reasonPhrase = text.substr(index, endOfResponseLine - index);

   return(reasonPhrase.c_str());
}

bool HTTPResponse::HaveCompleteMessage(void) const
{
   string::size_type headerEnd;
   string::size_type index1;
   string::size_type index2;
   unsigned int messageSize;
   // HTTP header should provide actual size
   unsigned int completeMessageSize;

   // If the header separator does not exist, we don't have the whole message
   if((headerEnd = text.find("\r\n\r\n")) == string::npos)
   {
      return(false);
   }

   messageSize = text.length() - headerEnd - 4;

   //Check for content range
   // Format: "Content-Range: bytes 409 - 1354"
   if((index1 = FindCaseInsensitiveString(text.substr(0, headerEnd).c_str(), "Content-Range:")) != string::npos)
   {
      // Find start of content range
      for(; !isdigit(text[index1]); index1++);
      for(; isdigit(text[index1]); index1++);

      // Get end of content range
      for(index1++; !isdigit(text[index1]); index1++);
      for(index2 = index1; isdigit(text[index2]); index2++);
      completeMessageSize = atoi(text.substr(index1, index2 - index1).c_str());
   }
   // Check for content length
   // Format: "Content-Length: 3969"
   else if((index1 = FindCaseInsensitiveString(text.substr(0, headerEnd).c_str(), "Content-Length:")) != string::npos)
   {
      // Get content length
      for(; !isdigit(text[index1]); index1++);
      for(index2 = index1; isdigit(text[index2]); index2++);
      completeMessageSize = atoi(text.substr(index1, index2 - index1).c_str());
   }
   else
   {
      // Nothing in header to indicate size, therefore get all data until none remains
      return(false);
   }
   
   return(messageSize >= completeMessageSize);
}

