#ifndef __INTERNETMESSAGETCPTRANSACTION_H__
#define __INTERNETMESSAGETCPTRANSACTION_H__

#ifdef __DLL__
#define INTERNETMESSAGETCPTRANSACTIONAPI __declspec(dllexport)
#else
#define INTERNETMESSAGETCPTRANSACTIONAPI __declspec(dllimport)
#endif

#include "Socket.h"
#include "InternetMessage.h"

// Base class for request/response transaction across a TCP socket
// Not instaniable
class INTERNETMESSAGETCPTRANSACTIONAPI InternetMessageTCPTransaction : public TCPSocket
{
   public:

      // Connects socket to the address specified by peerName and peerPort
      // Sets the event handle to signal WRITE events (connection is ready to transmit) for the O/S socket
      // Connects from an available port
      // Returns Socket::SUCCESS or a WSAError
      virtual int Connect(void);

      // Connects socket to the address specified by peer
      // Sets the event handle to signal WRITE events (connection is ready to transmit) for the O/S socket
      // Connects from port specified by host, or an available port by default
      // Returns Socket::SUCCESS or a WSAError
      virtual int Connect(const ACE_Addr& peer, const ACE_Addr& host = ACE_Addr::sap_any);

      // Sends request text on socket
      // Sends multiple times for lengths over the maximum packet size
      // Returns Socket::SUCCESS or a WSAError
      virtual int Send(void);

      // Loads the next string (text up to '\0' character) currently available from
      // the socket into response
      // Returns Socket::SUCCESS or a WSAError
      virtual int Receive(void);

      // Returns the peer name
      virtual const char* GetPeerName(void);

      // Returns the peer port
      // The peer port is ANY_PORT by default
      virtual unsigned short GetPeerPort(void);

      // Returns pointer to dynamic request object
      // All request object methods should be accessed via this method
      virtual InternetMessage* Request(void);

      // Returns pointer to dynamic response object
      // All response object methods should be accessed via this method
      virtual InternetMessage* Response(void);

      // Sets the peer name
      virtual void SetPeerName(const char* name);

      // Sets the peer port
      virtual void SetPeerPort(unsigned short port);

   protected:

      InternetMessage* request;
      InternetMessage* response;
      string peerName;
      unsigned short peerPort;

      // Creates a new O/S socket and an event handle
      // Sets the event handle to signal READ or CLOSE events for the O/S socket
      // persistant should be true if the O/S socket should not be closed by the
      // destructor
      // response and request are set to NULL
      // response and request objects must be created and destroyed by the child class
      // The peer port is set to ANY_PORT
      InternetMessageTCPTransaction(bool persistant = false);

      // Creates an event handle
      // Sets the event handle to signal READ or CLOSE events for the socket specified
      // by handle unless handle is ACE_INVALID_HANDLE
      // response and request are set to NULL
      // response and request objects must be created and destroyed by the child class
      // The peer port is set to ANY_PORT
      InternetMessageTCPTransaction(ACE_HANDLE handle);

      // Has no effect
      virtual ~InternetMessageTCPTransaction(void);
};

#endif
