#include <stdlib.h>                 // atoi
#include <ctype.h>                  // isdigit

#include "..\escutil\iv_str.h"      // FindCaseInsensitiveString
#include "SIPResponse.h"

SIPResponse::SIPResponse(void)
{
}

SIPResponse::~SIPResponse(void)
{
}

SIPResponse::STATUS_CODE SIPResponse::GetStatusCode(void) const
{
   string::size_type index1;
   string::size_type index2;

   // Format: "TTTT/V.V XXX MMMMMMMM\r\n..."
   // where TTTT is the internet message type and V.V is the version e.g. SIP/2.0,
   // XXX is status code, and MMMMMMMM is the reason phrase

   // Find space between version and status code
   if((index1 = text.find(' ')) == string::npos)
   {
      return(INVALID_STATUS);
   }

   // Move to status code
   index1++;

   // Find space between status code and reason phrase
   if((index2 = text.find(' ', index1)) == string::npos)
   {
      return(INVALID_STATUS);
   }

   return((STATUS_CODE) atoi(text.substr(index1, index2).c_str()));
}

const char* SIPResponse::GetReasonPhrase(void)
{
   string::size_type index1;
   string::size_type index2;

   // Format: "TTTT/V.V XXX MMMMMMMM\r\n..."
   // where TTTT is the internet message type and V.V is the version e.g. SIP/2.0,
   // XXX is status code, and MMMMMMMM is the reason phrase

   // Find space between version and status code
   if((index1 = text.find(' ')) == string::npos)
   {
      return(NULL);
   }

   // Find space between status code and reason phrase
   if((index1 = text.find(' ', index1 + 1)) == string::npos)
   {
      return(NULL);
   }

   // Move to reason phrase
   index1++;

   // Find end of reason phrase
   if((index2 = text.find("\r\n", index1)) == string::npos)
   {
      return(NULL);
   }

   // Get reason phrase
   reasonPhrase = text.substr(index1, index2 - index1);

   return(reasonPhrase.c_str());
}

bool SIPResponse::HaveCompleteMessage(void) const
{
   string::size_type index1;
   string::size_type index2;
   unsigned int messageSize = text.length();
   unsigned int completeMessageSize = 4;

   //Check for content range
   // Format: "Content-Range: bytes 409 - 1354"
   if((index1 = FindCaseInsensitiveString(text.c_str(), "Content-Range:")) != string::npos)
   {
      // Find start of content range
      for(; !isdigit(text[index1]); index1++);
      for(; isdigit(text[index1]); index1++);

      // Get end of content range
      for(index1++; !isdigit(text[index1]); index1++);
      for(index2 = index1; isdigit(text[index2]); index2++);
      completeMessageSize = atoi(text.substr(index1, index2 - index1).c_str());
   }
   // Check for content length
   // Format: "Content-Length: 3969"
   else if((index1 = FindCaseInsensitiveString(text.c_str(), "Content-Length:")) != string::npos)
   {
      // Get content length
      for(; !isdigit(text[index1]); index1++);
      for(index2 = index1; isdigit(text[index2]); index2++);
      completeMessageSize = atoi(text.substr(index1, index2 - index1).c_str());
   }

   // 4 is the size of an empty message 
   return((messageSize > 4) && (messageSize >= completeMessageSize));
}

