/* -------------------------------------------------------------------------
   File:        INET_CODE.CPP

   Notes:       - Name mangling has been turned off.
                - Function names are prefixed with an underscore.
   ------------------------------------------------------------------------- */
#include <stdio.h>
#include <winsock2.h>

// Warnings disabled to avoid warnings generated by Microsoft code
#pragma warn -8027
#pragma warn -8060
#pragma warn -8061
#include <ws2tcpip.h>
// Warnings re-enabled
#pragma warn +8027
#pragma warn +8060
#pragma warn +8061

#include "inet_code.h"

#ifdef __DLL__
#define DMSG(s)  printf s
#else
#define DMSG(s)  printf s
#endif

#define MY_MAKEWORD(a, b) \
        ((WORD)((BYTE)(a))) | (((WORD)((BYTE)(b))) << (WORD)(8))

/* *************************************************************************
   ************************** COMMON FUNCTIONS *****************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        PrintErrorMsg
   ------------------------------------------------------------------------- */
void PrintErrorMsg (char *msg, int errorCode)
{
   char *szCode;
   int   err;

   err = WSAGetLastError();
   if (err == 0)
      err = errorCode;

   switch (err)
   {
   // Windows Sockets definitions of regular Microsoft C error constants
   case WSAEINTR:               szCode = "WSAEINTR";               break;
   case WSAEBADF:               szCode = "WSAEBADF";               break;
   case WSAEACCES:              szCode = "WSAEACCES";              break;
   case WSAEFAULT:              szCode = "WSAEFAULT";              break;
   case WSAEINVAL:              szCode = "WSAEINVAL";              break;
   case WSAEMFILE:              szCode = "WSAEMFILE";              break;
   // Windows Sockets definitions of regular Berkeley error constants
   case WSAEWOULDBLOCK:         szCode = "WSAEWOULDBLOCK";         break;
   case WSAEINPROGRESS:         szCode = "WSAEINPROGRESS";         break;
   case WSAEALREADY:            szCode = "WSAEALREADY";            break;
   case WSAENOTSOCK:            szCode = "WSAENOTSOCK";            break;
   case WSAEDESTADDRREQ:        szCode = "WSAEDESTADDRREQ";        break;
   case WSAEMSGSIZE:            szCode = "WSAEMSGSIZE";            break;
   case WSAEPROTOTYPE:          szCode = "WSAEPROTOTYPE";          break;
   case WSAENOPROTOOPT:         szCode = "WSAENOPROTOOPT";         break;
   case WSAEPROTONOSUPPORT:     szCode = "WSAEPROTONOSUPPORT";     break;
   case WSAESOCKTNOSUPPORT:     szCode = "WSAESOCKTNOSUPPORT";     break;
   case WSAEOPNOTSUPP:          szCode = "WSAEOPNOTSUPP";          break;
   case WSAEPFNOSUPPORT:        szCode = "WSAEPFNOSUPPORT";        break;
   case WSAEAFNOSUPPORT:        szCode = "WSAEAFNOSUPPORT";        break;
   case WSAEADDRINUSE:          szCode = "WSAEADDRINUSE";          break;
   case WSAEADDRNOTAVAIL:       szCode = "WSAEADDRNOTAVAIL";       break;
   case WSAENETDOWN:            szCode = "WSAENETDOWN";            break;
   case WSAENETUNREACH:         szCode = "WSAENETUNREACH";         break;
   case WSAENETRESET:           szCode = "WSAENETRESET";           break;
   case WSAECONNABORTED:        szCode = "WSAECONNABORTED";        break;
   case WSAECONNRESET:          szCode = "WSAECONNRESET";          break;
   case WSAENOBUFS:             szCode = "WSAENOBUFS";             break;
   case WSAEISCONN:             szCode = "WSAEISCONN";             break;
   case WSAENOTCONN:            szCode = "WSAENOTCONN";            break;
   case WSAESHUTDOWN:           szCode = "WSAESHUTDOWN";           break;
   case WSAETOOMANYREFS:        szCode = "WSAETOOMANYREFS";        break;
   case WSAETIMEDOUT:           szCode = "WSAETIMEDOUT";           break;
   case WSAECONNREFUSED:        szCode = "WSAECONNREFUSED";        break;
   case WSAELOOP:               szCode = "WSAELOOP";               break;
   case WSAENAMETOOLONG:        szCode = "WSAENAMETOOLONG";        break;
   case WSAEHOSTDOWN:           szCode = "WSAEHOSTDOWN";           break;
   case WSAEHOSTUNREACH:        szCode = "WSAEHOSTUNREACH";        break;
   case WSAENOTEMPTY:           szCode = "WSAENOTEMPTY";           break;
   case WSAEPROCLIM:            szCode = "WSAEPROCLIM";            break;
   case WSAEUSERS:              szCode = "WSAEUSERS";              break;
   case WSAEDQUOT:              szCode = "WSAEDQUOT";              break;
   case WSAESTALE:              szCode = "WSAESTALE";              break;
   case WSAEREMOTE:             szCode = "WSAEREMOTE";             break;
   // Extended Windows Sockets error constant definitions
   case WSASYSNOTREADY:         szCode = "WSASYSNOTREADY";         break;
   case WSAVERNOTSUPPORTED:     szCode = "WSAVERNOTSUPPORTED";     break;
   case WSANOTINITIALISED:      szCode = "WSANOTINITIALISED";      break;
   case WSAEDISCON:             szCode = "WSAEDISCON";             break;
   case WSAENOMORE:             szCode = "WSAENOMORE";             break;
   case WSAECANCELLED:          szCode = "WSAECANCELLED";          break;
   case WSAEINVALIDPROCTABLE:   szCode = "WSAEINVALIDPROCTABLE";   break;
   case WSAEINVALIDPROVIDER:    szCode = "WSAEINVALIDPROVIDER";    break;
   case WSAEPROVIDERFAILEDINIT: szCode = "WSAEPROVIDERFAILEDINIT"; break;
   case WSASYSCALLFAILURE:      szCode = "WSASYSCALLFAILURE";      break;
   case WSASERVICE_NOT_FOUND:   szCode = "WSASERVICE_NOT_FOUND";   break;
   case WSATYPE_NOT_FOUND:      szCode = "WSATYPE_NOT_FOUND";      break;
   case WSA_E_NO_MORE:          szCode = "WSA_E_NO_MORE";          break;
   case WSA_E_CANCELLED:        szCode = "WSA_E_CANCELLED";        break;
   case WSAEREFUSED:            szCode = "WSAEREFUSED";            break;

   // Error return codes from gethostbyname and gethostbyaddr.
   case WSAHOST_NOT_FOUND:      szCode = "WSAHOST_NOT_FOUND";      break;
   case WSATRY_AGAIN:           szCode = "WSATRY_AGAIN";           break;
   case WSANO_RECOVERY:         szCode = "WSANO_RECOVERY";         break;
   case WSANO_DATA:             szCode = "WSANO_DATA";             break;
// case WSANO_ADDRESS:  Maps to WSANO_DATA.
   default:                     szCode = "unknown";
   }

   if (msg)
      DMSG (("NET: Error: %s (%d): %s\n", szCode, err, msg));
   else
      DMSG (("NET: Error: %s (%d)\n", szCode, err));
}
/*PrintErrorMsg*/

const char* WSAErrorMsg(int errorCode)
{
   switch(errorCode)
   {
   // Windows Sockets definitions of regular Microsoft C error constants
   case WSAEINTR:               return("WSAEINTR");
   case WSAEBADF:               return("WSAEBADF");
   case WSAEACCES:              return("WSAEACCES");
   case WSAEFAULT:              return("WSAEFAULT");
   case WSAEINVAL:              return("WSAEINVAL");
   case WSAEMFILE:              return("WSAEMFILE");
   // Windows Sockets definitions of regular Berkeley error constants
   case WSAEWOULDBLOCK:         return("WSAEWOULDBLOCK");
   case WSAEINPROGRESS:         return("WSAEINPROGRESS");
   case WSAEALREADY:            return("WSAEALREADY");
   case WSAENOTSOCK:            return("WSAENOTSOCK");
   case WSAEDESTADDRREQ:        return("WSAEDESTADDRREQ");
   case WSAEMSGSIZE:            return("WSAEMSGSIZE");
   case WSAEPROTOTYPE:          return("WSAEPROTOTYPE");
   case WSAENOPROTOOPT:         return("WSAENOPROTOOPT");
   case WSAEPROTONOSUPPORT:     return("WSAEPROTONOSUPPORT");
   case WSAESOCKTNOSUPPORT:     return("WSAESOCKTNOSUPPORT");
   case WSAEOPNOTSUPP:          return("WSAEOPNOTSUPP");
   case WSAEPFNOSUPPORT:        return("WSAEPFNOSUPPORT");
   case WSAEAFNOSUPPORT:        return("WSAEAFNOSUPPORT");
   case WSAEADDRINUSE:          return("WSAEADDRINUSE");
   case WSAEADDRNOTAVAIL:       return("WSAEADDRNOTAVAIL");
   case WSAENETDOWN:            return("WSAENETDOWN");
   case WSAENETUNREACH:         return("WSAENETUNREACH");
   case WSAENETRESET:           return("WSAENETRESET");
   case WSAECONNABORTED:        return("WSAECONNABORTED");
   case WSAECONNRESET:          return("WSAECONNRESET");
   case WSAENOBUFS:             return("WSAENOBUFS");
   case WSAEISCONN:             return("WSAEISCONN");
   case WSAENOTCONN:            return("WSAENOTCONN");
   case WSAESHUTDOWN:           return("WSAESHUTDOWN");
   case WSAETOOMANYREFS:        return("WSAETOOMANYREFS");
   case WSAETIMEDOUT:           return("WSAETIMEDOUT");
   case WSAECONNREFUSED:        return("WSAECONNREFUSED");
   case WSAELOOP:               return("WSAELOOP");
   case WSAENAMETOOLONG:        return("WSAENAMETOOLONG");
   case WSAEHOSTDOWN:           return("WSAEHOSTDOWN");
   case WSAEHOSTUNREACH:        return("WSAEHOSTUNREACH");
   case WSAENOTEMPTY:           return("WSAENOTEMPTY");
   case WSAEPROCLIM:            return("WSAEPROCLIM");
   case WSAEUSERS:              return("WSAEUSERS");
   case WSAEDQUOT:              return("WSAEDQUOT");
   case WSAESTALE:              return("WSAESTALE");
   case WSAEREMOTE:             return("WSAEREMOTE");
   // Extended Windows Sockets error constant definitions
   case WSASYSNOTREADY:         return("WSASYSNOTREADY");
   case WSAVERNOTSUPPORTED:     return("WSAVERNOTSUPPORTED");
   case WSANOTINITIALISED:      return("WSANOTINITIALISED");
   case WSAEDISCON:             return("WSAEDISCON");
   case WSAENOMORE:             return("WSAENOMORE");
   case WSAECANCELLED:          return("WSAECANCELLED");
   case WSAEINVALIDPROCTABLE:   return("WSAEINVALIDPROCTABLE");
   case WSAEINVALIDPROVIDER:    return("WSAEINVALIDPROVIDER");
   case WSAEPROVIDERFAILEDINIT: return("WSAEPROVIDERFAILEDINIT");
   case WSASYSCALLFAILURE:      return("WSASYSCALLFAILURE");
   case WSASERVICE_NOT_FOUND:   return("WSASERVICE_NOT_FOUND");
   case WSATYPE_NOT_FOUND:      return("WSATYPE_NOT_FOUND");
   case WSA_E_NO_MORE:          return("WSA_E_NO_MORE");
   case WSA_E_CANCELLED:        return("WSA_E_CANCELLED");
   case WSAEREFUSED:            return("WSAEREFUSED");

   // Error return codes from gethostbyname and gethostbyaddr.
   case WSAHOST_NOT_FOUND:      return("WSAHOST_NOT_FOUND");
   case WSATRY_AGAIN:           return("WSATRY_AGAIN");
   case WSANO_RECOVERY:         return("WSANO_RECOVERY");
   case WSANO_DATA:             return("WSANO_DATA");
// case WSANO_ADDRESS:  Maps to WSANO_DATA.
   default:                     return("unknown");
   }
};

/* -------------------------------------------------------------------------
   Name:        NetCloseSocket

   Purposes:    Closes the socket.
   ------------------------------------------------------------------------- */
bool NetCloseSocket(NET_STORAGE* p)
{
   bool status = true;

   if(p->hSocket != INVALID_SOCKET)
   {
      if(closesocket(p->hSocket))
      {
         status = false;
      }
      
      p->hSocket = INVALID_SOCKET;
      CloseHandle(p->hEvent);
   }
   
/* // SSL shutdown
   if(ssl != NULL)
   {
      // send SSL/TLS close_notify
      SSL_shutdown(ssl);

      // destroy SSL objects and set to NULL
      SSL_free(ssl);
      ssl = NULL;
      SSL_CTX_free(ctx);
      ctx = NULL;
   }
*/

   return(status);
}
/*NetCloseSocket*/

/* -------------------------------------------------------------------------
   Name:        NetSend

   Purpose:     This function writes data to the socket.
   ------------------------------------------------------------------------- */
int NetSend(NET_STORAGE* p, const char* buf, int bufSize)
{
/* // SSL used
   if(p->ssl != NULL)
   {
      return(SSL_write(p->ssl, buf, bufSize);
   }
   else
*/
   {
      return(send(p->hSocket, buf, bufSize, 0));
   }
}
/*NetSend*/

/* -------------------------------------------------------------------------
   Name:        NetRecv

   Purpose:     This function reads data from the socket.  This function
                should only be called after the wait object has been
                signalled and the event indicates that data is waiting to be
                read.

   Notes:       - If the socket is closed before the data is read, the data
                  will be lost.
   ------------------------------------------------------------------------- */
int NetRecv(NET_STORAGE* p, void* buf, int bufSize)
{
   return(recv(p->hSocket, (char*) buf, bufSize, 0));
}
/*NetRecv*/

/* -------------------------------------------------------------------------
   Name:        NetGetEvent

   Purpose:     This function should be called after the wait object is
                signalled.

   Notes:       - Thanks to Ron McLeod from Telos Engineering for this code.
   ------------------------------------------------------------------------- */
bool NetGetEvent (NET_STORAGE *p, int &event)
{
   WSANETWORKEVENTS  networkEvents;

   if (!p || !p->hSocket || p->hSocket == INVALID_SOCKET || !p->hEvent) {
      if (p->hEvent)
         ResetEvent (p->hEvent);
      event = NET_EVT_NULL;
      return true;
   }
   // Check to see what event occurred.
   // BCB6 upgrade - TF Mar.28/02 - WAS:
   // err = WSAEnumNetworkEvents (p->hSocket, EVT_TYPE p->hEvent, &networkEvents);
   if(WSAEnumNetworkEvents (p->hSocket, p->hEvent, &networkEvents) != 0)
   {
      return(false);
   }

   // **************************************************************** FD_CLOSE
   if(networkEvents.lNetworkEvents & FD_CLOSE)
   {
      // The other end of the socket has been closed.
      event = NET_EVT_CLOSED;
      return(true);
   }

   // ************************************************************** FD_CONNECT
   else if(networkEvents.lNetworkEvents & FD_CONNECT)
   {
      // Connection established with other side of socket.

      // Check error code for this event.
      if (networkEvents.iErrorCode[FD_CONNECT_BIT]) {
         DMSG (("NET: Connect event indicates an error.\n"));
         return false;
      }
      // Tell the socket that we want to be notified when certain network
      // events occur.  We want to know when there is data to read or the
      // connection is closed.
      /* BCB6 upgrade - TF Mar.28/02 - WAS:
      err = WSAEventSelect (p->hSocket, EVT_TYPE p->hEvent,
                            FD_WRITE | FD_READ | FD_CLOSE);
      */
      if(WSAEventSelect (p->hSocket, p->hEvent, FD_WRITE | FD_READ | FD_CLOSE) != 0)
      {
         return(false);
      }
      event = NET_EVT_OPEN;
      return true;
   }

   // ***************************************************************** FD_READ
   else if(networkEvents.lNetworkEvents & FD_READ)
   {
      // A message has been received.
      event = NET_EVT_READ;
      return(true);
   }

   // **************************************************************** FD_WRITE
   else if(networkEvents.lNetworkEvents & FD_WRITE)
   {
      // Buffer is available to send more data.
      event = NET_EVT_WRITE;
      return(true);
   }

   // *************************************************************** FD_ACCEPT
   else if(networkEvents.lNetworkEvents & FD_ACCEPT)
   {
      // Connection established with other side of socket.

      // Check error code for this event.
      if(networkEvents.iErrorCode[FD_ACCEPT_BIT])
      {
         return(false);
      }

      event = NET_EVT_ACCEPT;
      return(true);
   }

   // ***************************************************************** ???????
   else
   {
      return(true);
   }
}
/*NetGetEvent*/

/* *************************************************************************
   ********************** SETUP / TEARDOWN FUNCTIONS ***********************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        NetInitialize

   Purpose:     Windows socket initialization.
   ------------------------------------------------------------------------- */
bool NetInitialize()
{
   WORD     wVersionRequested = MY_MAKEWORD (WINSOCK_HI, WINSOCK_LO);
   WSADATA  wsaData;

   if(WSAStartup(wVersionRequested, &wsaData) != 0)
   {
      return(false);
   }

   if (HIBYTE (wsaData.wVersion) != WINSOCK_LO ||
       LOBYTE (wsaData.wVersion) != WINSOCK_HI)
   {
      WSACleanup();
      return(false);
   }

   return(true);
}
/*NetInitialize*/

/* -------------------------------------------------------------------------
   Name:        NetShutdown

   Purpose:     Windows socket cleanup.

   Warnings:    * *** Don't call this function if NetInitialize was never
                  called.  This is *not* thread safe - it affects all
                  threads running in the process. ***
   ------------------------------------------------------------------------- */
void NetShutdown(void)
{
   WSACleanup();
}
/*NetShutdown*/

/* *************************************************************************
   ************************** SERVER FUNCTIONS *****************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        NetListen
   ------------------------------------------------------------------------- */
bool NetListen(NET_STORAGE* p, int port)
{
   int                 err;
   struct sockaddr_in  sin;

   p->hSocket = socket(AF_INET, SOCK_STREAM, 0);

   if(p->hSocket == INVALID_SOCKET)
   {
      return(false);
   }

   sin.sin_family      = AF_INET;
   sin.sin_addr.s_addr = ADDR_ANY;
   sin.sin_port        = htons ((short)port);

   err = bind(p->hSocket, (struct sockaddr*) &sin, sizeof(sin));

   if(err == SOCKET_ERROR)
   {
      closesocket(p->hSocket);
      p->hSocket = INVALID_SOCKET;
      return(false);
   }

   // Allow a maximum of 5 connections.
   err = listen(p->hSocket, 5);

   if(err == SOCKET_ERROR)
   {
      closesocket(p->hSocket);
      p->hSocket = INVALID_SOCKET;
      return(false);
   }

   p->hEvent = CreateEvent(0, false, false, 0);

   // Associate the wait object with the socket.
   // BCB6 upgrade - TF Mar.28/02 - WAS:
   // err = WSAEventSelect (p->hSocket, EVT_TYPE p->hEvent, FD_ACCEPT);
   if(WSAEventSelect(p->hSocket, p->hEvent, FD_ACCEPT) != 0)
   {
      closesocket(p->hSocket);
      p->hSocket = INVALID_SOCKET;
      CloseHandle(p->hEvent);
      return(false);
   }

   return(true);
}
/*NetListen*/

/* -------------------------------------------------------------------------
   Name:        NetAccept
   ------------------------------------------------------------------------- */
bool NetAccept (NET_STORAGE *p, NET_STORAGE *np, HANDLE hEvent)
{
   int          fromlen;
   SOCKADDR_IN  fsin;

   np->hEvent   = hEvent;

   fromlen = sizeof (SOCKADDR_IN);
   np->hSocket = accept (p->hSocket, (LPSOCKADDR)&fsin, &fromlen);
   if(np->hSocket == INVALID_SOCKET)
   {
      if(WSAGetLastError() == WSAEWOULDBLOCK)
      {
         return(false);
      }
      else
      {
         return(false);
      }
   }

   /* BCB6 upgrade - TF Mar.28/02 - WAS:
   err = WSAEventSelect (np->hSocket, EVT_TYPE np->hEvent,
                         FD_WRITE | FD_READ | FD_CLOSE);
   */
   if(WSAEventSelect (np->hSocket, np->hEvent, FD_WRITE | FD_READ | FD_CLOSE) != 0)
   {
      return(false);
   }
   return(true);
}
/*NetAccept*/

/* *************************************************************************
   ************************** CLIENT FUNCTIONS *****************************
   ************************************************************************* */

// ---------------------------------------------------------------------------
//   Name:        NetConnect
//
//   Purpose:     This function attempts to connect to the specified server
//                and port number.  When the connection is made, the wait
//                object will be signalled.
//
//   Notes:       - The host name can either by fully qualified domain name
//                  (FQDN) or an ip address.
//   -------------------------------------------------------------------------
bool NetConnect(NET_STORAGE* p, const char* pszHost, int port, long lNetworkEvents)
{
   ADDRINFO* addrInfo;
   char portNumber[14];

   sprintf(portNumber, "%d", port);

   if(getaddrinfo(pszHost, portNumber, NULL, &addrInfo) != 0)
   {
      return(false);
   }

   if((p->hSocket = socket(addrInfo->ai_family, SOCK_STREAM, 0)) == INVALID_SOCKET)
   {
      return(false);
   }

   p->hEvent = CreateEvent(0, false, false, 0);

   if(WSAEventSelect(p->hSocket, p->hEvent, lNetworkEvents))
   {
      closesocket(p->hSocket);
      p->hSocket = INVALID_SOCKET;
      CloseHandle(p->hEvent);
      return(false);
   }

   if(connect(p->hSocket, addrInfo->ai_addr, addrInfo->ai_addrlen) == SOCKET_ERROR)
   {
      int lastError = WSAGetLastError();
      if((lastError != WSAEWOULDBLOCK) && (lastError != WSAEALREADY) && (lastError != WSAEISCONN))
      {
         closesocket(p->hSocket);
         p->hSocket = INVALID_SOCKET;
         return(false);
      }
   }

   return(true);
}
// NetConnect

bool NetSendConnect(NET_STORAGE* p, const char* pszHost, int port)
{
   ADDRINFO* addrInfo;
   char portNumber[14];

   sprintf(portNumber, "%d", port);

   if(getaddrinfo(pszHost, portNumber, NULL, &addrInfo) != 0)
   {
      return(false);
   }

   if(connect(p->hSocket, addrInfo->ai_addr, addrInfo->ai_addrlen) == SOCKET_ERROR)
   {
      int lastError = WSAGetLastError();
      if((lastError != WSAEWOULDBLOCK) && (lastError != WSAEALREADY) && (lastError != WSAEISCONN))
      {
         closesocket(p->hSocket);
         p->hSocket = INVALID_SOCKET;
         return(false);
      }
   }

   return(true);
}
