#ifndef INET_MSG_H
#define INET_MSG_H

#include "inet_code.h"

// Any one of the NET_EVT_* defines will be returned in the 'id' field of
// the NET_MSG structure.

#define NET_EVT_CLIENT_BASE                                   100
#define NET_EVT_CLIENT_MAX             NET_EVT_CLIENT_BASE +   99
#define NET_EVT_SERVER_BASE            NET_EVT_CLIENT_MAX  +    1

// These are events sent to the server from the client.

#define NET_EVT_STRING                 NET_EVT_CLIENT_BASE +    1
#define NET_EVT_SEND_UPDATE            NET_EVT_CLIENT_BASE +    2
#define NET_EVT_SD_NORMAL              NET_EVT_CLIENT_BASE +    3
#define NET_EVT_SD_ABORT               NET_EVT_CLIENT_BASE +    4
#define NET_EVT_RELOAD_NORMAL          NET_EVT_CLIENT_BASE +    5
#define NET_EVT_RELOAD_ABORT           NET_EVT_CLIENT_BASE +    6
#define NET_EVT_ALL_SD_NORMAL          NET_EVT_CLIENT_BASE +    7
#define NET_EVT_ALL_SD_ABORT           NET_EVT_CLIENT_BASE +    8
#define NET_EVT_SEND_OUTPUT            NET_EVT_CLIENT_BASE +    9
#define NET_EVT_ALL_RELOAD_NORMAL      NET_EVT_CLIENT_BASE +    10
#define NET_EVT_ALL_RELOAD_ABORT       NET_EVT_CLIENT_BASE +    11
#define NET_EVT_RELOAD_TAL		    	   NET_EVT_CLIENT_BASE +    12
#define NET_EVT_START_STREAM           NET_EVT_CLIENT_BASE +    13
#define NET_EVT_STOP_STREAM            NET_EVT_CLIENT_BASE +    14
#define NET_EVT_PAUSE_STREAM           NET_EVT_CLIENT_BASE +    15

// These are events sent to the client from the server.

#define NET_EVT_UPDATE                 NET_EVT_SERVER_BASE +    1
#define NET_EVT_OUTPUT                 NET_EVT_SERVER_BASE +    2

// Any one of the NET_CAUSE_* defines will be returned in the 'cause' field of
// the NET_MSG structure.

#define NET_CAUSE_NONE                 0
#define NET_CAUSE_SUCCESS              0
#define NET_CAUSE_FAILURE              1
#define NET_CAUSE_BASE               100

typedef struct
{
   unsigned long  id;
   unsigned long  cause;
   int            value;
   long           dataSize;     // Non-NULL if additional structure follows.
   void          *data;         // Client must not delete the memory.  It will
                                // be created and destroyed by these functions.
} NET_MSG;

bool NetReadMessage (NET_STORAGE *p, NET_MSG *event, DWORD timeout = 100);
bool NetSendMessage (NET_STORAGE *p, NET_MSG *event);

#endif
