/* -------------------------------------------------------------------------
   Copyright (c) 2002 enGenic.  All rights reserved.
   ------------------------------------------------------------------------- */
#include "xmllist.h"
#include <msxmldom.hpp>

/* *************************************************************************
   *************************  XMLList Class  *******************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        XMLList::XMLList                               (Constructor)
   ------------------------------------------------------------------------- */
XMLList::XMLList(Log* outLog) : Head()
{
   ivLog = outLog;
}
/*XMLList::XMLList*/


/* -------------------------------------------------------------------------
   Name:        XMLList::~XMLList                               (Destructor)
   ------------------------------------------------------------------------- */
XMLList::~XMLList(void)
{
   // Nothing required yet
}
/*XMLList::~XMLList*/


/* -------------------------------------------------------------------------
   Name:        XMLList::FindXML
   ------------------------------------------------------------------------- */
XML* XMLList::FindXML (int refNo)
{
   XML *temp;

   temp = (XML*)First();
   while (temp)
   {
      if (temp->GetReferenceNo () == refNo)
         return temp;
      temp = (XML*)temp->Suc();
   }
   return NULL;
}
/*XMLList::FindXML*/

void XMLList::Reset(void)
{
   Clear();
}

/* *************************************************************************
   **************************  XML Class  **********************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        XML::XML                                       (Constructor)
   ------------------------------------------------------------------------- */
XML::XML (Log *outLog) : Link()
{
   ivLog      = outLog;
   refNo      = 0;
   xFetchList = new XMLFetchList (ivLog);
}
/*XML::XML*/


/* -------------------------------------------------------------------------
   Name:        XML::~XML                                       (Destructor)
   ------------------------------------------------------------------------- */
XML::~XML()
{
   ivLog->Print("            XML destructor called\n");   // @@@ DEBUG
   if (xFetchList != NULL)
   {
      ivLog->Print("            deleting xFetchList, 0x%x\n", xFetchList);   // @@@ DEBUG
      delete xFetchList;
      ivLog->Print("            xFetchList deleted\n");   // @@@ DEBUG
   }
   else
   {
      ivLog->Print("            xFetchList is NULL\n");   // @@@ DEBUG
   }
   ivLog->Print("            XML destructor done\n");   // @@@ DEBUG
}
/*XML::~XML*/


/* -------------------------------------------------------------------------
   Name:        XML::SetReferenceNo                                 (Public)
   ------------------------------------------------------------------------- */
void XML::SetReferenceNo (int value)
{
   refNo = value;
}
/*XML::SetReferenceNo*/


/* -------------------------------------------------------------------------
   Name:        XML::GetReferenceNo                                 (Public)
   ------------------------------------------------------------------------- */
int  XML::GetReferenceNo()
{
   return refNo;
}
/*XML::GetReferenceNo*/


/* -------------------------------------------------------------------------
   Name:        XML::LoadFromBuffer                                 (Public)

   Purpose:     Load XML from a memory buffer

   Modified:    May 13/02 - TF: Created
   ------------------------------------------------------------------------- */
IV_STATUS XML::LoadFromBuffer(const char* pBuf)
{
   AnsiString  Buffer = pBuf;

   // Load buffer into XML object
   // ---------------------------
   try
   {
      xmlDoc = LoadXMLData(Buffer);
   }
   __except (EXCEPTION_EXECUTE_HANDLER)
   {
      ivLog->Print("      ERROR: Not a valid XML document:\n");
      ivLog->Print("      -----\n");
      if (pBuf != NULL)
         ivLog->Print("%s\n", pBuf);
      ivLog->Print("      -----\n");
      return IVR_FAIL;
   }

   // Check the results
   // -----------------
   if (xmlDoc == NULL)
   {
      ivLog->Print("      ERROR: Out of memory.\n");
      return IVR_FATAL;
   }
   return IVR_SUCCESS;
}
/*XML::LoadFromBuffer*/


/* -------------------------------------------------------------------------
   Name:        XML::LoadFromFile                                   (Public)

   Purpose:     Load XML from a file on disk

   Modified:    May 13/02 - TF: Created
   ------------------------------------------------------------------------- */
IV_STATUS XML::LoadFromFile(const char* pszFile)
{
   FILE     *pFile;         // File pointer to check validity of pszFile

   // Verify requested file exists
   // ----------------------------
   pFile = fopen(pszFile, "r");
   if (pFile == NULL)
   {
      ivLog->Print ("      ERROR: Could not find file '%s'\n", pszFile);
      return IVR_FAIL;
   }
   fclose(pFile);

   // Load file into XML object
   // -------------------------
   try
   {
      xmlDoc = LoadXMLDocument((WideString) pszFile);
   }
   __except (EXCEPTION_EXECUTE_HANDLER)
   {
      ivLog->Print("      ERROR: File does not contain a valid XML document.\n");
      ivLog->Print("      File name = '%s'\n", pszFile);
      return IVR_FAIL;
   }

   // Check the results
   // -----------------
   if (xmlDoc == NULL)
   {
      ivLog->Print("      ERROR: Out of memory.\n");
      return IVR_FATAL;
   }

   return IVR_SUCCESS;
}
/*XML::LoadFromFile*/


/* -------------------------------------------------------------------------
   Name:        XML::CopyToBuffer                                   (Public)

   Purpose:     Save XML to a character buffer

   Modified:    May 14/02 - TF: Created
   ------------------------------------------------------------------------- */
IV_STATUS XML::CopyToBuffer(char* &pText)
{
   AnsiString     XMLString = "";
   char *         pszLocal;
   size_t         iLen;

   try
   {
      xmlDoc->SaveToXML(XMLString);
   }
   __except (EXCEPTION_EXECUTE_HANDLER)
   {
      ivLog->Print("      ERROR: Could not retrieve XML.\n");
      return IVR_FATAL;
   }

   iLen = XMLString.Length() + 1;
   pszLocal = new char [iLen];
   pszLocal[0] = '\0';

   strcpy(pszLocal, XMLString.c_str());

   pText = pszLocal;

   return IVR_SUCCESS;
}
/*XML::CopyToBuffer*/


/* *************************************************************************
   **********************  XMLFetchList Class  *****************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        XMLFetchList::XMLFetchList                     (Constructor)
   ------------------------------------------------------------------------- */
XMLFetchList::XMLFetchList (Log *outLog) : Head()
{
   ivLog = outLog;
}
/*XMLFetchList::XMLFetchList*/


/* -------------------------------------------------------------------------
   Name:        XMLFetchList::~XMLFetchList                     (Destructor)
   ------------------------------------------------------------------------- */
XMLFetchList::~XMLFetchList()
{
   // Nothing required yet - TF May 16/02
   ivLog->Print("               XMLFetchList destructor done\n");   // @@@ DEBUG
}
/*XMLFetchList::~XMLFetchList*/


/* -------------------------------------------------------------------------
   Name:        XMLFetchList::FindFetch                             (Public)

   Purpose:     Find an open fetch object
   ------------------------------------------------------------------------- */
XMLFetch* XMLFetchList::FindFetch(const char* pszReqCtxt)
{
   XMLFetch* temp;

   temp = (XMLFetch*)First();
   while (temp)
   {
      if (stricmp(temp->pszContext, pszReqCtxt) == 0)
      {
         // Found!
         return temp;
      }
      temp = (XMLFetch*)temp->Suc();
   }
   // Not found.
   return NULL;
}
/*XMLFetchList::FindFetch*/


/* *************************************************************************
   ************************  XMLFetch Class  *******************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        XMLFetch::XMLFetch                             (Constructor)
   ------------------------------------------------------------------------- */
XMLFetch::XMLFetch (Log *outLog, const char *pszNodes) : Link()
{
   ivLog         = outLog;
   pszCurToken   = NULL;
   iCurNodePos   = 0;
   xCurNode      = NULL;
   xParentNode   = NULL;
   iTotChildren  = 0;

   pszContext    = new char [strlen(pszNodes)+1];
   if (pszContext != NULL)
   {
      strcpy (pszContext, pszNodes);
      pszCurToken = new char [strlen(pszContext)+1];
      if (pszCurToken != NULL)
         pszCurToken[0] = '\0';
   }
}
/*XMLFetch::XMLFetch*/


/* -------------------------------------------------------------------------
   Name:        XMLFetch::~XMLFetch                             (Destructor)
   ------------------------------------------------------------------------- */
XMLFetch::~XMLFetch()
{
   ivLog->Print("                  XMLFetch destructor called\n");   // @@@ DEBUG
   if (pszContext != NULL)
      delete [] pszContext;
   if (pszCurToken != NULL)
      delete [] pszCurToken;
   ivLog->Print("                  XMLFetch destructor done\n");   // @@@ DEBUG
}
/*XMLFetch::~XMLFetch*/


/* -------------------------------------------------------------------------
   Name:        XMLFetch::FindFirst                                 (Public)

   Purpose:     Find the first occurence of the node pointed to by
                'pszContext'.

   Notes:       - Need special handling for first occurence because
                  we must parse the tree until we find the node.
                - Subsequent searches use 'FindNext' and continue under
                  the same parent node.
   ------------------------------------------------------------------------- */
IV_STATUS XMLFetch::FindFirst(_di_IXMLDocument xmlDoc)
{
   int           ctxtPos = 0;        // Current position in Context string
   int           tokenPos;           // Current position in token string
   bool          FirstToken = true;  // Flag to indicate working on the first token (special case)
   bool          Found = false;      // Flag for search loop control
   WideString    wsTemp;             // Temporary string for comparisons

   // Check validity of required strings
   if (pszContext == NULL)
   {
      ivLog->Print ("      ERROR: No node specified to fetch from.\n");
      return IVR_FATAL;
   }
   if (pszCurToken == NULL)
   {
      ivLog->Print ("      ERROR: Out of memory.\n");
      return IVR_FATAL;
   }

   ivLog->Print("      Fetching first node...\n");

   while (pszContext[ctxtPos] != '\0')
   {
      // --- Parse out next token
      tokenPos = 0;
      if (xCurNode != NULL)
      {
         // Moving down a level in the XML document tree...
         xParentNode = xCurNode;
         xCurNode = NULL;
         iTotChildren = xParentNode->ChildNodes->Count;
         iCurNodePos = 0;
      }

      while (pszContext[ctxtPos] != '\0' && (pszContext[ctxtPos] != '-' && pszContext[ctxtPos+1] != '>'))
      {
         pszCurToken[tokenPos] = pszContext[ctxtPos];
         ctxtPos++;
         tokenPos++;
      }
      pszCurToken[tokenPos] = '\0';
      // Assertion: have a complete token in pszCurToken

      if (pszContext[ctxtPos] != '\0')
      {
         // Skip the special TeleFlow characters '->'
         ctxtPos = ctxtPos + 2;
      }

      if (xParentNode == NULL)
      {
         // --- Get the document element node
         xParentNode = xmlDoc->DocumentElement;
         iTotChildren = xParentNode->ChildNodes->Count;
         iCurNodePos = 0;

         // --- Determine handling of Document Element
         wsTemp = pszCurToken;
         if(FirstToken && xParentNode->GetNodeName() == wsTemp)
         {
            // Special case: Token found at doc level; loop back to parse next token
            Found = true;
         }
      }

      // --- Find next node
      while (iCurNodePos < iTotChildren && !Found)
      {
         xCurNode = xParentNode->ChildNodes->GetNode(iCurNodePos);
         wsTemp = pszCurToken;
         if (xCurNode->GetNodeName() == wsTemp)
            Found = true;
         else
            iCurNodePos++;
      }

      if (!Found)
      {
         ivLog->Print("         XML Node '%s' not found.\n", pszCurToken);
         return IVR_FAIL;
      }

      FirstToken = false;
      Found = false;
   } // endwhile

   return IVR_SUCCESS;
}
/*XMLFetch::FindFirst*/


/* -------------------------------------------------------------------------
   Name:        XMLFetch::FindNext                                  (Public)

   Purpose:     Find the next occurrence of the node pointed to by
                'pszContext'.

   Modified:    May 17/02 - TF: Created

   Notes:       -  Assumes 'FindFirst' has already been called to
                   parse the XML document tree.

                -  'FindNext' only searches for subsequent occurrences of
                   the node under a single parent node.
   ------------------------------------------------------------------------- */
IV_STATUS XMLFetch::FindNext()
{
   ivLog->Print("      Fetching next node...\n");
   if (iCurNodePos+1 < xParentNode->ChildNodes->Count)
   {
      iCurNodePos++;
      xCurNode = xParentNode->ChildNodes->GetNode(iCurNodePos);
      return IVR_SUCCESS;
   }
   else
   {
      return IVR_FAIL;
   }
}
/*XMLFetch::FindNext*/


/* -------------------------------------------------------------------------
   Name:        XMLFetch::FindPrev                                  (Public)

   Purpose:     Find the previous occurrence of the node pointed to by
                'pszContext'.

   Modified:    Jul.22/02 - TF: Created

   Notes:       -  Assumes 'FindFirst' has already been called to
                   parse the XML document tree.

                -  'FindPrev' only searches for previous occurrences of
                   the node under a single parent node.  If attempt to
                   search beyond that point, IVR_FAIL will be returned.
   ------------------------------------------------------------------------- */
IV_STATUS XMLFetch::FindPrev()
{
   ivLog->Print("      Fetching previous node...\n");

   if (iCurNodePos-1 >= 0)
   {
      iCurNodePos--;
      xCurNode = xParentNode->ChildNodes->GetNode(iCurNodePos);
      return IVR_SUCCESS;
   }
   else
   {
      return IVR_FAIL;
   }
}
/*XMLFetch::FindPrev*/


/* -------------------------------------------------------------------------
   Name:        XMLFetch::CaptureData                               (Public)

   Purpose:     Capture the value and attributes of the current node
                of the XML Fetch object into TeleFlow variables

   Modified:    May 17/02 - TF: Created
                Jun 01/02 - TF: Fixed memory leak
                Jun 12/02 - TF: Support '@' symbols within XML data

   Notes:       -  TO DO: Make the handling of missing attributes a user
                   defined setting, ie: ignore (return IVR_SUCCESS) or error
                   (return IVR_FATAL).  Also requires a TFDesigner change
                   to XML Fetch step - TF Jun.6/02
   ------------------------------------------------------------------------- */
IV_STATUS XMLFetch::CaptureData(PIVRINFO pInfo,
                                const char *pszValueVar,
                                const char *pszAttrText,
                                XMLFetch::SCOPE varScope)
{
   char*          pszValue;            // temporary string to retieve data
   size_t         bufsize;             // buffer size
   char*          pszAttrToken = NULL; // attribute to be retrieved
   char*          pszVarToken = NULL;  // variable to save attribute value to
   unsigned int   srcpos;              // current position in source string
   unsigned int   dstpos;              // current position in destination string
   _di_IXMLNode   xAttrNode;           // Borland XML Node object
   WideString     wsTemp;              // Wide string required by Borland XML objects
   bool ret;

   if (pszValueVar != NULL && *pszValueVar != '\0')
   {
      // User wants to capture node value to a variable
      // Does the node have a text value?
      if(xCurNode->GetIsTextElement())
      {
         // Node has a text value - copy it into the variable specified
         pszValue = Wide2Char(xCurNode->Text);   // Convert node text to c_str
         if (pszValue == NULL)
         {
            ivLog->Print ("      ERROR: Out of memory\n");
            return IVR_FATAL;
         }

         switch(varScope)
         {
            case XMLFetch::DEFAULT:
            {
               ret = pInfo->variables->Set(pszValueVar, pszValue);
               break;
            }

            case XMLFetch::LOCAL:
            {
               ret = pInfo->variables->SetLocal(pszValueVar, pszValue);
               break;
            }

            case XMLFetch::GLOBAL:
            {
               ret = pInfo->variables->SetGlobal(pszValueVar, pszValue);
               break;
            }

            default:
            {
               ivLog->Print("      ERROR: Invalid XMLFetch::SCOPE value: %d\n", varScope);
               ret = false;
            }
         }

         if(!ret)
         {
            ivLog->Print("      ERROR: Failed to set variable '%s'\n", pszValueVar);
            delete[] pszValue;
            return(IVR_FATAL);
         }
         delete[] pszValue;
      }
      else
      {
         // Node does not have a text value - copy empty string into the var specified
         switch(varScope)
         {
            case XMLFetch::DEFAULT:
            {
               ret = pInfo->variables->Set(pszValueVar, "");
               break;
            }

            case XMLFetch::LOCAL:
            {
               ret = pInfo->variables->SetLocal(pszValueVar, "");
               break;
            }

            case XMLFetch::GLOBAL:
            {
               ret = pInfo->variables->SetGlobal(pszValueVar, "");
               break;
            }

            default:
            {
               ivLog->Print("      ERROR: Invalid XMLFetch::SCOPE value: %d\n", varScope);
               ret = false;
            }
         }

         if(!ret)
         {
            ivLog->Print("      ERROR: Failed to set variable '%s'\n", pszValueVar);
            delete[] pszValue;
            return(IVR_FATAL);
         }
      }
   // else: User doesn't care about node value
   } // endif

   if (pszAttrText != NULL && *pszAttrText != '\0')
   {
      // User wants to capture attribute value(s) to variable(s)
      bufsize = strlen(pszAttrText)+1;
      pszAttrToken = new char[bufsize];
      memset(pszAttrToken, 0, bufsize);
      pszVarToken = new char[bufsize+1]; // need 1 extra for the '@' symbol
      memset(pszVarToken, 0, bufsize);

      if (pszAttrToken == NULL || pszVarToken == NULL)
      {
         ivLog->Print ("      ERROR: Out of memory.\n");
         return IVR_FATAL;
      }
      srcpos = 0;

      while (pszAttrText[srcpos] != '\0')
      {
         pszAttrToken[0] = '\0';
         pszVarToken[0] = '\0';

         // Verify that the node has attributes - TF May 22/02
			//
			// *** The following statement will crash when the node searched for
         //     is the top-level node.  This problem may need to be fixed
         //     eventually, and is on the tech list.  We are not sure if the
         //     error is ours or in Borland's XML support.  Changes may also
         //     need to be made to the FindFirst() method as this is where
         //     the node xCurNode is created. - TF May 6, 2003.
			//
         if (xCurNode->AttributeNodes->Count == 0)
         {
            ivLog->Print("      Warning: Node has no attributes\n");
            if (pszAttrToken != NULL)
               delete [] pszAttrToken;
            if (pszVarToken != NULL)
               delete [] pszVarToken;
            return IVR_FAIL;
         }

         // Skip white space until we find a token
         while (isspace(pszAttrText[srcpos]) && pszAttrText[srcpos] != '\0')
         {
            srcpos++;
         }

         if (pszAttrText[srcpos] != '\0')
         {
            // Copy the current token as an attribute
            dstpos = 0;
            while (!isspace(pszAttrText[srcpos]) &&
            /***/  pszAttrText[srcpos] != '\0'   && pszAttrText[srcpos] != '\n' )
            {
               pszAttrToken[dstpos] = pszAttrText[srcpos];
               dstpos++;
               srcpos++;
            }
            pszAttrToken[dstpos] = '\0';

            // Skip any white space
            while (isspace(pszAttrText[srcpos]) && pszAttrText[srcpos] != '\0' &&
            /***/  pszAttrText[srcpos] != '\n' )
            {
               srcpos++;
            }

            // Check if next token is "as " or "AS "
            if ( toupper(pszAttrText[srcpos]) == 'A' &&
            /**/ toupper(pszAttrText[srcpos+1]) == 'S' &&
            /**/ isspace(pszAttrText[srcpos+2]) )
            {
               // Found "as", therefore next token is a variable token
               srcpos = srcpos + 3;
               pszVarToken[0] = '@';
               dstpos = 1;
               // Copy the current token as a variable
               while (pszAttrText[srcpos] != '\n' && pszAttrText[srcpos] != '\0')
               {
                  if (!isspace(pszAttrText[srcpos]))
                  {
                     pszVarToken[dstpos] = pszAttrText[srcpos];
                     dstpos++;
                  }
                  srcpos++;
               } // endwhile
               pszVarToken[dstpos] = '\0';
            } else {
               // Did not find "as", therefore there is no variable token
               // Use the uppercase of the attribute token as the variable name
               pszVarToken[0] = '@';
               for (dstpos = 0; dstpos < strlen(pszAttrToken); dstpos++)
               {
                  pszVarToken[1+dstpos] = (char)toupper(pszAttrToken[dstpos]);
               }
               dstpos++;
               pszVarToken[dstpos] = '\0';
            } // endif

            wsTemp = pszAttrToken;

            // Determine if this node has an attribute by this name
            if (xCurNode->HasAttribute(wsTemp))
            {
               xAttrNode = xCurNode->AttributeNodes->GetNode(wsTemp);
               pszValue = Wide2Char(xAttrNode->Text);   // Convert attribute text to c_str

               if (pszValue == NULL)
               {
                  ivLog->Print("      ERROR: Failed to set variable: %s\n", pszValueVar);
                  delete[] pszAttrToken;
                  delete[] pszVarToken;
                  return IVR_FATAL;
               }

               // Display TeleFlow variables as we set them
               ivLog->Print ("         %s = (%s) '%s'\n", pszVarToken, pszAttrToken, pszValue);

               switch(varScope)
               {
                  case XMLFetch::DEFAULT:
                  {
                     ret = pInfo->variables->Set(pszVarToken, pszValue);
                     break;
                  }

                  case XMLFetch::LOCAL:
                  {
                     ret = pInfo->variables->SetLocal(pszVarToken, pszValue);
                     break;
                  }

                  case XMLFetch::GLOBAL:
                  {
                     ret = pInfo->variables->SetGlobal(pszVarToken, pszValue);
                     break;
                  }

                  default:
                  {
                     ivLog->Print("      ERROR: Invalid XMLFetch::SCOPE value: %d\n", varScope);
                     ret = false;
                  }
               }

               if(!ret)
               {
                  ivLog->Print("      ERROR: Failed to set variable: %s\n", pszValueVar);
                  delete[] pszValue;
                  delete[] pszAttrToken;
                  delete[] pszVarToken;
                  return(IVR_FATAL);
               }
               delete[] pszValue;
            }
            else
            {
               ivLog->Print("      ERROR: No such attribute '%s'\n", pszAttrToken);
               delete[] pszAttrToken;
               delete[] pszVarToken;
               return IVR_FATAL;
               //
               // TO DO: Make the handling of missing attributes a user-defined
               //        setting  - TF Jun.6/02
            }

         } // endif
      } // endwhile

   // else: User doesn't care about attributes
   } // endif

   // --- Cleanup memory used
   delete[] pszAttrToken;
   delete[] pszVarToken;

   return(IVR_SUCCESS);
}
/*XMLFetch::CaptureData*/


/* *************************************************************************
   *******************  Misc Supporting Functions  *************************
   ************************************************************************* */


/* -------------------------------------------------------------------------
   Name:        Wide2Char

   Purpose:     Convert a WideString into a c_str

   Modified:    May 07/2002 - TF: Created
                Jun.05/2002 - TF: Fixed memory over-run

   Notes:     - Assumes that the WideString contains ASCII data.  This only
                works with a string containing US/Canada English data.

              - Creates a new character buffer and returns a pointer to it.
                The programmer is responsible for deleting the buffer when
                finished with it.
   ------------------------------------------------------------------------- */
char* Wide2Char(WideString wsSource)
{
   int   srcSize = wsSource.Length();
   char *pBuf;
   int   i = 0;

   if (srcSize > 0)
   {
      pBuf = new char[srcSize+1];
      if (pBuf != NULL)
      {
         for (i = 0; wsSource[i+1] != '\0'; i++)
         {
            pBuf[i] = wsSource[i+1];
         }
      }
   }
   else
   {
      pBuf = new char[1];
   }
   if (pBuf != NULL)
   {
      pBuf[i] = '\0';
   }

   return pBuf;
}
/*Wide2Char*/

