#ifndef XMLLIST_H
#define XMLLIST_H

#include "..\TFServer\ivr_defs.h"

#include <XMLDoc.hpp>
#pragma hdrstop

#include "..\escutil\link.h"
#include "..\TFServer\ivrinfo.h"
#include "..\escutil\con_log.h"

#if defined(__DLL__)
#define XMLLISTAPI __declspec(dllexport)
#else
#define XMLLISTAPI __declspec(dllimport)
#endif

class XML;
class XMLFetchList;
class XMLFetch;

// Miscellaneous function definitions:
char* Wide2Char(WideString wsSource);

/* -------------------------------------------------------------------------
   Class:       XMLList

   Purpose:     Defines the top level (Head) of linked list of TeleFlow
                XML Document objects, and provides access to XML
                session-level methods.

   Created:     May 1/02: TF

   Notes:       None
   ------------------------------------------------------------------------- */
class XMLLISTAPI XMLList : public Head
{
   public:

      XMLList(Log* outLog);
      virtual ~XMLList(void);

      XML* FindXML(int refNo);           // Locate an XML object in the list
      void Reset(void);

   private:

      Log* ivLog;                // Local pointer to output log
};


/* -------------------------------------------------------------------------
   Class:       XML

   Purpose:     Defines a single TeleFlow XML object in the linked list
                (XMLList), and provides access to XML transaction-level
                functions.

   Created:     May 1/02: TF

   Notes:       - Type "_di_IXMLDocument" is a Borland XML document class.
   ------------------------------------------------------------------------- */
class XMLLISTAPI XML : public Link
{
   public:

      _di_IXMLDocument  xmlDoc;           // Borland XML document object
      XMLFetchList     *xFetchList;       // Pointer to 'XML Fetch' list header object
                                          //    (see definition below)
      XML(Log* outLog);
      virtual ~XML(void);

      void SetReferenceNo(int value);        // Assign TeleFlow ref number ("XML Handle")
      int  GetReferenceNo(void);                  // Get TeleFlow ref number
      IV_STATUS LoadFromBuffer(const char* pBuf);   // Load XML from a memory buffer
      IV_STATUS LoadFromFile(const char* pszFile);  // Load XML from a file
      IV_STATUS CopyToBuffer(char*& pText);   // Save XML to a character buffer

   private:
   
      Log*              ivLog;            // Local pointer to output log
      int               refNo;            // TeleFlow reference number ("XML handle")
};


/* -------------------------------------------------------------------------
   Class:       XMLFetchList

   Purpose:     Defines the top level (Head) of linked list of TeleFlow
                'XML Fetch' objects, and provides access to 'XML Fetch'
                session-level methods.

   Modified:    May 16/02 - TF: Created

   Notes:       -  There is a list of XML Fetch objects for every XML
                   Document object currently open (with a TeleFlow handle)
                -  See definition of 'XML' object for more info.
   ------------------------------------------------------------------------- */
class XMLLISTAPI XMLFetchList : public Head
{
   public:

      XMLFetchList(Log* outLog);
      virtual ~XMLFetchList(void);

      XMLFetch* FindFetch(const char* pszReqCtxt);  // Find an open fetch object

   private:

      Log*          ivLog;                // Local pointer to output log
};


/* -------------------------------------------------------------------------
   Class:       XMLFetch

   Purpose:     Defines a single TeleFlow 'XML Fetch' object in the linked
                list (XMLFetchList), and provides access to 'XML Fetch'
                transaction-level functions.

   Modified:    May 16/02 - TF: Created
                Jul.22/02 - TF: Added support for reverse fetching

   Notes:       -  Type "_di_IXMLNode" is a Borland XML Node class
   ------------------------------------------------------------------------- */
class XMLLISTAPI XMLFetch : public Link
{
   friend class XMLFetchList;

   public:

      enum SCOPE { DEFAULT, LOCAL, GLOBAL };

      XMLFetch(Log* outLog, const char* pszNodes);
      virtual ~XMLFetch(void);

      IV_STATUS FindFirst(_di_IXMLDocument xmlDoc);  // Find a node for the first time
      IV_STATUS FindNext(void);                      // Continue a previous search, forward
      IV_STATUS FindPrev(void);                      // Continue a previous search, backward
      IV_STATUS CaptureData(PIVRINFO pInfo,          // Copy data in a node to TeleFlow variables
                            const char* pszVarName,
                            const char* pszAttrText,
                            XMLFetch::SCOPE varScope);

   private:

      Log*              ivLog;            // Local pointer to output log
      char*             pszContext;       // Search context (defines the path to the desired node)
      char*             pszCurToken;      // Token currently being searched for
      _di_IXMLNode      xCurNode;         // Current node object (likely the most recent one fetched)
      int               iCurNodePos;      // Current node position under the parent node
      _di_IXMLNode      xParentNode;      // Parent node containing the desired node as a child
      int               iTotChildren;     // Total number of child nodes under the parent node
};


#undef XMLLISTAPI

#endif
